@@ -12,10 +12,13 @@
 
 #include <osmocom/abis/e1_input.h>
 
+#include "osmocom/bsc/power_control.h"
 #include "osmocom/bsc/gsm_data.h"
 #include "osmocom/bsc/bts_trx.h"
 #include "osmocom/bsc/bts_sm.h"
 #include "osmocom/bsc/abis_om2000.h"
+#include "osmocom/bsc/paging.h"
+#include "osmocom/bsc/bts_setup_ramp.h"
 
 enum bts_counter_id {
 	BTS_CTR_CHREQ_TOTAL,
@@ -53,10 +56,15 @@ enum bts_counter_id {
 	BTS_CTR_PAGING_EXPIRED,
 	BTS_CTR_PAGING_NO_ACTIVE_PAGING,
 	BTS_CTR_PAGING_MSC_FLUSH,
+	BTS_CTR_PAGING_OVERLOAD,
 	BTS_CTR_CHAN_ACT_TOTAL,
 	BTS_CTR_CHAN_ACT_SDCCH,
 	BTS_CTR_CHAN_ACT_TCH,
 	BTS_CTR_CHAN_ACT_NACK,
+	BTS_CTR_CHAN_TCH_ACTIVE_MILLISECONDS_TOTAL,
+	BTS_CTR_CHAN_SDCCH_ACTIVE_MILLISECONDS_TOTAL,
+	BTS_CTR_CHAN_TCH_FULLY_ESTABLISHED,
+	BTS_CTR_CHAN_SDCCH_FULLY_ESTABLISHED,
 	BTS_CTR_RSL_UNKNOWN,
 	BTS_CTR_RSL_IPA_NACK,
 	BTS_CTR_RSL_DELETE_IND,
@@ -220,6 +228,9 @@ enum {
 	BTS_STAT_TS_BORKEN,
 	BTS_STAT_NUM_TRX_RSL_CONNECTED,
 	BTS_STAT_NUM_TRX_TOTAL,
+	BTS_STAT_PAGING_REQ_QUEUE_LENGTH,
+	BTS_STAT_PAGING_AVAILABLE_SLOTS,
+	BTS_STAT_PAGING_T3113,
 };
 
 extern const struct osmo_stat_item_desc bts_stat_desc[];
@@ -253,6 +264,13 @@ enum bts_attribute {
 	TRX_PHY_VERSION,
 };
 
+enum bts_tch_signalling_policy {
+	BTS_TCH_SIGNALLING_NEVER,
+	BTS_TCH_SIGNALLING_EMERG,
+	BTS_TCH_SIGNALLING_VOICE,
+	BTS_TCH_SIGNALLING_ALWAYS,
+};
+
 struct vty;
 
 struct gsm_bts_model {
@@ -293,9 +311,12 @@ struct gsm_bts_model {
 
 	struct tlv_definition nm_att_tlvdef;
 
-	/* features of a given BTS model set via gsm_bts_model_register() locally */
+	/* features of a given BTS model set via gsm_bts_model_register()
+	 * locally, see doc/bts-features.txt */
 	struct bitvec features;
 	uint8_t _features_data[MAX_BTS_FEATURES/8];
+	/* BTS reports features during OML bring up */
+	bool features_get_reported;
 };
 
 struct gsm_gprs_cell {
@@ -332,9 +353,13 @@ struct gsm_bts {
 	char version[MAX_VERSION_LENGTH];
 	char sub_model[MAX_VERSION_LENGTH];
 
-	/* features of a given BTS set/reported via OML */
+	/* features of a given BTS either hardcoded or set/reported via OML,
+	 * see doc/bts-features.txt */
 	struct bitvec features;
 	uint8_t _features_data[MAX_BTS_FEATURES/8];
+	/* Features have been reported by the BTS or were copied from the BTS
+	 * model */
+	bool features_known;
 
 	/* Connected PCU version (if any) */
 	char pcu_version[MAX_VERSION_LENGTH];
@@ -469,8 +494,10 @@ struct gsm_bts {
 		} pwr_ctrl; /* TS 44.060 Table 12.9.1 */
 	} gprs;
 
-	/* threshold (in percent) when BTS shall send CCCH LOAD IND */
-	int ccch_load_ind_thresh;
+	/* CCCH Load Threshold: threshold (in percent) when BTS shall send CCCH LOAD IND */
+	uint8_t ccch_load_ind_thresh;
+	/* CCCH Load Indication Period: how often (secs) to send CCCH LOAD IND when over CCCH Load Threshold. */
+	uint8_t ccch_load_ind_period;
 
 	/* RACH NM values */
 	int rach_b_thresh;
@@ -493,15 +520,26 @@ struct gsm_bts {
 
 	/* should the channel allocator allocate channels from high TRX to TRX0,
 	 * rather than starting from TRX0 and go upwards? */
-	int chan_alloc_reverse;
+	bool chan_alloc_chan_req_reverse;
+	bool chan_alloc_assignment_reverse;
+	bool chan_alloc_handover_reverse;
+
+	/* Whether to use dynamic allocation mode for assignment */
+	bool chan_alloc_assignment_dynamic;
+	/* Parameters used for dynamic mode of allocation */
+	struct {
+		bool sort_by_trx_power;
+		uint8_t ul_rxlev_thresh;
+		uint8_t ul_rxlev_avg_num;
+		uint8_t c0_chan_load_thresh;
+	} chan_alloc_dyn_params;
 
 	/* When true, interference measurements from the BTS are used in the channel allocator to favor lchans with less
 	 * interference reported in RSL Resource Indication. */
 	bool chan_alloc_avoid_interf;
 
-	/* When true (default), TCH can be allocated to serve
-	 * non-voicecall-related signalling services when SDCCHs are exhausted */
-	bool chan_alloc_allow_tch_for_signalling;
+	/* If SDCCHs are exhausted, when can we use TCH for signalling purposes. */
+	enum bts_tch_signalling_policy chan_alloc_tch_signalling_policy;
 
 	enum neigh_list_manual_mode neigh_list_manual_mode;
 	/* parameters from which we build SYSTEM INFORMATION */
@@ -545,6 +583,10 @@ struct gsm_bts {
 	/* exclude the BTS from the global RF Lock handling */
 	int excl_from_rf_lock;
 
+	/* MGW specificities for this BTS: */
+	int mgw_pool_target; /* Pin to specific MGW. -1 = wildcard */
+	bool mgw_pool_target_strict; /* Only allow pinned MGW */
+
 	/* supported codecs beside FR */
 	struct bts_codec_conf codec;
 
@@ -555,6 +597,9 @@ struct gsm_bts {
 	struct amr_multirate_conf mr_full;
 	struct amr_multirate_conf mr_half;
 
+	/* osmux config: */
+	enum osmux_usage use_osmux;
+
 	/* PCU socket state */
 	char *pcu_sock_path;
 	struct pcu_sock_state *pcu_state;
@@ -585,7 +630,7 @@ struct gsm_bts {
 	struct osmo_timer_list cbch_timer;
 	struct bts_smscb_chan_state cbch_basic;
 	struct bts_smscb_chan_state cbch_extended;
-	struct osmo_timer_list etws_timer;	/* when to stop ETWS PN */
+	struct bts_etws_state etws;
 
 	struct llist_head oml_fail_rep;
 	struct llist_head chan_rqd_queue;
@@ -622,10 +667,10 @@ struct gsm_bts {
 	/* At what point in the channel allocation sequence to dispatch the Immediate Assignment (Abis optimization) */
 	enum imm_ass_time imm_ass_time;
 
-	struct osmo_time_cc all_allocated_sdcch;
-	struct osmo_time_cc all_allocated_static_sdcch;
-	struct osmo_time_cc all_allocated_tch;
-	struct osmo_time_cc all_allocated_static_tch;
+	struct chan_counts chan_counts;
+	struct all_allocated all_allocated;
+
+	struct bts_setup_ramp bts_setup_ramp;
 };
 
 #define GSM_BTS_SI2Q(bts, i)   (struct gsm48_system_information_type_2quater *)((bts)->si_buf[SYSINFO_TYPE_2quater][i])
@@ -717,6 +762,7 @@ static inline const struct osmo_location_area_id *bts_lai(struct gsm_bts *bts)
 }
 
 struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm, uint8_t bts_num);
+int gsm_bts_check_cfg(struct gsm_bts *bts);
 
 char *gsm_bts_name(const struct gsm_bts *bts);
 
@@ -731,6 +777,7 @@ int gsm_bts_local_neighbor_del(struct gsm_bts *bts, const struct gsm_bts *neighb
 /* return the gsm_lchan for the CBCH (if it exists at all) */
 struct gsm_lchan *gsm_bts_get_cbch(struct gsm_bts *bts);
 
+int gsm_set_bts_model(struct gsm_bts *bts, struct gsm_bts_model *model);
 int gsm_set_bts_type(struct gsm_bts *bts, enum gsm_bts_type type);
 
 struct gsm_bts_trx *gsm_bts_trx_num(const struct gsm_bts *bts, int num);
@@ -742,6 +789,9 @@ void bts_store_uptime(struct gsm_bts *bts);
 
 unsigned long long bts_uptime(const struct gsm_bts *bts);
 
+#define BTS_STORE_LCHAN_DURATIONS_INTERVAL 1 /* in seconds */
+void bts_store_lchan_durations(struct gsm_bts *bts);
+
 char *get_model_oml_status(const struct gsm_bts *bts);
 /* reset the state of all MO in the BTS */
 void gsm_bts_mo_reset(struct gsm_bts *bts);
@@ -779,3 +829,5 @@ const char *btsatttr2str(enum bts_attribute v);
 
 enum gsm_bts_type_variant str2btsvariant(const char *arg);
 const char *btsvariant2str(enum gsm_bts_type_variant v);
+
+bool gsm_bts_check_ny1(const struct gsm_bts *bts);
