@@ -29,13 +29,28 @@
 #include <osmocom/gsm/rxlev_stat.h>
 #include <osmocom/gsm/protocol/gsm_08_58.h>
 #include <osmocom/gsm/protocol/gsm_12_21.h>
+#include <osmocom/gsm/protocol/gsm_03_41.h>
 #include <osmocom/abis/e1_input.h>
+#include <osmocom/bsc/bts_setup_ramp.h>
 #include <osmocom/bsc/meas_rep.h>
 #include <osmocom/bsc/acc.h>
 #include <osmocom/bsc/osmux.h>
+#include <osmocom/bsc/chan_counts.h>
+#include <osmocom/bsc/lchan.h>
 
 #define GSM_T3122_DEFAULT 10
 
+#define GSM_T3105_DEFAULT 100UL
+#define GSM_T3124_SDCCH 675UL
+#define GSM_T3124_OTHER_CH 320UL
+#define GSM_T3124_MAX GSM_T3124_SDCCH
+
+/* Some guess for delta (see comment below) */
+#define GSM_NY1_REQ_DELTA 1000UL
+/* Requirements: We want Ny1 to be as low as possible, while respecting T3105 * Ny1 > T3124 + delta
+ * with delta = time between expiration of T3124 and receiving HANDOVER FAILURE by the serving BSC. */
+#define GSM_NY1_DEFAULT ((unsigned long)((GSM_T3124_MAX + GSM_NY1_REQ_DELTA)/GSM_T3105_DEFAULT + 1))
+
 struct mgcp_client_conf;
 struct mgcp_client;
 struct gsm0808_cell_id;
@@ -56,6 +71,26 @@ struct smlc_config;
 
 #define OBSC_LINKID_CB(__msgb)	(__msgb)->cb[3]
 
+/* Data Link Connection Identifier (DLCI) is defined in 3GPP TS 48.006, section 9.3.2.
+ * .... .SSS - SAPI value used on the radio link;
+ * CC.. .... - control channel identification:
+ *   00.. .... - indicates that the control channel is not further specified,
+ *   10.. .... - represents the FACCH or the SDCCH,
+ *   11.. .... - represents the SACCH,
+ *   other values are reserved. */
+#define RSL_LINK_ID2DLCI(link_id) \
+	((link_id & 0x40 ? 0xc0 : 0x80) | (link_id & 0x07))
+
+/* RSL Link Identifier is defined in 3GPP TS 3GPP TS 48.058, section 9.3.2.
+ * .... .SSS - SAPI value used on the radio link;
+ * ...P P... - priority for SAPI0 messages;
+ * CC.. .... - control channel identification:
+ *   00.. .... - main signalling channel (FACCH or SDCCH),
+ *   01.. .... - SACCH,
+ *   other values are reserved. */
+#define DLCI2RSL_LINK_ID(dlci) \
+	((dlci & 0xc0) == 0xc0 ? 0x40 : 0x00) | (dlci & 0x07)
+
 /* 3-bit long values */
 #define EARFCN_PRIO_INVALID 8
 #define EARFCN_MEAS_BW_INVALID 8
@@ -69,22 +104,6 @@ typedef int gsm_cbfn(unsigned int hooknum,
 		     struct msgb *msg,
 		     void *data, void *param);
 
-/* Maximum number of neighbor cells whose average we track */
-#define MAX_NEIGH_MEAS		10
-/* Maximum size of the averaging window for neighbor cells */
-#define MAX_WIN_NEIGH_AVG	10
-/* Maximum number of report history we store */
-#define MAX_MEAS_REP		10
-
-/* processed neighbor measurements for one cell */
-struct neigh_meas_proc {
-	uint16_t arfcn;
-	uint8_t bsic;
-	uint8_t rxlev[MAX_WIN_NEIGH_AVG];
-	unsigned int rxlev_cnt;
-	uint8_t last_seen_nr;
-};
-
 struct gsm_classmark {
 	bool classmark1_set;
 	struct gsm48_classmark1 classmark1;
@@ -100,34 +119,6 @@ enum subscr_sccp_state {
 	SUBSCR_SCCP_ST_CONNECTED
 };
 
-enum channel_rate {
-	CH_RATE_SDCCH,
-	CH_RATE_HALF,
-	CH_RATE_FULL,
-};
-
-enum channel_rate chan_t_to_chan_rate(enum gsm_chan_t chan_t);
-
-enum lchan_csd_mode {
-	LCHAN_CSD_M_NT,
-	LCHAN_CSD_M_T_1200_75,
-	LCHAN_CSD_M_T_600,
-	LCHAN_CSD_M_T_1200,
-	LCHAN_CSD_M_T_2400,
-	LCHAN_CSD_M_T_9600,
-	LCHAN_CSD_M_T_14400,
-	LCHAN_CSD_M_T_29000,
-	LCHAN_CSD_M_T_32000,
-};
-
-struct channel_mode_and_rate {
-	enum gsm48_chan_mode chan_mode;
-	enum channel_rate chan_rate;
-	uint16_t s15_s0;
-	/* only used for GSM48_CMODE_DATA_* */
-	enum lchan_csd_mode csd_mode;
-};
-
 enum assign_for {
 	ASSIGN_FOR_NONE,
 	ASSIGN_FOR_BSSMAP_REQ,
@@ -139,14 +130,6 @@ extern const struct value_string assign_for_names[];
 static inline const char *assign_for_name(enum assign_for assign_for)
 { return get_value_string(assign_for_names, assign_for); }
 
-/* If .present is false, use the default value defined elsewhere. If true, use .val below.
- * (A practical benefit of this is that the default initialization sets .present to false, so that even if a .val == 0
- * is a valid value, a struct containing this as member does not need to explicitly set .val = INVALID_VAL_CONSTANT.) */
-struct optional_val {
-	bool present;
-	int val;
-};
-
 /* Information retrieved during an Assignment Request from the MSC. This is storage of the Assignment instructions
  * parsed from the Assignment Request message, to pass on until the gscon and assignment FSMs have decided whether an
  * Assignment is actually going to be carried out. Should remain unchanged after initial decoding. */
@@ -250,6 +233,8 @@ struct handover_in_req {
 	struct gsm0808_channel_type ct;
 	struct gsm0808_speech_codec_list scl;
 	struct gsm0808_encrypt_info ei;
+	/* The same information as in 'ei' but as the handy bitmask as on the wire. */
+	uint8_t ei_as_bitmask;
 	bool kc128_present;
 	uint8_t kc128[16];
 	struct gsm_classmark classmark;
@@ -404,6 +389,8 @@ struct gsm_subscriber_connection {
 		bool last_eutran_plmn_valid; /* Is information stored in field below available? */
 		struct osmo_plmn_id last_eutran_plmn;
 	} fast_return;
+
+	enum gsm0808_cause clear_cause;
 };
 
 
@@ -418,21 +405,6 @@ struct osmo_bsc_data;
 
 struct osmo_bsc_sccp_con;
 
-/* Channel Request reason */
-enum gsm_chreq_reason_t {
-	GSM_CHREQ_REASON_EMERG,
-	GSM_CHREQ_REASON_PAG,
-	GSM_CHREQ_REASON_CALL,
-	GSM_CHREQ_REASON_LOCATION_UPD,
-	GSM_CHREQ_REASON_OTHER,
-	GSM_CHREQ_REASON_PDCH,
-};
-
-static inline bool gsm_chreq_reason_is_voicecall(enum gsm_chreq_reason_t reason)
-{
-	return reason == GSM_CHREQ_REASON_EMERG || reason == GSM_CHREQ_REASON_CALL;
-}
-
 /* lchans 0..3 are SDCCH in combined channel configuration,
    use 4 as magic number for BCCH hack - see osmo-bts-../oml.c:opstart_compl() */
 #define CCCH_LCHAN 4
@@ -487,6 +459,8 @@ struct gsm_abis_mo {
 	bool get_attr_rep_received;
 	bool set_attr_sent;
 	bool set_attr_ack_received;
+	bool rsl_connect_sent;
+	bool rsl_connect_ack_received;
 	bool force_rf_lock;
 };
 
@@ -506,17 +480,30 @@ struct om2k_mo {
 #define A38_XOR_MIN_KEY_LEN	12
 #define A38_XOR_MAX_KEY_LEN	16
 #define A38_COMP128_KEY_LEN	16
-#define RSL_ENC_ALG_A5(x)	(x+1)
+
+/* There are these representations of A5/n:
+ *
+ * - (uint8_t)(1<<n), either as a single bit, or combined as a list of
+ *   permitted algorithms.
+ *   A5/0 == 0x01, A5/3 == 0x08, none = 0
+ *
+ * - n+1, used on the RSL wire.
+ *   A5/0 == 1, A5/3 == 4, none = 0
+ *
+ * - n, used for human interaction and returned by select_best_cipher().
+ *   A5/0 == 0, A5/3 == 3, none <= -1
+ *
+ * These macros convert from n to the other representations:
+ */
+#define ALG_A5_NR_TO_RSL(A5_N) ((A5_N) >= 0 ? (A5_N)+1 : 0)
+#define ALG_A5_NR_TO_BSSAP(A5_N) ALG_A5_NR_TO_RSL(A5_N)
+#define ALG_A5_NR_TO_PERM_ALG_BITS(A5_N) ((A5_N) >= 0 ? 1<<(A5_N) : 0)
+
 /* Up to 16 SI2quater are multiplexed; each fits 3 EARFCNS, so the practical maximum is 3*16.
  * The real maximum that fits in a total of 16 SI2quater rest octets also depends on the bits left by other SI2quater
  * rest octets elements, so to really fit 48 EARFCNs most other SI2quater elements need to be omitted. */
 #define MAX_EARFCN_LIST (3*16)
 
-/* is the data link established? who established it? */
-#define LCHAN_SAPI_UNUSED	0
-#define LCHAN_SAPI_MS		1
-#define LCHAN_SAPI_NET		2
-
 /* BTS ONLY */
 #define MAX_NUM_UL_MEAS	104
 #define LC_UL_M_F_L1_VALID	(1 << 0)
@@ -555,33 +542,6 @@ struct amr_multirate_conf {
 };
 /* /BTS ONLY */
 
-/* State of the SAPIs in the lchan */
-enum lchan_sapi_state {
-	LCHAN_SAPI_S_NONE,
-	LCHAN_SAPI_S_REQ,
-	LCHAN_SAPI_S_ASSIGNED,
-	LCHAN_SAPI_S_REL,
-	LCHAN_SAPI_S_ERROR,
-};
-
-#define MAX_A5_KEY_LEN	(128/8)
-
-struct gsm_encr {
-	uint8_t alg_id;
-	uint8_t key_len;
-	uint8_t key[MAX_A5_KEY_LEN];
-	bool kc128_present;
-	uint8_t kc128[16];
-};
-
-#define LOGPLCHAN(lchan, ss, level, fmt, args...) \
-	LOGP(ss, level, "%s (ss=%d,%s) (%s) " fmt, \
-	     lchan ? gsm_ts_and_pchan_name(lchan->ts) : "-", \
-	     lchan ? lchan->nr : 0, \
-	     lchan ? gsm_lchant_name(lchan->type) : "-", \
-	     bsc_subscr_name(lchan && lchan->conn ? lchan->conn->bsub : NULL), \
-	     ## args)
-
 /* Iterate at most N lchans of the given timeslot.
  * usage:
  * struct gsm_lchan *lchan;
@@ -597,257 +557,9 @@ struct gsm_encr {
 	     && ((lchan - (ts)->lchan) < (N)); \
 	     lchan++)
 
-enum lchan_activate_for {
-	ACTIVATE_FOR_NONE,
-	ACTIVATE_FOR_MS_CHANNEL_REQUEST,
-	ACTIVATE_FOR_ASSIGNMENT,
-	ACTIVATE_FOR_HANDOVER,
-	ACTIVATE_FOR_VTY,
-	ACTIVATE_FOR_MODE_MODIFY_RTP,
-};
-
-extern const struct value_string lchan_activate_mode_names[];
-static inline const char *lchan_activate_mode_name(enum lchan_activate_for activ_for)
-{ return get_value_string(lchan_activate_mode_names, activ_for); }
-
-enum imm_ass_time {
-	IMM_ASS_TIME_POST_CHAN_ACK = 0,
-	IMM_ASS_TIME_PRE_CHAN_ACK,
-	IMM_ASS_TIME_PRE_TS_ACK,
-};
-
-struct lchan_activate_info {
-	enum lchan_activate_for activ_for;
-	/* If activ_for == ACTIVATE_FOR_MS_CHANNEL_REQUEST, the original CHREQ reason. */
-	enum gsm_chreq_reason_t chreq_reason;
-	struct gsm_subscriber_connection *for_conn;
-	struct channel_mode_and_rate ch_mode_rate;
-	struct gsm_encr encr;
-	bool requires_voice_stream;
-	bool wait_before_switching_rtp; /*< true = requires LCHAN_EV_READY_TO_SWITCH_RTP */
-	uint16_t msc_assigned_cic;
-	/* During intra-BSC handover, we keep the MGW endpoint intact and just re-route to the new lchan. This
-	 * activate_info is for the new lchan, the re_use_mgw_endpoint_from_lchan points at the old lchan. */
-	struct gsm_lchan *re_use_mgw_endpoint_from_lchan;
-	bool ta_known;
-	uint8_t ta;
-
-	/* The TSC Set to use if 'use' is true, otherwise automatically determine the TSC Set value to use. Valid range
-	 * is 1 to 4, as described in 3GPP TS 45.002. */
-	struct optional_val tsc_set;
-	/* The TSC to use if 'use' is true, otherwise automatically determine the TSC value to use. Valid range is 0 to
-	 * 7, as described in 3GPP TS 45.002. */
-	struct optional_val tsc;
-
-	bool vamos;
-
-	/* A copy of bts->imm_ass_time at the time where Channel Activation was requested. A change in the VTY
-	 * configuration has immediate effect on the value, so make sure we don't get mixed up when it gets changed
-	 * while a channel activation is in progress. */
-	enum imm_ass_time imm_ass_time;
-};
-
-enum lchan_modify_for {
-	MODIFY_FOR_NONE,
-	MODIFY_FOR_ASSIGNMENT,
-	MODIFY_FOR_VTY,
-};
-
-extern const struct value_string lchan_modify_for_names[];
-static inline const char *lchan_modify_for_name(enum lchan_modify_for modify_for)
-{ return get_value_string(lchan_modify_for_names, modify_for); }
-
-struct lchan_modify_info {
-	enum lchan_modify_for modify_for;
-	struct channel_mode_and_rate ch_mode_rate;
-	bool requires_voice_stream;
-	uint16_t msc_assigned_cic;
-
-	/* The TSC Set to use if 'use' is true, otherwise automatically determine the TSC Set value to use. Valid range
-	 * is 1 to 4, as described in 3GPP TS 45.002. */
-	struct optional_val tsc_set;
-	/* The TSC to use if 'use' is true, otherwise automatically determine the TSC value to use. Valid range is 0 to
-	 * 7, as described in 3GPP TS 45.002. */
-	struct optional_val tsc;
-
-	bool vamos;
-};
-
 #define INTERF_DBM_UNKNOWN 0
 #define INTERF_BAND_UNKNOWN 0xff
 
-/* Measurement pre-processing state */
-struct gsm_power_ctrl_meas_proc_state {
-	/* Number of measurements processed */
-	unsigned int meas_num;
-	/* Algorithm specific data */
-	union {
-		struct {
-			/* Scaled up 100 times average value */
-			int Avg100;
-		} ewma;
-	};
-};
-
-struct lchan_power_ctrl_state {
-	/* Measurement pre-processing state (for dynamic mode) */
-	struct gsm_power_ctrl_meas_proc_state rxlev_meas_proc;
-	struct gsm_power_ctrl_meas_proc_state rxqual_meas_proc;
-	/* Number of SACCH blocks to skip (for dynamic mode) */
-	int skip_block_num;
-};
-
-struct gsm_lchan {
-	/* The TS that we're part of */
-	struct gsm_bts_trx_ts *ts;
-	/* The logical subslot number in the TS */
-	uint8_t nr;
-	char *name;
-
-	char *last_error;
-
-	struct osmo_fsm_inst *fi;
-	struct osmo_fsm_inst *fi_rtp;
-	struct osmo_mgcpc_ep_ci *mgw_endpoint_ci_bts;
-
-	struct {
-		/* The request as made by the caller, see lchan_activate().
-		 * lchan->activate.info is treated immutable: remains unchanged throughout the Activation.
-		 * The mutable versions are below: some values need automatic adjustments, in which case they are copied
-		 * from immutable lchan->activate.info.* to lchan->activate.*, for example lchan->activate.ch_mode_rate
-		 * is initially a copy of lchan->activate.info.ch_mode_rate, and is possibly adjusted afterwards. */
-		struct lchan_activate_info info;
-
-		struct channel_mode_and_rate ch_mode_rate;
-		struct gsm48_multi_rate_conf mr_conf_filtered;
-		bool activ_ack; /*< true as soon as RSL Chan Activ Ack is received */
-		bool immediate_assignment_sent;
-		/*! This flag ensures that when an lchan activation has succeeded, and we have already
-		 * sent ACKs like Immediate Assignment or BSSMAP Assignment Complete, and if other errors
-		 * occur later, e.g. during release, that we don't send a NACK out of context. */
-		bool concluded;
-		enum gsm0808_cause gsm0808_error_cause;
-		/* Actually used TSC Set. */
-		int tsc_set;
-		/* Actually used TSC. */
-		uint8_t tsc;
-	} activate;
-
-	struct {
-		/* The request as made by the caller, see lchan_mode_modify().
-		 * lchan->modify.info is treated immutable: remains unchanged throughout the Mode Modify.
-		 * The mutable versions are below: some values need automatic adjustments, in which case they are copied
-		 * from immutable lchan->modify.info.* to lchan->modify.*, for example lchan->modify.ch_mode_rate
-		 * is initially a copy of lchan->modify.info.ch_mode_rate, and is possibly adjusted afterwards. */
-		struct lchan_modify_info info;
-
-		struct channel_mode_and_rate ch_mode_rate;
-		struct gsm48_multi_rate_conf mr_conf_filtered;
-		/* Actually used TSC Set. */
-		int tsc_set;
-		/* Actually used TSC. */
-		uint8_t tsc;
-		bool concluded;
-	} modify;
-
-	struct {
-		/* If an event to release the lchan comes in while still waiting for responses, just mark this
-		 * flag, so that the lchan will gracefully release at the next sensible junction. */
-		bool requested;
-		bool do_rr_release;
-		enum gsm48_rr_cause rr_cause;
-		bool last_eutran_plmn_valid;
-		struct osmo_plmn_id last_eutran_plmn;
-
-		/* There is an RSL error cause of value 0, so we need a separate flag. */
-		bool in_error;
-		/* RSL error code, RSL_ERR_* */
-		uint8_t rsl_error_cause;
-
-		/* If a release event is being handled, ignore other ricocheting release events until that
-		 * release handling has concluded. */
-		bool in_release_handler;
-	} release;
-
-	/* The logical channel type */
-	enum gsm_chan_t type;
-	/* Power levels for MS and BTS */
-	uint8_t bs_power_db;
-	uint8_t ms_power;
-	/* Encryption information */
-	struct gsm_encr encr;
-
-	/* Established data link layer services */
-	uint8_t sapis[8];
-
-	struct {
-		uint32_t bound_ip; /*< where the BTS receives RTP */
-		uint16_t bound_port;
-		uint32_t connect_ip; /*< where the BTS sends RTP to (MGW) */
-		uint16_t connect_port;
-		uint16_t conn_id;
-		uint8_t rtp_payload;
-		uint8_t rtp_payload2;
-		uint8_t speech_mode;
-
-		/* info we need to postpone the AoIP
-		 * assignment completed message */
-		struct {
-			uint8_t rr_cause;
-			bool valid;
-		} ass_compl;
-	} abis_ip;
-
-	/* At first, the Timing Advance from the initial Channel Request. Later, the Timing Advance value received from
-	 * the most recent Measurement Report. */
-	uint8_t last_ta;
-
-	/* table of neighbor cell measurements */
-	struct neigh_meas_proc neigh_meas[MAX_NEIGH_MEAS];
-
-	/* cache of last measurement reports on this lchan */
-	struct gsm_meas_rep meas_rep[MAX_MEAS_REP];
-	int meas_rep_idx;
-	int meas_rep_count;
-	uint8_t meas_rep_last_seen_nr;
-
-	/* GSM Random Access data */
-	/* TODO: don't allocate this, rather keep an "is_present" flag */
-	struct gsm48_req_ref *rqd_ref;
-
-	struct gsm_subscriber_connection *conn;
-
-	/* After the Channel Activation ACK or RSL Mode Modify ACK is received, this reflects the actually used
-	 * channel_mode_and_rate. */
-	struct channel_mode_and_rate current_ch_mode_rate;
-	struct gsm48_multi_rate_conf current_mr_conf;
-
-	/* Circuit-Switched TSC Set in use, or -1 if no specific TSC Set was requested. The valid range is 1-4 as
-	 * described in the spec 3GPP TS 45.002. */
-	int tsc_set;
-	/* Training Sequence Code in use. The valid range is 0-7 as described in the spec 3GPP TS 45.002. */
-	uint8_t tsc;
-
-	struct {
-		/* Whether this lchan represents a secondary "shadow" lchan to multiplex a second MS onto a primary
-		 * "normal" lchan */
-		bool is_secondary;
-
-		/* Whether this lchan is activated/modified into a mode that allows VAMOS multiplexing at this moment */
-		bool enabled;
-	} vamos;
-
-	/* dBm value of interference level as reported in the most recent Resource Indication, if any for this lchan. Or
-	 * INTERF_DBM_UNKNOWN if this lchan was not included in the most recent Resource Indication.
-	 * The range is typically -115 to -85 dBm, here stored 1:1 as a signed integer, to ease comparison. */
-	int16_t interf_dbm;
-	/* Actual reported interference band index, or INTERF_BAND_UNKNOWN if this lchan was not included in the most
-	 * recent Resource Indication. */
-	uint8_t interf_band;
-	/* MS power control state */
-	struct lchan_power_ctrl_state ms_power_ctrl;
-};
-
 /* One Timeslot in a TRX */
 struct gsm_bts_trx_ts {
 	struct gsm_bts_trx *trx;
@@ -916,28 +628,8 @@ struct gsm_bts_trx_ts {
 	 * Does not include count of secondary VAMOS lchans. */
 	uint8_t max_primary_lchans;
 	struct gsm_lchan lchan[TS_MAX_LCHAN];
-};
 
-#define GSM_LCHAN_SI(lchan, i) (void *)((lchan)->si.buf[i][0])
-
-/*
- * This keeps track of the paging status of one BTS. It
- * includes a number of pending requests, a back pointer
- * to the gsm_bts, a timer and some more state.
- */
-struct gsm_bts_paging_state {
-	/* pending requests */
-	struct llist_head pending_requests;
-	struct gsm_bts *bts;
-
-	struct osmo_timer_list work_timer;
-	struct osmo_timer_list credit_timer;
-
-	/* free chans needed */
-	int free_chans_need;
-
-	/* load */
-	uint16_t available_slots;
+	struct chan_counts chan_counts;
 };
 
 struct gsm_envabtse {
@@ -1064,6 +756,27 @@ struct bts_smscb_chan_state {
 	uint8_t overflow;
 };
 
+#define ETWS_PRIM_NOTIF_SIZE	56
+#define ETWS_SEC_INFO_SIZE	(ETWS_PRIM_NOTIF_SIZE - sizeof(struct gsm341_etws_message))
+
+/* per-BTS ETWS/PWS (Emergency) state */
+struct bts_etws_state {
+	/* are we actively broadcasting emergency in this cell? */
+	bool active;
+	/* input parameters received from CBC */
+	struct {
+		uint16_t msg_id;
+		uint16_t serial_nr;
+		uint16_t warn_type;
+		uint8_t sec_info[ETWS_SEC_INFO_SIZE];
+	} input;
+	/* encoded ETWS primary notification */
+	uint8_t primary[ETWS_PRIM_NOTIF_SIZE];
+
+	/* timer running for the duration of the ETWS Primary Notification (PN) */
+	struct osmo_timer_list timer;
+};
+
 struct bts_oml_fail_rep {
 	struct llist_head list;
 	time_t time;
@@ -1098,17 +811,9 @@ const char *gsm_pchan_name(enum gsm_phys_chan_config c);
 static inline const char *gsm_pchan_id(enum gsm_phys_chan_config c)
 { return get_value_string(gsm_pchan_ids, c); }
 enum gsm_phys_chan_config gsm_pchan_parse(const char *name);
-const char *gsm_lchant_name(enum gsm_chan_t c);
 const char *gsm_chreq_name(enum gsm_chreq_reason_t c);
 char *gsm_ts_name(const struct gsm_bts_trx_ts *ts);
 char *gsm_ts_and_pchan_name(const struct gsm_bts_trx_ts *ts);
-void lchan_update_name(struct gsm_lchan *lchan);
-
-static inline char *gsm_lchan_name(const struct gsm_lchan *lchan)
-{
-	OSMO_ASSERT(lchan);
-	return lchan->name;
-}
 
 void gsm_abis_mo_reset(struct gsm_abis_mo *mo);
 void gsm_mo_init(struct gsm_abis_mo *mo, struct gsm_bts *bts,
@@ -1140,13 +845,9 @@ uint8_t pchan_subslots(enum gsm_phys_chan_config pchan);
 uint8_t pchan_subslots_vamos(enum gsm_phys_chan_config pchan);
 bool ts_is_tch(struct gsm_bts_trx_ts *ts);
 
-struct gsm_lchan *gsm_lchan_vamos_to_primary(const struct gsm_lchan *lchan_vamos);
-struct gsm_lchan *gsm_lchan_primary_to_vamos(const struct gsm_lchan *lchan_primary);
-
 struct gsm_bts *conn_get_bts(struct gsm_subscriber_connection *conn);
 
 void conn_update_ms_power_class(struct gsm_subscriber_connection *conn, uint8_t power_class);
-void lchan_update_ms_power_ctrl_level(struct gsm_lchan *lchan, int ms_power_dbm);
 
 struct gsm_tz {
 	int override; /* if 0, use system's time zone instead. */
@@ -1155,6 +856,13 @@ struct gsm_tz {
 	int dst; /* daylight savings */
 };
 
+struct all_allocated {
+	struct osmo_time_cc sdcch;
+	struct osmo_time_cc static_sdcch;
+	struct osmo_time_cc tch;
+	struct osmo_time_cc static_tch;
+};
+
 struct gsm_network {
 	struct osmo_plmn_id plmn;
 
@@ -1176,10 +884,10 @@ struct gsm_network {
 	struct llist_head bts_list;
 	struct llist_head bts_rejected;
 
-        /* BTS-based counters when we can't find the actual BTS
-         * e.g. when conn->lchan is NULL */
-        struct rate_ctr_group *bts_unknown_ctrs;
-        struct osmo_stat_item_group *bts_unknown_statg;
+	/* BTS-based counters when we can't find the actual BTS
+	 * e.g. when conn->lchan is NULL */
+	struct rate_ctr_group *bts_unknown_ctrs;
+	struct osmo_stat_item_group *bts_unknown_statg;
 
 	/* see gsm_network_T_defs */
 	struct osmo_tdef *T_defs;
@@ -1234,6 +942,9 @@ struct gsm_network {
 	/* Timer to write each BTS's uptime counter state to the stats system. */
 	struct osmo_timer_list bts_store_uptime_timer;
 
+	/* Timer to write each BTS's set of lchan duration counters' state to the stats system. */
+	struct osmo_timer_list bts_store_lchan_durations_timer;
+
 	struct {
 		/* Single MGCP client configuration under msc node (also required for
 		 * MGCP proxy when sccp-lite is used) */
@@ -1257,15 +968,15 @@ struct gsm_network {
 	/* Don't refuse to start with mutually exclusive codec settings */
 	bool allow_unusable_timeslots;
 
+	struct bts_setup_ramp_net bts_setup_ramp;
+
 	uint8_t nri_bitlen;
 	struct osmo_nri_ranges *null_nri_ranges;
 
 	struct smlc_config *smlc;
 
-	struct osmo_time_cc all_allocated_sdcch;
-	struct osmo_time_cc all_allocated_static_sdcch;
-	struct osmo_time_cc all_allocated_tch;
-	struct osmo_time_cc all_allocated_static_tch;
+	struct chan_counts chan_counts;
+	struct all_allocated all_allocated;
 };
 
 struct gsm_audio_support {
@@ -1301,9 +1012,6 @@ void set_ts_e1link(struct gsm_bts_trx_ts *ts, uint8_t e1_nr,
 /* generic E1 line operations for all ISDN-based BTS. */
 extern struct e1inp_line_ops bts_isdn_e1inp_line_ops;
 
-/* control interface handling */
-int bsc_base_ctrl_cmds_install(void);
-
 bool ts_is_usable(const struct gsm_bts_trx_ts *ts);
 
 int gsm_lchan_type_by_pchan(enum gsm_phys_chan_config pchan);
@@ -1314,98 +1022,6 @@ enum gsm48_rr_cause bsc_gsm48_rr_cause_from_rsl_cause(uint8_t c);
 
 int bsc_sccp_inst_next_conn_id(struct osmo_sccp_instance *sccp);
 
-/* MS/BS Power related measurement averaging algo */
-enum gsm_power_ctrl_meas_avg_algo {
-	GSM_PWR_CTRL_MEAS_AVG_ALGO_NONE			= 0x00,
-	GSM_PWR_CTRL_MEAS_AVG_ALGO_UNWEIGHTED		= 0x01,
-	GSM_PWR_CTRL_MEAS_AVG_ALGO_WEIGHTED		= 0x02,
-	GSM_PWR_CTRL_MEAS_AVG_ALGO_MOD_MEDIAN		= 0x03,
-	/* EWMA is an Osmocom specific algo */
-	GSM_PWR_CTRL_MEAS_AVG_ALGO_OSMO_EWMA		= 0x04,
-};
-
-/* MS/BS Power related measurement parameters */
-struct gsm_power_ctrl_meas_params {
-	/* Are these measurement paremeters to be taken into account by loop? */
-	bool enabled;
-
-	/* Thresholds (see 3GPP TS 45.008, section A.3.2.1) */
-	uint8_t lower_thresh; /* lower (decreasing) direction */
-	uint8_t upper_thresh; /* upper (increasing) direction */
-
-	/* Threshold Comparators for lower (decreasing) direction */
-	uint8_t lower_cmp_p; /* P1 for RxLev, P3 for RxQual */
-	uint8_t lower_cmp_n; /* N1 for RxLev, N3 for RxQual */
-	/* Threshold Comparators for upper (increasing) direction */
-	uint8_t upper_cmp_p; /* P2 for RxLev, P4 for RxQual */
-	uint8_t upper_cmp_n; /* N2 for RxLev, N4 for RxQual */
-
-	/* Hreqave and Hreqt (see 3GPP TS 45.008, Annex A) */
-	uint8_t h_reqave;
-	uint8_t h_reqt;
-
-	/* AVG algorithm and its specific parameters */
-	enum gsm_power_ctrl_meas_avg_algo algo;
-	union {
-		/* Exponentially Weighted Moving Average */
-		struct {
-			/* Smoothing factor: higher the value - less smoothing */
-			uint8_t alpha; /* 1 .. 99 (in %) */
-		} ewma;
-	};
-};
-
-enum gsm_power_ctrl_dir {
-	GSM_PWR_CTRL_DIR_UL, /* MS Power Control */
-	GSM_PWR_CTRL_DIR_DL, /* BS Power Control */
-};
-
-enum gsm_power_ctrl_mode {
-	/* Do not send MS/BS Power Control IEs */
-	GSM_PWR_CTRL_MODE_NONE = 0,
-	/* Send MS/BS Power IE only (with target level) */
-	GSM_PWR_CTRL_MODE_STATIC,
-	/* Send MS/BS Power [Parameters] IEs (dynamic mode) */
-	GSM_PWR_CTRL_MODE_DYN_BTS,
-	/* Do not send MS/BS Power IEs and use BSC Power Loop */
-	GSM_PWR_CTRL_MODE_DYN_BSC,
-
-};
-
-/* MS/BS Power Control Parameters */
-struct gsm_power_ctrl_params {
-	/* Power Control direction: Uplink or Downlink */
-	enum gsm_power_ctrl_dir dir;
-	/* Power Control mode to be used by the BTS */
-	enum gsm_power_ctrl_mode mode;
-
-	/* BS Power reduction value / maximum (in dB) */
-	uint8_t bs_power_val_db; /* for static mode */
-	uint8_t bs_power_max_db; /* for dynamic mode */
-
-	/* Power change step size (dynamic mode only) */
-	uint8_t inc_step_size_db; /* increasing direction */
-	uint8_t red_step_size_db; /* reducing direction */
-
-	/* Minimum interval between power level changes */
-	uint8_t ctrl_interval; /* 1 step is 2 SACCH periods */
-
-	/* Measurement averaging parameters for RxLev & RxQual */
-	struct gsm_power_ctrl_meas_params rxqual_meas;
-	struct gsm_power_ctrl_meas_params rxlev_meas;
-	/* Measurement averaging parameters for C/I: */
-	struct gsm_power_ctrl_meas_params ci_fr_meas;
-	struct gsm_power_ctrl_meas_params ci_hr_meas;
-	struct gsm_power_ctrl_meas_params ci_amr_fr_meas;
-	struct gsm_power_ctrl_meas_params ci_amr_hr_meas;
-	struct gsm_power_ctrl_meas_params ci_sdcch_meas;
-	struct gsm_power_ctrl_meas_params ci_gprs_meas;
-};
-
-extern const struct gsm_power_ctrl_params power_ctrl_params_def;
-void power_ctrl_params_def_reset(struct gsm_power_ctrl_params *params,
-				 enum gsm_power_ctrl_dir dir);
-
 /* Interference Measurement Parameters */
 struct gsm_interf_meas_params {
 	/* Intave: Interference Averaging period (see 3GPP TS 45.008, table A.1) */
@@ -1418,4 +1034,6 @@ extern const struct gsm_interf_meas_params interf_meas_params_def;
 
 enum rsl_cmod_spd chan_mode_to_rsl_cmod_spd(enum gsm48_chan_mode chan_mode);
 
+int select_best_cipher(uint8_t msc_mask, uint8_t bsc_mask);
+
 #endif /* _GSM_DATA_H */
