@@ -6,11 +6,18 @@
 
 /* This macro automatically includes a final \n, if omitted. */
 #define LOG_LCHAN(lchan, level, fmt, args...) do { \
-	if (lchan->fi) \
-		LOGPFSML(lchan->fi, level, "(type=%s) " fmt, gsm_lchant_name(lchan->type), ## args); \
+	if ((lchan)->fi) \
+		LOGPFSML((lchan)->fi, level, "(type=%s) " fmt, gsm_chan_t_name((lchan)->type), ## args); \
 	else \
 		LOGP(DRSL, level, "%s (not initialized) " fmt, gsm_lchan_name(lchan), ## args); \
-	} while(0)
+	} while (0)
+
+#define LCHAN_SET_LAST_ERROR(LCHAN, fmt, args...) do { \
+		if ((LCHAN)->last_error) \
+			talloc_free((LCHAN)->last_error); \
+		(LCHAN)->last_error = talloc_asprintf((LCHAN)->ts->trx, fmt, ##args); \
+		LOG_LCHAN(LCHAN, LOGL_ERROR, "%s\n", (LCHAN)->last_error); \
+	} while (0)
 
 enum lchan_fsm_state {
 	LCHAN_ST_UNUSED,
@@ -49,21 +56,23 @@ enum lchan_fsm_event {
 	LCHAN_EV_REQUEST_MODE_MODIFY,
 };
 
-void lchan_fsm_init();
+void lchan_fsm_init(void);
 
 void lchan_fsm_alloc(struct gsm_lchan *lchan);
 void lchan_release(struct gsm_lchan *lchan, bool do_rr_release,
-		   bool err, enum gsm48_rr_cause cause_rr);
+		   bool err, enum gsm48_rr_cause cause_rr,
+		   const struct osmo_plmn_id *last_eutran_plmn);
 
 void lchan_activate(struct gsm_lchan *lchan, struct lchan_activate_info *info);
 void lchan_ready_to_switch_rtp(struct gsm_lchan *lchan);
+void lchan_mode_modify(struct gsm_lchan *lchan, struct lchan_modify_info *info);
 
 static inline const char *lchan_state_name(struct gsm_lchan *lchan)
 {
 	return lchan->fi ? osmo_fsm_inst_state_name(lchan->fi) : "NULL";
 }
 
-static inline bool lchan_state_is(struct gsm_lchan *lchan, uint32_t state)
+static inline bool lchan_state_is(const struct gsm_lchan *lchan, uint32_t state)
 {
 	return (!lchan->fi && state == LCHAN_ST_UNUSED)
 		|| (lchan->fi && lchan->fi->state == state);
@@ -73,6 +82,5 @@ bool lchan_may_receive_data(struct gsm_lchan *lchan);
 
 void lchan_forget_conn(struct gsm_lchan *lchan);
 
-void lchan_set_last_error(struct gsm_lchan *lchan, const char *fmt, ...);
-
 void lchan_fsm_skip_error(struct gsm_lchan *lchan);
+void lchan_fsm_update_id(struct gsm_lchan *lchan);
