@@ -51,6 +51,7 @@
 #include <osmocom/bsc/bts.h>
 #include <osmocom/bsc/nm_common_fsm.h>
 #include <osmocom/gsm/bts_features.h>
+#include <osmocom/bsc/ipaccess.h>
 
 #define OM_ALLOC_SIZE		1024
 #define OM_HEADROOM_SIZE	128
@@ -209,7 +210,7 @@ static int abis_nm_rcvmsg_sw(struct msgb *mb);
 static int update_admstate(struct gsm_bts *bts, uint8_t obj_class,
 			   struct abis_om_obj_inst *obj_inst, uint8_t adm_state)
 {
-	struct gsm_nm_state *nm_state, new_state;
+	struct gsm_nm_state *nm_state;
 	struct nm_statechg_signal_data nsd;
 
 	memset(&nsd, 0, sizeof(nsd));
@@ -221,18 +222,18 @@ static int update_admstate(struct gsm_bts *bts, uint8_t obj_class,
 	if (!nm_state)
 		return -1;
 
-	new_state = *nm_state;
-	new_state.administrative = adm_state;
-
 	nsd.bts = bts;
 	nsd.obj_class = obj_class;
-	nsd.old_state = nm_state;
-	nsd.new_state = &new_state;
+	nsd.old_state = *nm_state;
+	nsd.new_state = *nm_state;
 	nsd.obj_inst = obj_inst;
-	osmo_signal_dispatch(SS_NM, S_NM_STATECHG_ADM, &nsd);
 
+	nsd.new_state.administrative = adm_state;
+
+	/* Update current state before emitting signal: */
 	nm_state->administrative = adm_state;
 
+	osmo_signal_dispatch(SS_NM, S_NM_STATECHG, &nsd);
 	return 0;
 }
 
@@ -243,72 +244,67 @@ static int abis_nm_rx_statechg_rep(struct msgb *mb)
 	struct e1inp_sign_link *sign_link = mb->dst;
 	struct gsm_bts *bts = sign_link->trx->bts;
 	struct tlv_parsed tp;
-	struct gsm_nm_state *nm_state, new_state;
+	struct gsm_nm_state *nm_state;
+	struct nm_statechg_signal_data nsd;
 
-	memset(&new_state, 0, sizeof(new_state));
+	memset(&nsd, 0, sizeof(nsd));
 
+	nsd.obj = gsm_objclass2obj(bts, foh->obj_class, &foh->obj_inst);
+	if (!nsd.obj) {
+		LOGPFOH(DNM, LOGL_ERROR, foh, "unknown managed object\n");
+		return -EINVAL;
+	}
 	nm_state = gsm_objclass2nmstate(bts, foh->obj_class, &foh->obj_inst);
 	if (!nm_state) {
 		LOGPFOH(DNM, LOGL_ERROR, foh, "unknown managed object\n");
 		return -EINVAL;
 	}
 
-	new_state = *nm_state;
+	nsd.obj_class = foh->obj_class;
+	nsd.old_state = *nm_state;
+	nsd.new_state = *nm_state;
+	nsd.obj_inst = &foh->obj_inst;
+	nsd.bts = bts;
+
+	if (abis_nm_tlv_parse(&tp, bts, foh->data, oh->length - sizeof(*foh)) < 0) {
+		LOGPFOH(DNM, LOGL_ERROR, foh, "%s(): tlv_parse failed\n", __func__);
+		return -EINVAL;
+	}
 
 	DEBUGPFOH(DNM, foh, "STATE CHG: ");
-	abis_nm_tlv_parse(&tp, bts, foh->data, oh->length-sizeof(*foh));
 	if (TLVP_PRESENT(&tp, NM_ATT_OPER_STATE)) {
-		new_state.operational = *TLVP_VAL(&tp, NM_ATT_OPER_STATE);
+		nsd.new_state.operational = *TLVP_VAL(&tp, NM_ATT_OPER_STATE);
 		DEBUGPC(DNM, "OP_STATE=%s ",
-			abis_nm_opstate_name(new_state.operational));
+			abis_nm_opstate_name(nsd.new_state.operational));
 	}
 	if (TLVP_PRESENT(&tp, NM_ATT_AVAIL_STATUS)) {
 		if (TLVP_LEN(&tp, NM_ATT_AVAIL_STATUS) == 0)
-			new_state.availability = 0xff;
+			nsd.new_state.availability = NM_AVSTATE_OK;
 		else
-			new_state.availability = *TLVP_VAL(&tp, NM_ATT_AVAIL_STATUS);
+			nsd.new_state.availability = *TLVP_VAL(&tp, NM_ATT_AVAIL_STATUS);
 		DEBUGPC(DNM, "AVAIL=%s(%02x) ",
-			abis_nm_avail_name(new_state.availability),
-			new_state.availability);
+			abis_nm_avail_name(nsd.new_state.availability),
+			nsd.new_state.availability);
 	} else
-		new_state.availability = 0xff;
+		nsd.new_state.availability = NM_AVSTATE_OK;
 	if (TLVP_PRESENT(&tp, NM_ATT_ADM_STATE)) {
-		new_state.administrative = *TLVP_VAL(&tp, NM_ATT_ADM_STATE);
+		nsd.new_state.administrative = *TLVP_VAL(&tp, NM_ATT_ADM_STATE);
 		DEBUGPC(DNM, "ADM=%2s ",
 			get_value_string(abis_nm_adm_state_names,
-					 new_state.administrative));
+					 nsd.new_state.administrative));
 	}
 
-	if ((new_state.administrative != 0 && nm_state->administrative == 0) ||
-	    new_state.operational != nm_state->operational ||
-	    new_state.availability != nm_state->availability) {
+	if ((nsd.new_state.administrative != 0 && nsd.old_state.administrative == 0) ||
+	    nsd.new_state.operational != nsd.old_state.operational ||
+	    nsd.new_state.availability != nsd.old_state.availability) {
 		DEBUGPC(DNM, "\n");
-		/* Update the operational state of a given object in our in-memory data
+		/* Update the state of a given object in our in-memory data
  		* structures and send an event to the higher layer */
-		struct nm_statechg_signal_data nsd;
-		nsd.obj = gsm_objclass2obj(bts, foh->obj_class, &foh->obj_inst);
-		nsd.obj_class = foh->obj_class;
-		nsd.old_state = nm_state;
-		nsd.new_state = &new_state;
-		nsd.obj_inst = &foh->obj_inst;
-		nsd.bts = bts;
-		osmo_signal_dispatch(SS_NM, S_NM_STATECHG_OPER, &nsd);
-		nm_state->operational = new_state.operational;
-		nm_state->availability = new_state.availability;
-		if (nm_state->administrative == 0)
-			nm_state->administrative = new_state.administrative;
+		*nm_state = nsd.new_state;
+		osmo_signal_dispatch(SS_NM, S_NM_STATECHG, &nsd);
 	} else {
 		DEBUGPC(DNM, "(No State change detected)\n");
 	}
-#if 0
-	if (op_state == 1) {
-		/* try to enable objects that are disabled */
-		abis_nm_opstart(bts, foh->obj_class,
-				foh->obj_inst.bts_nr,
-				foh->obj_inst.trx_nr,
-				foh->obj_inst.ts_nr);
-	}
-#endif
 	return 0;
 }
 
@@ -374,8 +370,10 @@ struct nm_fail_rep_signal_data *abis_nm_fail_evt_rep_parse(struct msgb *mb, stru
 	sd = talloc_zero(tall_bsc_ctx, struct nm_fail_rep_signal_data);
 	OSMO_ASSERT(sd);
 
-	if (abis_nm_tlv_parse(&sd->tp, bts, foh->data, oh->length-sizeof(*foh)) < 0)
+	if (abis_nm_tlv_parse(&sd->tp, bts, foh->data, oh->length - sizeof(*foh)) < 0) {
+		LOGPFOH(DNM, LOGL_ERROR, foh, "%s(): tlv_parse failed\n", __func__);
 		goto fail;
+	}
 
 	if (TLVP_PRESENT(&sd->tp, NM_ATT_ADD_TEXT)) {
 		const uint8_t *val = TLVP_VAL(&sd->tp, NM_ATT_ADD_TEXT);
@@ -593,21 +591,32 @@ static int parse_attr_resp_info_attr(struct gsm_bts *bts, const struct gsm_bts_t
 		}
 
 		memcpy(bts->_features_data, TLVP_VAL(tp, NM_ATT_MANUF_ID), len);
+		bts->features_known = true;
 
-		/* Check each BTS feature in the reported vector */
+		/* Log each BTS feature in the reported vector */
 		for (i = 0; i < len * 8; i++) {
-			bool Frep = osmo_bts_has_feature(&bts->features, i);
-			if (i >= _NUM_BTS_FEAT && Frep) {
-				LOGPMO(&bts->mo, DNM, LOGL_NOTICE, "Get Attributes Response: "
-				       "unknown feature 0x%02x. Consider upgrading osmo-bsc.\n", i);
+			if (!osmo_bts_has_feature(&bts->features, i))
 				continue;
-			}
 
-			bool Fexp = osmo_bts_has_feature(&bts->model->features, i);
-			if (!Frep && Fexp) {
-				LOGPMO(&bts->mo, DNM, LOGL_NOTICE, "Get Attributes Response: "
-				       "reported feature '%s' is not supported, while we thought it is.\n",
-				       osmo_bts_features_name(i));
+			if (i >= _NUM_BTS_FEAT)
+				LOGPMO(&bts->mo, DNM, LOGL_NOTICE, "Get Attributes Response: unknown feature 0x%02x is"
+				       " supported\n", i);
+			else
+				LOGPMO(&bts->mo, DNM, LOGL_NOTICE, "Get Attributes Response: feature '%s' is"
+				       " supported\n", osmo_bts_features_name(i));
+		}
+
+		/* Add features from the BTS model: nanobts may support more
+		 * features than it reports, since we extend the enum of
+		 * features for osmo-bts. */
+		if (bts->type == GSM_BTS_TYPE_NANOBTS) {
+			for (i = 0; i < _NUM_BTS_FEAT; i++) {
+				if (osmo_bts_has_feature(&bts->model->features, i) /* intentional check against bts model */
+				    && !osmo_bts_has_feature(&bts->features, i)) {
+					LOGPMO(&bts->mo, DNM, LOGL_NOTICE, "Get Attributes Response: feature '%s' is"
+					       " assumed to be supported\n", osmo_bts_features_name(i));
+					osmo_bts_set_feature(&bts->features, i);
+				}
 			}
 		}
 	}
@@ -677,7 +686,11 @@ static int parse_attr_resp_info(struct gsm_bts *bts, const struct gsm_bts_trx *t
 	/* After parsing unreported attribute id list inside Response info,
 	   there's a list of reported attribute ids and their values, in a TLV
 	   list form. */
-	abis_nm_tlv_parse(tp, bts, data, data_len);
+	if (abis_nm_tlv_parse(tp, bts, data, data_len) < 0) {
+		LOGPFOH(DNM, LOGL_ERROR, foh, "%s(): tlv_parse failed\n", __func__);
+		return -EINVAL;
+	}
+
 	return parse_attr_resp_info_attr(bts, trx, foh, tp);
 }
 
@@ -697,7 +710,10 @@ static int abis_nm_rx_get_attr_resp(struct msgb *mb)
 
 	DEBUGPFOH(DNM, foh, "Get Attributes Response\n");
 
-	abis_nm_tlv_parse(&tp, bts, foh->data, oh->length-sizeof(*foh));
+	if (abis_nm_tlv_parse(&tp, bts, foh->data, oh->length - sizeof(*foh)) < 0) {
+		LOGPFOH(DNM, LOGL_ERROR, foh, "%s(): tlv_parse failed\n", __func__);
+		return -EINVAL;
+	}
 
 	/* nanoBTS doesn't send Get Attribute Response Info, uses its own format */
 	if (bts->type != GSM_BTS_TYPE_NANOBTS)
@@ -705,6 +721,12 @@ static int abis_nm_rx_get_attr_resp(struct msgb *mb)
 	else
 		rc = parse_attr_resp_info_attr(bts, trx, foh, &tp);
 
+	if (gsm_bts_check_cfg(bts) != 0) {
+		LOGP(DLINP, LOGL_ERROR, "(bts=%u) BTS config invalid, dropping BTS!\n", bts->nr);
+		ipaccess_drop_oml_deferred(bts);
+		return -EINVAL;
+	}
+
 	osmo_signal_dispatch(SS_NM, S_NM_GET_ATTR_REP, mb);
 
 	return rc;
@@ -733,7 +755,11 @@ static int abis_nm_rx_sw_act_req(struct msgb *mb)
 		return ret;
 	}
 
-	abis_nm_tlv_parse(&tp, sign_link->trx->bts, foh->data, oh->length-sizeof(*foh));
+	if (abis_nm_tlv_parse(&tp, sign_link->trx->bts, foh->data, oh->length - sizeof(*foh)) < 0) {
+		LOGPFOH(DNM, LOGL_ERROR, foh, "%s(): tlv_parse failed\n", __func__);
+		return -EINVAL;
+	}
+
 	sw_config = TLVP_VAL(&tp, NM_ATT_SW_CONFIG);
 	sw_config_len = TLVP_LEN(&tp, NM_ATT_SW_CONFIG);
 	if (!TLVP_PRESENT(&tp, NM_ATT_SW_CONFIG)) {
@@ -770,7 +796,11 @@ static int abis_nm_rx_chg_adm_state_ack(struct msgb *mb)
 	struct tlv_parsed tp;
 	uint8_t adm_state;
 
-	abis_nm_tlv_parse(&tp, sign_link->trx->bts, foh->data, oh->length-sizeof(*foh));
+	if (abis_nm_tlv_parse(&tp, sign_link->trx->bts, foh->data, oh->length - sizeof(*foh)) < 0) {
+		LOGPFOH(DNM, LOGL_ERROR, foh, "%s(): tlv_parse failed\n", __func__);
+		return -EINVAL;
+	}
+
 	if (!TLVP_PRESENT(&tp, NM_ATT_ADM_STATE))
 		return -EINVAL;
 
@@ -789,8 +819,12 @@ static int abis_nm_rx_lmt_event(struct msgb *mb)
 	struct e1inp_sign_link *sign_link = mb->dst;
 	struct tlv_parsed tp;
 
+	if (abis_nm_tlv_parse(&tp, sign_link->trx->bts, foh->data, oh->length - sizeof(*foh)) < 0) {
+		LOGPFOH(DNM, LOGL_ERROR, foh, "%s(): tlv_parse failed\n", __func__);
+		return -EINVAL;
+	}
+
 	DEBUGPFOH(DNM, foh, "LMT Event ");
-	abis_nm_tlv_parse(&tp, sign_link->trx->bts, foh->data, oh->length-sizeof(*foh));
 	if (TLVP_PRESENT(&tp, NM_ATT_BS11_LMT_LOGON_SESSION) &&
 	    TLVP_LEN(&tp, NM_ATT_BS11_LMT_LOGON_SESSION) >= 1) {
 		uint8_t onoff = *TLVP_VAL(&tp, NM_ATT_BS11_LMT_LOGON_SESSION);
@@ -942,9 +976,12 @@ static int abis_nm_rcvmsg_fom(struct msgb *mb)
 		struct nm_nack_signal_data nack_data;
 		struct tlv_parsed tp;
 
-		LOGPFOH(DNM, LOGL_NOTICE, foh, "%s NACK ", abis_nm_nack_name(mt));
+		if (abis_nm_tlv_parse(&tp, bts, foh->data, oh->length - sizeof(*foh)) < 0) {
+			LOGPFOH(DNM, LOGL_ERROR, foh, "%s(): tlv_parse failed\n", __func__);
+			return -EINVAL;
+		}
 
-		abis_nm_tlv_parse(&tp, bts, foh->data, oh->length-sizeof(*foh));
+		LOGPFOH(DNM, LOGL_NOTICE, foh, "%s NACK ", abis_nm_nack_name(mt));
 		if (TLVP_PRESENT(&tp, NM_ATT_NACK_CAUSES))
 			LOGPC(DNM, LOGL_NOTICE, "CAUSE=%s\n",
 				abis_nm_nack_cause_name(*TLVP_VAL(&tp, NM_ATT_NACK_CAUSES)));
@@ -1342,13 +1379,6 @@ static int sw_activate(struct abis_nm_sw *sw)
 	return abis_nm_sendmsg(sw->bts, msg);
 }
 
-struct sdp_firmware {
-	char magic[4];
-	char more_magic[4];
-	unsigned int header_length;
-	unsigned int file_length;
-} __attribute__ ((packed));
-
 static int parse_sdp_header(struct abis_nm_sw *sw)
 {
 	const struct gsm_abis_mo *mo = &sw->bts->mo;
@@ -1363,18 +1393,20 @@ static int parse_sdp_header(struct abis_nm_sw *sw)
 	}
 
 	if (strncmp(firmware_header.magic, " SDP", 4) != 0) {
-		LOGPMO(mo, DNM, LOGL_ERROR, "The magic number1 is wrong.\n");
+		LOGPMO(mo, DNM, LOGL_ERROR, "The magic number is wrong.\n");
 		return -1;
 	}
 
 	if (firmware_header.more_magic[0] != 0x10 ||
-	    firmware_header.more_magic[1] != 0x02 ||
-	    firmware_header.more_magic[2] != 0x00 ||
-	    firmware_header.more_magic[3] != 0x00) {
+	    firmware_header.more_magic[1] != 0x02) {
 		LOGPMO(mo, DNM, LOGL_ERROR, "The more magic number is wrong.\n");
 		return -1;
 	}
 
+	if (firmware_header.more_more_magic != 0x0000) {
+		LOGPMO(mo, DNM, LOGL_ERROR, "The more more magic number is wrong.\n");
+		return -1;
+	}
 
 	if (fstat(sw->fd, &stat) == -1) {
 		LOGPMO(mo, DNM, LOGL_ERROR, "Could not stat the file.\n");
@@ -2833,6 +2865,7 @@ static int abis_nm_rx_ipacc(struct msgb *msg)
 	struct tlv_parsed tp;
 	struct ipacc_ack_signal_data signal;
 	struct e1inp_sign_link *sign_link = msg->dst;
+	struct gsm_bts *bts = sign_link->trx->bts;
 	struct gsm_bts_trx *trx;
 
 	foh = (struct abis_om_fom_hdr *) (oh->data + 1 + idstrlen);
@@ -2842,11 +2875,14 @@ static int abis_nm_rx_ipacc(struct msgb *msg)
 		return -EINVAL;
 	}
 
-	abis_nm_tlv_parse(&tp, sign_link->trx->bts, foh->data, oh->length-sizeof(*foh));
+	if (abis_nm_tlv_parse(&tp, bts, foh->data, oh->length - sizeof(*foh)) < 0) {
+		LOGPFOH(DNM, LOGL_ERROR, foh, "%s(): tlv_parse failed\n", __func__);
+		return -EINVAL;
+	}
 
 	/* The message might be received over the main OML link, so we cannot
 	 * just use sign_link->trx. Resolve it by number from the FOM header. */
-	trx = gsm_bts_trx_num(sign_link->trx->bts, foh->obj_inst.trx_nr);
+	trx = gsm_bts_trx_num(bts, foh->obj_inst.trx_nr);
 
 	DEBUGPFOH(DNM, foh, "Rx IPACCESS(0x%02x): %s\n", foh->msg_type,
 		  osmo_hexdump(foh->data, oh->length - sizeof(*foh)));
@@ -2926,21 +2962,16 @@ static int abis_nm_rx_ipacc(struct msgb *msg)
 	case NM_MT_IPACC_RSL_CONNECT_NACK:
 	case NM_MT_IPACC_SET_NVATTR_NACK:
 	case NM_MT_IPACC_GET_NVATTR_NACK:
-		if (!trx)
-			goto obj_inst_error;
-		signal.trx = trx;
-		signal.msg_type = foh->msg_type;
+		signal.bts = bts;
+		signal.foh = foh;
 		osmo_signal_dispatch(SS_NM, S_NM_IPACC_NACK, &signal);
 		break;
+	case NM_MT_IPACC_RSL_CONNECT_ACK:
 	case NM_MT_IPACC_SET_NVATTR_ACK:
-		if (!trx)
-			goto obj_inst_error;
-		signal.trx = trx;
-		signal.msg_type = foh->msg_type;
-		osmo_signal_dispatch(SS_NM, S_NM_IPACC_ACK, &signal);
-		break;
 	case NM_MT_IPACC_SET_ATTR_ACK:
-		osmo_signal_dispatch(SS_NM, S_NM_IPACC_SET_ATTR_ACK, msg);
+		signal.bts = bts;
+		signal.foh = foh;
+		osmo_signal_dispatch(SS_NM, S_NM_IPACC_ACK, &signal);
 		break;
 	default:
 		break;
@@ -3006,8 +3037,17 @@ static void rsl_connect_timeout(void *data)
 	LOG_TRX(trx, DRSL, LOGL_NOTICE, "RSL connection request timed out\n");
 
 	/* Fake an RSL CONNECT NACK message from the BTS. */
-	signal.trx = trx;
-	signal.msg_type = NM_MT_IPACC_RSL_CONNECT_NACK;
+	struct abis_om_fom_hdr foh = {
+		.msg_type = NM_MT_IPACC_RSL_CONNECT_NACK,
+		.obj_class = NM_OC_BASEB_TRANSC,
+		.obj_inst = {
+			.bts_nr = trx->bts->bts_nr,
+			.trx_nr = trx->nr,
+			.ts_nr = 0xff,
+		},
+	};
+	signal.foh = &foh;
+	signal.bts = trx->bts;
 	osmo_signal_dispatch(SS_NM, S_NM_IPACC_NACK, &signal);
 }
 
@@ -3028,6 +3068,8 @@ int abis_nm_ipaccess_rsl_connect(struct gsm_bts_trx *trx,
 	if (ip != 0) {
 		ia.s_addr = htonl(ip);
 		msgb_tv_fixed_put(attr, NM_ATT_IPACC_DST_IP, 4, (void*)&ia.s_addr);
+	} else {
+		ia = (struct in_addr){};
 	}
 
 	LOG_TRX(trx, DNM, LOGL_INFO, "IPA RSL CONNECT IP=%s PORT=%u STREAM=0x%02x\n",
