@@ -44,9 +44,22 @@
 #include <osmocom/bsc/abis_om2000.h>
 #include <osmocom/bsc/signal.h>
 #include <osmocom/bsc/timeslot_fsm.h>
+#include <osmocom/bsc/nm_common_fsm.h>
 #include <osmocom/bsc/bts.h>
 #include <osmocom/abis/e1_input.h>
 
+static inline void abis_om2000_fsm_transc_becomes_enabled(struct gsm_bts_trx *trx)
+{
+	nm_obj_fsm_becomes_enabled_disabled(trx->bts, trx, NM_OC_RADIO_CARRIER, true);
+	nm_obj_fsm_becomes_enabled_disabled(trx->bts, &trx->bb_transc, NM_OC_BASEB_TRANSC, true);
+}
+
+static inline void abis_om2000_fsm_transc_becomes_disabled(struct gsm_bts_trx *trx)
+{
+	nm_obj_fsm_becomes_enabled_disabled(trx->bts, trx, NM_OC_RADIO_CARRIER, false);
+	nm_obj_fsm_becomes_enabled_disabled(trx->bts, &trx->bb_transc, NM_OC_BASEB_TRANSC, false);
+}
+
 /* FIXME: move to libosmocore */
 struct osmo_fsm_inst *osmo_fsm_inst_alloc_child_id(struct osmo_fsm *fsm,
 						   struct osmo_fsm_inst *parent,
@@ -310,6 +323,7 @@ enum abis_om2k_dei {
 	OM2K_DEI_MAX_ALLOWED_POWER		= 0xa9,
 	OM2K_DEI_MAX_ALLOWED_NUM_TRXCS		= 0xaa,
 	OM2K_DEI_MCTR_FEAT_STATUS_BMAP		= 0xab,
+	OM2K_DEI_SEEN_UNKNOWN_D2		= 0xd2,
 };
 
 enum abis_om2k_mostate {
@@ -388,6 +402,7 @@ const struct tlv_definition om2k_att_tlvdef = {
 		[OM2K_DEI_FS_OFFSET] =		{ TLV_TYPE_FIXED, 5 },
 		[OM2K_DEI_EXT_COND_MAP_2_EXT] = { TLV_TYPE_FIXED, 4 },
 		[OM2K_DEI_TSS_MO_STATE] = 	{ TLV_TYPE_FIXED, 4 },
+		[OM2K_DEI_SEEN_UNKNOWN_D2] =	{ TLV_TYPE_FIXED, 6 },
 	},
 };
 
@@ -919,7 +934,7 @@ static struct gsm_nm_state *mo2nm_state(struct gsm_bts *bts, const struct abis_o
 	return nm_state;
 }
 
-static void *mo2obj(struct gsm_bts *bts, struct abis_om2k_mo *mo)
+static void *mo2obj(struct gsm_bts *bts, const struct abis_om2k_mo *mo)
 {
 	struct gsm_bts_trx *trx;
 
@@ -947,16 +962,11 @@ static void *mo2obj(struct gsm_bts *bts, struct abis_om2k_mo *mo)
 	return NULL;
 }
 
-static void update_mo_state(struct gsm_bts *bts, struct abis_om2k_mo *mo, uint8_t mo_state)
+/* Derive an OML Availability state from an OM2000 MO state */
+static enum abis_nm_avail_state abis_nm_av_state_from_om2k_av_state(struct abis_om2k_mo *mo, uint8_t mo_state)
 {
-	struct gsm_nm_state *nm_state = mo2nm_state(bts, mo);
-	struct gsm_nm_state new_state;
-	struct nm_statechg_signal_data nsd;
 	bool has_enabled_state;
 
-	if (!nm_state)
-		return;
-
 	switch (mo->class) {
 	case OM2K_MO_CLS_CF:
 	case OM2K_MO_CLS_TRXC:
@@ -967,60 +977,132 @@ static void update_mo_state(struct gsm_bts *bts, struct abis_om2k_mo *mo, uint8_
 		break;
 	}
 
-	new_state = *nm_state;
 	switch (mo_state) {
 	case OM2K_MOSTATE_RESET:
-		new_state.availability = NM_AVSTATE_POWER_OFF;
-		break;
+		return NM_AVSTATE_POWER_OFF;
 	case OM2K_MOSTATE_STARTED:
-		new_state.availability = has_enabled_state ? NM_AVSTATE_OFF_LINE : NM_AVSTATE_OK;
-		break;
+		return has_enabled_state ? NM_AVSTATE_OFF_LINE : NM_AVSTATE_OK;
 	case OM2K_MOSTATE_ENABLED:
-		new_state.availability = NM_AVSTATE_OK;
-		break;
+		return NM_AVSTATE_OK;
 	case OM2K_MOSTATE_DISABLED:
-		new_state.availability = NM_AVSTATE_POWER_OFF;
-		break;
+		return NM_AVSTATE_POWER_OFF;
 	default:
-		new_state.availability = NM_AVSTATE_DEGRADED;
-		break;
+		return NM_AVSTATE_DEGRADED;
 	}
+}
+
+/* The OM2000 -> 12.21 mapping we do doesn't have a separate bb_transc MO,
+ * for compatibility reasons we pretend to have it anyway. */
+static void update_bb_trxc_mo_state(struct gsm_bts *bts, struct abis_om2k_mo *mo, uint8_t mo_state)
+{
+	struct nm_statechg_signal_data nsd;
+	struct gsm_bts_trx *trx;
+
+	trx = gsm_bts_trx_num(bts, mo->inst);
+	if (!trx)
+		return;
 
 	memset(&nsd, 0, sizeof(nsd));
 
 	nsd.bts = bts;
 	nsd.obj = mo2obj(bts, mo);
-	nsd.old_state = nm_state;
-	nsd.new_state = &new_state;
+	nsd.old_state = trx->bb_transc.mo.nm_state;
+	nsd.new_state = trx->bb_transc.mo.nm_state;
 	nsd.om2k_mo = mo;
 
-	osmo_signal_dispatch(SS_NM, S_NM_STATECHG_ADM, &nsd);
+	nsd.new_state.availability = abis_nm_av_state_from_om2k_av_state(mo, mo_state);
+	trx->bb_transc.mo.nm_state.availability = nsd.new_state.availability;
+	osmo_signal_dispatch(SS_NM, S_NM_STATECHG, &nsd);
+}
 
-	nm_state->availability = new_state.availability;
+static void update_mo_state(struct gsm_bts *bts, struct abis_om2k_mo *mo, uint8_t mo_state)
+{
+	struct gsm_nm_state *nm_state = mo2nm_state(bts, mo);
+	struct nm_statechg_signal_data nsd;
+
+	if (!nm_state)
+		return;
+
+	memset(&nsd, 0, sizeof(nsd));
+
+	nsd.bts = bts;
+	nsd.obj = mo2obj(bts, mo);
+	nsd.old_state = *nm_state;
+	nsd.new_state = *nm_state;
+	nsd.om2k_mo = mo;
+
+	nsd.new_state.availability = abis_nm_av_state_from_om2k_av_state(mo, mo_state);
+
+	/* Update current state before emitting signal: */
+	nm_state->availability = nsd.new_state.availability;
+	osmo_signal_dispatch(SS_NM, S_NM_STATECHG, &nsd);
+
+	/* When the TRXC MO is updated, also update the BB TRXC accordingly. */
+	if (mo->class == OM2K_MO_CLS_TRXC)
+		update_bb_trxc_mo_state(bts, mo, mo_state);
+}
+
+/* Derive an OML Operational state from an OM2000 OP state */
+static enum abis_nm_op_state abis_nm_op_state_from_om2k_op_state(uint8_t op_state)
+{
+	switch (op_state) {
+	case 1:
+		return NM_OPSTATE_ENABLED;
+	case 0:
+		return NM_OPSTATE_DISABLED;
+	default:
+		return NM_OPSTATE_NULL;
+	}
+}
+
+/* (see comment in update_bb_trxc_mo_state() above) */
+static void update_bb_trxc_op_state(struct gsm_bts *bts, const struct abis_om2k_mo *mo, uint8_t op_state)
+{
+	struct nm_statechg_signal_data nsd;
+	struct gsm_bts_trx *trx;
+
+	trx = gsm_bts_trx_num(bts, mo->inst);
+	if (!trx)
+		return;
+
+	memset(&nsd, 0, sizeof(nsd));
+
+	nsd.bts = bts;
+	nsd.obj = mo2obj(bts, mo);
+	nsd.old_state = trx->bb_transc.mo.nm_state;
+	nsd.new_state = trx->bb_transc.mo.nm_state;
+	nsd.om2k_mo = mo;
+
+	nsd.new_state.operational = abis_nm_op_state_from_om2k_op_state(op_state);
+	trx->bb_transc.mo.nm_state.operational = nsd.new_state.operational;
+	osmo_signal_dispatch(SS_NM, S_NM_STATECHG, &nsd);
 }
 
 static void update_op_state(struct gsm_bts *bts, const struct abis_om2k_mo *mo, uint8_t op_state)
 {
 	struct gsm_nm_state *nm_state = mo2nm_state(bts, mo);
-	struct gsm_nm_state new_state;
+	struct nm_statechg_signal_data nsd;
 
 	if (!nm_state)
 		return;
 
-	new_state = *nm_state;
-	switch (op_state) {
-	case 1:
-		new_state.operational = NM_OPSTATE_ENABLED;
-		break;
-	case 0:
-		new_state.operational = NM_OPSTATE_DISABLED;
-		break;
-	default:
-		new_state.operational = NM_OPSTATE_NULL;
-		break;
-	}
+	memset(&nsd, 0, sizeof(nsd));
 
-	nm_state->operational = new_state.operational;
+	nsd.bts = bts;
+	nsd.obj = mo2obj(bts, mo);
+	nsd.old_state = *nm_state;
+	nsd.new_state = *nm_state;
+	nsd.om2k_mo = mo;
+
+	nsd.new_state.operational = abis_nm_op_state_from_om2k_op_state(op_state);
+
+	/* Update current state before emitting signal: */
+	nm_state->operational = nsd.new_state.operational;
+	osmo_signal_dispatch(SS_NM, S_NM_STATECHG, &nsd);
+
+	/* When the TRXC MO is updated, also update the BB TRXC accordingly. */
+	if (mo->class == OM2K_MO_CLS_TRXC)
+		update_bb_trxc_op_state(bts, mo, op_state);
 }
 
 static int abis_om2k_sendmsg(struct gsm_bts *bts, struct msgb *msg)
@@ -2172,10 +2254,6 @@ static void om2k_trx_s_wait_ts(struct osmo_fsm_inst *fi, uint32_t event, void *d
 	struct om2k_trx_fsm_priv *otfp = fi->priv;
 	struct gsm_bts_trx_ts *ts;
 
-	/* notify TS is ready */
-	ts = &otfp->trx->ts[otfp->cur_ts_nr];
-	osmo_fsm_inst_dispatch(ts->fi, TS_EV_OML_READY, NULL);
-
 	/* next ? */
 	if (++otfp->cur_ts_nr < 8) {
 		/* iterate to the next timeslot */
@@ -2200,18 +2278,54 @@ static void om2k_trx_s_send_si(struct osmo_fsm_inst *fi, uint32_t prev_state)
 static void om2k_trx_s_done_onenter(struct osmo_fsm_inst *fi, uint32_t prev_state)
 {
 	struct om2k_trx_fsm_priv *otfp = fi->priv;
+	struct nm_statechg_signal_data nsd;
+	struct nm_statechg_signal_data nsd_bb_transc;
+	struct gsm_bts_trx *trx = otfp->trx;
+	unsigned int i;
+
+	memset(&nsd, 0, sizeof(nsd));
+
+	nsd.bts = trx->bts;
+	nsd.obj = trx;
+	nsd.old_state = trx->mo.nm_state;
+	nsd.new_state = trx->mo.nm_state;
+	nsd.om2k_mo = &trx->rbs2000.trxc.om2k_mo.addr;
 
 	/* See e1_config:bts_isdn_sign_link() / OS#4914 */
-	otfp->trx->mo.nm_state.administrative = NM_STATE_UNLOCKED;
+	nsd.new_state.administrative = NM_STATE_UNLOCKED;
+	trx->mo.nm_state.administrative = nsd.new_state.administrative;
+	osmo_signal_dispatch(SS_NM, S_NM_STATECHG, &nsd);
+
+	/* The OM2000 -> 12.21 mapping we do doesn't have separate bb_transc MO,
+	 * for compatibility reasons we pretend to have it anyway and mirror the
+	 * mo state of the TRXC on it. */
+	memset(&nsd_bb_transc, 0, sizeof(nsd));
+	nsd_bb_transc.bts = trx->bts;
+	nsd_bb_transc.obj = &trx->bb_transc;
+	nsd_bb_transc.old_state = trx->bb_transc.mo.nm_state;
+	nsd_bb_transc.new_state = trx->bb_transc.mo.nm_state;
+	nsd_bb_transc.om2k_mo = &trx->rbs2000.trxc.om2k_mo.addr;
+	nsd_bb_transc.new_state.administrative = NM_STATE_UNLOCKED;
+	trx->bb_transc.mo.nm_state.administrative = nsd_bb_transc.new_state.administrative;
+	osmo_signal_dispatch(SS_NM, S_NM_STATECHG, &nsd_bb_transc);
+
+	abis_om2000_fsm_transc_becomes_enabled(trx);
 
 	if (fi->proc.parent)
 		osmo_fsm_inst_dispatch(fi->proc.parent, otfp->done_event, NULL);
+
+	/* Notify the timeslot FSM that all TRX initialization steps are done. */
+	for (i = 0; i < ARRAY_SIZE(trx->ts); i++)
+		osmo_fsm_inst_dispatch(trx->ts[i].fi, TS_EV_OML_READY, NULL);
 }
 
 static void om2k_trx_allstate(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
+	struct om2k_trx_fsm_priv *otfp = fi->priv;
+
 	switch (event) {
 	case OM2K_TRX_EVT_RESET:
+		abis_om2000_fsm_transc_becomes_disabled(otfp->trx);
 		osmo_fsm_inst_broadcast_children(fi, event, data);
 		osmo_fsm_inst_state_chg(fi, OM2K_TRX_S_INIT, 0, 0);
 		break;
@@ -2883,7 +2997,7 @@ static void display_fault_maps(const uint8_t *src, unsigned int src_len,
 
 		/* Bail if an the maximum number of TLV fields
 		 * have been parsed */
-		if (tlv_count >= 11) {
+		if (tlv_count >= 20) {
 			LOGP(DNM, LOGL_ERROR, "Rx MO=%s Fault Report: too many tlv elements!\n",
 			     abis_om2k_mo_name(mo));
 			return;
@@ -2965,11 +3079,13 @@ int abis_om2k_rcvmsg(struct msgb *msg)
 	switch (msg_type) {
 	case OM2K_MSGT_CAL_TIME_REQ:
 		rc = abis_om2k_cal_time_resp(bts);
-		break;
+		/* we receive this from MOs without FSM (https://osmocom.org/issues/4670) */
+		goto no_mo;
 	case OM2K_MSGT_FAULT_REP:
 		display_fault_maps(msg->l2h, msgb_l2len(msg), &o2h->mo);
 		rc = abis_om2k_tx_simple(bts, &o2h->mo, OM2K_MSGT_FAULT_REP_ACK);
-		break;
+		/* we receive this from MOs without FSM (https://osmocom.org/issues/4643) */
+		goto no_mo;
 	case OM2K_MSGT_NEGOT_REQ:
 		rc = om2k_rx_negot_req(msg);
 		break;
