@@ -37,7 +37,7 @@
 #include <osmocom/bsc/abis_rsl.h>
 #include <osmocom/bsc/bts.h>
 #include <osmocom/bsc/bsc_stats.h>
-
+#include <osmocom/bsc/lchan.h>
 #include <osmocom/bsc/assignment_fsm.h>
 
 static struct osmo_fsm assignment_fsm;
@@ -51,10 +51,10 @@ struct gsm_subscriber_connection *assignment_fi_conn(struct osmo_fsm_inst *fi)
 }
 
 static const struct osmo_tdef_state_timeout assignment_fsm_timeouts[32] = {
-	[ASSIGNMENT_ST_WAIT_LCHAN_ACTIVE] = { .T=10 },
-	[ASSIGNMENT_ST_WAIT_RR_ASS_COMPLETE] = { .keep_timer=true },
-	[ASSIGNMENT_ST_WAIT_LCHAN_ESTABLISHED] = { .keep_timer=true },
-	[ASSIGNMENT_ST_WAIT_MGW_ENDPOINT_TO_MSC] = { .T=23042 },
+	[ASSIGNMENT_ST_WAIT_LCHAN_ACTIVE] = { .T = 10 },
+	[ASSIGNMENT_ST_WAIT_RR_ASS_COMPLETE] = { .keep_timer = true },
+	[ASSIGNMENT_ST_WAIT_LCHAN_ESTABLISHED] = { .keep_timer = true },
+	[ASSIGNMENT_ST_WAIT_MGW_ENDPOINT_TO_MSC] = { .T = 23042 },
 };
 
 /* Transition to a state, using the T timer defined in assignment_fsm_timeouts.
@@ -190,7 +190,7 @@ static void send_assignment_complete(struct gsm_subscriber_connection *conn)
 		assignment_fail(GSM0808_CAUSE_EQUIPMENT_FAILURE,
 				"Unable to compose Chosen Channel for mode=%s type=%s",
 				get_value_string(gsm48_chan_mode_names, lchan->current_ch_mode_rate.chan_mode),
-				gsm_lchant_name(lchan->type));
+				gsm_chan_t_name(lchan->type));
 		return;
 	}
 
@@ -222,7 +222,7 @@ static void send_assignment_complete(struct gsm_subscriber_connection *conn)
 		if (gscon_is_aoip(conn)) {
 			/* Extrapolate speech codec from speech mode */
 			gsm0808_speech_codec_from_chan_type(&sc, perm_spch);
-			sc.cfg = conn->lchan->activate.ch_mode_rate.s15_s0;
+			sc.cfg = conn->lchan->current_ch_mode_rate.s15_s0;
 			sc_ptr = &sc;
 		}
 	}
@@ -364,7 +364,7 @@ static bool lchan_type_compat_with_mode(enum gsm_chan_t type, const struct chann
 	}
 }
 
-void assignment_fsm_init()
+void assignment_fsm_init(void)
 {
 	OSMO_ASSERT(osmo_fsm_register(&assignment_fsm) == 0);
 }
@@ -597,14 +597,16 @@ void assignment_fsm_start(struct gsm_subscriber_connection *conn, struct gsm_bts
 					"Assignment of lchan %s to %s type %s requested, but lchan is not compatible",
 					gsm_lchan_name(conn->lchan),
 					gsm_lchan_name(req->target_lchan),
-					gsm_lchant_name(conn->assignment.new_lchan->type));
+					gsm_chan_t_name(conn->assignment.new_lchan->type));
 			return;
 		}
 	} else {
 		/* Try to allocate a new lchan in order of preference */
 		for (i = 0; i < req->n_ch_mode_rate; i++) {
 			conn->assignment.new_lchan = lchan_select_by_chan_mode(bts,
-			    req->ch_mode_rate_list[i].chan_mode, req->ch_mode_rate_list[i].chan_rate);
+									       req->ch_mode_rate_list[i].chan_mode,
+									       req->ch_mode_rate_list[i].chan_rate,
+									       SELECT_FOR_ASSIGNMENT, conn->lchan);
 			if (!conn->assignment.new_lchan)
 				continue;
 			LOG_ASSIGNMENT(conn, LOGL_DEBUG, "selected new lchan %s for mode[%d] = %s channel_rate=%d\n",
