@@ -88,6 +88,18 @@ static void bsc_store_bts_uptime(void *data)
 	osmo_timer_schedule(&net->bts_store_uptime_timer, BTS_STORE_UPTIME_INTERVAL, 0);
 }
 
+static void bsc_store_bts_lchan_durations(void *data)
+{
+	struct gsm_network *net = data;
+	struct gsm_bts *bts;
+
+	llist_for_each_entry(bts, &net->bts_list, list)
+		bts_store_lchan_durations(bts);
+
+	/* Keep this timer ticking. */
+	osmo_timer_schedule(&net->bts_store_lchan_durations_timer, BTS_STORE_LCHAN_DURATIONS_INTERVAL, 0);
+}
+
 static struct gsm_network *bsc_network_init(void *ctx)
 {
 	struct gsm_network *net = gsm_network_init(ctx);
@@ -120,7 +132,7 @@ static struct gsm_network *bsc_network_init(void *ctx)
 	if (!net->bts_unknown_statg)
 		goto err_free_all;
 
-	net->all_allocated_sdcch = (struct osmo_time_cc){
+	net->all_allocated.sdcch = (struct osmo_time_cc){
 		.cfg = {
 			.gran_usec = 1*1000000,
 			.forget_sum_usec = 60*1000000,
@@ -131,7 +143,7 @@ static struct gsm_network *bsc_network_init(void *ctx)
 			.T_defs = net->T_defs,
 		},
 	};
-	net->all_allocated_static_sdcch = (struct osmo_time_cc){
+	net->all_allocated.static_sdcch = (struct osmo_time_cc){
 		.cfg = {
 			.gran_usec = 1*1000000,
 			.forget_sum_usec = 60*1000000,
@@ -142,7 +154,7 @@ static struct gsm_network *bsc_network_init(void *ctx)
 			.T_defs = net->T_defs,
 		},
 	};
-	net->all_allocated_tch = (struct osmo_time_cc){
+	net->all_allocated.tch = (struct osmo_time_cc){
 		.cfg = {
 			.gran_usec = 1*1000000,
 			.forget_sum_usec = 60*1000000,
@@ -153,7 +165,7 @@ static struct gsm_network *bsc_network_init(void *ctx)
 			.T_defs = net->T_defs,
 		},
 	};
-	net->all_allocated_static_tch = (struct osmo_time_cc){
+	net->all_allocated.static_tch = (struct osmo_time_cc){
 		.cfg = {
 			.gran_usec = 1*1000000,
 			.forget_sum_usec = 60*1000000,
@@ -180,6 +192,10 @@ static struct gsm_network *bsc_network_init(void *ctx)
 	osmo_timer_setup(&net->bts_store_uptime_timer, bsc_store_bts_uptime, net);
 	osmo_timer_schedule(&net->bts_store_uptime_timer, BTS_STORE_UPTIME_INTERVAL, 0);
 
+	/* Init lchan duration tracking timer. */
+	osmo_timer_setup(&net->bts_store_lchan_durations_timer, bsc_store_bts_lchan_durations, net);
+	osmo_timer_schedule(&net->bts_store_lchan_durations_timer, BTS_STORE_LCHAN_DURATIONS_INTERVAL, 0);
+
 	net->cbc->net = net;
 	net->cbc->mode = BSC_CBC_LINK_MODE_DISABLED;
 	net->cbc->server.local_addr = bsc_cbc_default_server_local_addr;
