@@ -32,6 +32,7 @@
 #include <osmocom/bsc/handover_fsm.h>
 #include <osmocom/bsc/lchan_fsm.h>
 #include <osmocom/bsc/lchan_rtp_fsm.h>
+#include <osmocom/bsc/lchan.h>
 #include <osmocom/bsc/bsc_subscriber.h>
 #include <osmocom/bsc/osmo_bsc_sigtran.h>
 #include <osmocom/bsc/osmo_bsc_lcls.h>
@@ -620,6 +621,62 @@ static bool same_mgw_info(const struct mgcp_conn_peer *a, const struct mgcp_conn
 	return true;
 }
 
+static struct mgcp_client *select_mgw(struct gsm_subscriber_connection *conn, struct gsm_lchan *for_lchan)
+{
+	struct mgcp_client_pool_member *mgcp_pmemb;
+	struct mgcp_client *mgcp_client;
+	struct gsm_bts *bts = for_lchan->ts->trx->bts;
+
+	/* If BTS is not pinned to a given MGW, let regular allocation which
+	 * spreads load over available MGWs: */
+	if (bts->mgw_pool_target == -1)
+		goto pick_any;
+
+	/* BTS is pinned to an MGW, retrieve pointer to it: */
+	mgcp_pmemb = mgcp_client_pool_find_member_by_nr(conn->network->mgw.mgw_pool, bts->mgw_pool_target);
+	if (!mgcp_pmemb) {
+		if (!bts->mgw_pool_target_strict) {
+			LOGPFSML(conn->fi, LOGL_NOTICE,
+				 "mgw pool-target %u not found! selecting another one.\n", bts->mgw_pool_target);
+			goto pick_any;
+		} else {
+			LOGPFSML(conn->fi, LOGL_ERROR, "mgw pool-target %u not found!\n", bts->mgw_pool_target);
+			return NULL;
+		}
+	}
+	if (mgcp_client_pool_member_is_blocked(mgcp_pmemb)) {
+		if (!bts->mgw_pool_target_strict) {
+			LOGPFSML(conn->fi, LOGL_NOTICE,
+				 "mgw pool-target %u is administratively blocked! selecting another one.\n",
+				 bts->mgw_pool_target);
+			goto pick_any;
+		} else {
+			LOGPFSML(conn->fi, LOGL_ERROR, "mgw pool-target %u is administratively blocked!\n",
+				 bts->mgw_pool_target);
+			return NULL;
+		}
+	}
+
+	mgcp_client = mgcp_client_pool_member_get(mgcp_pmemb);
+	if (!mgcp_client) {
+		if (!bts->mgw_pool_target_strict) {
+			LOGPFSML(conn->fi, LOGL_NOTICE,
+				 "mgw pool-target %u is not connected! selecting another one.\n",
+				 bts->mgw_pool_target);
+			goto pick_any;
+		} else {
+			LOGPFSML(conn->fi, LOGL_ERROR, "mgw pool-target %u is not connected!\n",
+				 bts->mgw_pool_target);
+			return NULL;
+		}
+	}
+	return mgcp_client;
+
+pick_any:
+	mgcp_client = mgcp_client_pool_get(conn->network->mgw.mgw_pool);
+	return mgcp_client;
+}
+
 /* Make sure a conn->user_plane.mgw_endpoint is allocated with the proper mgw endpoint name.  For
  * SCCPlite, pass in msc_assigned_cic the CIC received upon BSSMAP Assignment Command or BSSMAP Handover
  * Request form the MSC (which is only stored in conn->user_plane after success). Ignored for AoIP. */
@@ -639,11 +696,10 @@ struct osmo_mgcpc_ep *gscon_ensure_mgw_endpoint(struct gsm_subscriber_connection
 
 	if (gscon_is_sccplite(conn) || gscon_is_aoip(conn)) {
 		/* Get MGCP client from pool */
-		mgcp_client = mgcp_client_pool_get(conn->network->mgw.mgw_pool);
+		mgcp_client = select_mgw(conn, for_lchan);
 		if (!mgcp_client) {
 			LOGPFSML(conn->fi, LOGL_ERROR,
 				 "cannot ensure MGW endpoint -- no MGW configured, check configuration!\n");
-			conn->user_plane.mgw_endpoint = NULL;
 			return NULL;
 		}
 	}
@@ -940,6 +996,10 @@ static void gscon_forget_mgw_endpoint(struct gsm_subscriber_connection *conn)
 	mgcp_client = osmo_mgcpc_ep_client(conn->user_plane.mgw_endpoint);
 	mgcp_client_pool_put(mgcp_client);
 
+	/* Be sure that the endpoint CI we are maintaining in user_plane
+	 * is also removed from the other locations as well. */
+	gscon_forget_mgw_endpoint_ci(conn, conn->user_plane.mgw_endpoint_ci_msc);
+
 	conn->user_plane.mgw_endpoint = NULL;
 	conn->user_plane.mgw_endpoint_ci_msc = NULL;
 	conn->ho.created_ci_for_msc = NULL;
@@ -1067,7 +1127,7 @@ static void gscon_pre_term(struct osmo_fsm_inst *fi, enum osmo_fsm_term_cause ca
 
 	osmo_mgcpc_ep_clear(conn->user_plane.mgw_endpoint);
 	conn->user_plane.mgw_endpoint = NULL;
-	conn->user_plane.mgw_endpoint = NULL;
+	conn->user_plane.mgw_endpoint_ci_msc = NULL;
 
 	if (conn->ho.fi)
 		osmo_fsm_inst_dispatch(conn->ho.fi, HO_EV_CONN_RELEASING, NULL);
@@ -1142,7 +1202,7 @@ static struct osmo_fsm gscon_fsm = {
 	.event_names = gscon_fsm_event_names,
 };
 
-void bsc_subscr_conn_fsm_init()
+void bsc_subscr_conn_fsm_init(void)
 {
 	OSMO_ASSERT(osmo_fsm_register(&gscon_fsm) == 0);
 	OSMO_ASSERT(osmo_fsm_register(&lcls_fsm) == 0);
