@@ -47,7 +47,7 @@ static const struct osmo_tdef_state_timeout bssmap_reset_timeouts[32] = {
 	osmo_tdef_fsm_inst_state_chg(FI, STATE, \
 				     bssmap_reset_timeouts, \
 				     (bsc_gsmnet)->T_defs, \
-				     5)
+				     -1)
 
 struct bssmap_reset *bssmap_reset_alloc(void *ctx, const char *label, const struct bssmap_reset_cfg *cfg)
 {
@@ -245,11 +245,13 @@ bool bssmap_reset_is_conn_ready(const struct bssmap_reset *bssmap_reset)
 
 void bssmap_reset_resend_reset(struct bssmap_reset *bssmap_reset)
 {
+	OSMO_ASSERT(bssmap_reset);
+
 	/* Immediately (1ms) kick off reset sending mechanism */
 	osmo_fsm_inst_state_chg_ms(bssmap_reset->fi, BSSMAP_RESET_ST_DISC, 1, 0);
 }
 
-static __attribute__((constructor)) void bssmap_reset_fsm_init()
+static __attribute__((constructor)) void bssmap_reset_fsm_init(void)
 {
 	OSMO_ASSERT(osmo_fsm_register(&bssmap_reset_fsm) == 0);
 }
