@@ -24,6 +24,8 @@
 #include <osmocom/bsc/bts.h>
 #include <osmocom/bsc/debug.h>
 #include <osmocom/bsc/nm_common_fsm.h>
+#include <osmocom/bsc/paging.h>
+#include <osmocom/bsc/smscb.h>
 
 const struct value_string bts_attribute_names[] = {
 	OSMO_VALUE_STRING(BTS_TYPE_VARIANT),
@@ -91,12 +93,6 @@ const char *btstype2str(enum gsm_bts_type type)
 	return get_value_string(bts_type_names, type);
 }
 
-static void bts_init_cbch_state(struct bts_smscb_chan_state *cstate, struct gsm_bts *bts)
-{
-	cstate->bts = bts;
-	INIT_LLIST_HEAD(&cstate->messages);
-}
-
 static LLIST_HEAD(bts_models);
 
 struct gsm_bts_model *bts_model_find(enum gsm_bts_type type)
@@ -148,6 +144,11 @@ static const struct gprs_rlc_cfg rlc_cfg_default = {
 
 static int gsm_bts_talloc_destructor(struct gsm_bts *bts)
 {
+	paging_destructor(bts);
+	bts_setup_ramp_remove(bts);
+
+	osmo_timer_del(&bts->cbch_timer);
+
 	bts->site_mgr->bts[0] = NULL;
 
 	if (bts->gprs.cell.mo.fi) {
@@ -159,6 +160,9 @@ static int gsm_bts_talloc_destructor(struct gsm_bts *bts)
 		osmo_fsm_inst_free(bts->mo.fi);
 		bts->mo.fi = NULL;
 	}
+
+	osmo_stat_item_group_free(bts->bts_statg);
+	rate_ctr_group_free(bts->bts_ctrs);
 	return 0;
 }
 
@@ -169,8 +173,6 @@ static int gsm_bts_talloc_destructor(struct gsm_bts *bts)
 struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm, uint8_t bts_num)
 {
 	struct gsm_bts *bts = talloc_zero(bts_sm, struct gsm_bts);
-	struct gsm48_multi_rate_conf mr_cfg;
-	int i;
 
 	if (!bts)
 		return NULL;
@@ -211,7 +213,7 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 	}
 	bts->bts_statg = osmo_stat_item_group_alloc(bts, &bts_statg_desc, bts->nr);
 
-	bts->all_allocated_sdcch = (struct osmo_time_cc){
+	bts->all_allocated.sdcch = (struct osmo_time_cc){
 		.cfg = {
 			.gran_usec = 1*1000000,
 			.forget_sum_usec = 60*1000000,
@@ -222,7 +224,7 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 			.T_defs = net->T_defs,
 		},
 	};
-	bts->all_allocated_static_sdcch = (struct osmo_time_cc){
+	bts->all_allocated.static_sdcch = (struct osmo_time_cc){
 		.cfg = {
 			.gran_usec = 1*1000000,
 			.forget_sum_usec = 60*1000000,
@@ -233,7 +235,7 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 			.T_defs = net->T_defs,
 		},
 	};
-	bts->all_allocated_tch = (struct osmo_time_cc){
+	bts->all_allocated.tch = (struct osmo_time_cc){
 		.cfg = {
 			.gran_usec = 1*1000000,
 			.forget_sum_usec = 60*1000000,
@@ -244,7 +246,7 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 			.T_defs = net->T_defs,
 		},
 	};
-	bts->all_allocated_static_tch = (struct osmo_time_cc){
+	bts->all_allocated.static_tch = (struct osmo_time_cc){
 		.cfg = {
 			.gran_usec = 1*1000000,
 			.forget_sum_usec = 60*1000000,
@@ -267,11 +269,11 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 	bts->c0->ts[0].pchan_from_config = GSM_PCHAN_CCCH_SDCCH4; /* TODO: really?? */
 
 	bts->ccch_load_ind_thresh = 10; /* 10% of Load: Start sending CCCH LOAD IND */
+	bts->ccch_load_ind_period = 1; /* Send CCCH LOAD IND every 1 second */
 	bts->rach_b_thresh = -1;
 	bts->rach_ldavg_slots = -1;
 
-	bts->paging.free_chans_need = -1;
-	INIT_LLIST_HEAD(&bts->paging.pending_requests);
+	paging_init(bts);
 
 	bts->features.data = &bts->_features_data[0];
 	bts->features.data_len = sizeof(bts->_features_data);
@@ -290,7 +292,10 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 	bts->neigh_list_manual_mode = NL_MODE_AUTOMATIC;
 	bts->early_classmark_allowed_3g = true; /* 3g Early Classmark Sending controlled by bts->early_classmark_allowed param */
 	bts->si_unused_send_empty = true;
-	bts->chan_alloc_allow_tch_for_signalling = true;
+	bts->chan_alloc_tch_signalling_policy = BTS_TCH_SIGNALLING_ALWAYS;
+	bts->chan_alloc_dyn_params.ul_rxlev_thresh = 50; /* >= -60 dBm */
+	bts->chan_alloc_dyn_params.ul_rxlev_avg_num = 2; /* at least 2 samples */
+	bts->chan_alloc_dyn_params.c0_chan_load_thresh = 60; /* >= 60% */
 	bts->si_common.cell_sel_par.cell_resel_hyst = 2; /* 4 dB */
 	bts->si_common.cell_sel_par.rxlev_acc_min = 0;
 	bts->si_common.si2quater_neigh_list.arfcn = bts->si_common.data.earfcn_list;
@@ -327,6 +332,9 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 	INIT_LLIST_HEAD(&bts->oml_fail_rep);
 	INIT_LLIST_HEAD(&bts->chan_rqd_queue);
 
+	/* Don't pin the BTS to any MGW by default: */
+	bts->mgw_pool_target = -1;
+
 	/* Enable all codecs by default. These get reset to a more fine grained selection IF a
 	 * 'codec-support' config appears in the config file (see bsc_vty.c). */
 	bts->codec = (struct bts_codec_conf){
@@ -336,64 +344,100 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 	};
 
 	/* Set reasonable defaults for AMR-FR and AMR-HR rate configuration.
-	 * (see also 3GPP TS 28.062, Table 7.11.3.1.3-2) */
-	mr_cfg = (struct gsm48_multi_rate_conf) {
+	 * The values are taken from 3GPP TS 51.010-1 (version 13.11.0).
+	 * See 14.2.19.4.1 and 14.2.20.4.1 for AMR-FR and AMR-HR, respectively. */
+	static const struct gsm48_multi_rate_conf amr_fr_mr_cfg = {
 		.m4_75 = 1,
-		.m5_15 = 0,
 		.m5_90 = 1,
-		.m6_70 = 0,
-		.m7_40 = 1,
-		.m7_95 = 0,
-		.m10_2 = 0,
+		.m7_95 = 1,
 		.m12_2 = 1
 	};
-	memcpy(bts->mr_full.gsm48_ie, &mr_cfg, sizeof(bts->mr_full.gsm48_ie));
-	bts->mr_full.ms_mode[0].mode = 0;
-	bts->mr_full.ms_mode[1].mode = 2;
-	bts->mr_full.ms_mode[2].mode = 4;
-	bts->mr_full.ms_mode[3].mode = 7;
-	bts->mr_full.bts_mode[0].mode = 0;
-	bts->mr_full.bts_mode[1].mode = 2;
-	bts->mr_full.bts_mode[2].mode = 4;
-	bts->mr_full.bts_mode[3].mode = 7;
-	for (i = 0; i < 3; i++) {
-		bts->mr_full.ms_mode[i].hysteresis = 8;
-		bts->mr_full.ms_mode[i].threshold = 32;
-		bts->mr_full.bts_mode[i].hysteresis = 8;
-		bts->mr_full.bts_mode[i].threshold = 32;
-	}
-	bts->mr_full.num_modes = 4;
-
-	mr_cfg = (struct gsm48_multi_rate_conf) {
+	static const struct gsm48_multi_rate_conf amr_hr_mr_cfg = {
 		.m4_75 = 1,
-		.m5_15 = 0,
 		.m5_90 = 1,
-		.m6_70 = 0,
-		.m7_40 = 1,
-		.m7_95 = 0,
-		.m10_2 = 0,
-		.m12_2 = 0
+		.m6_70 = 1,
+		.m7_95 = 1,
 	};
-	memcpy(bts->mr_half.gsm48_ie, &mr_cfg, sizeof(bts->mr_half.gsm48_ie));
-	bts->mr_half.ms_mode[0].mode = 0;
-	bts->mr_half.ms_mode[1].mode = 2;
-	bts->mr_half.ms_mode[2].mode = 4;
-	bts->mr_half.ms_mode[3].mode = 7;
-	bts->mr_half.bts_mode[0].mode = 0;
-	bts->mr_half.bts_mode[1].mode = 2;
-	bts->mr_half.bts_mode[2].mode = 4;
-	bts->mr_half.bts_mode[3].mode = 7;
-	for (i = 0; i < 3; i++) {
-		bts->mr_half.ms_mode[i].hysteresis = 8;
-		bts->mr_half.ms_mode[i].threshold = 32;
-		bts->mr_half.bts_mode[i].hysteresis = 8;
-		bts->mr_half.bts_mode[i].threshold = 32;
-	}
-	bts->mr_half.num_modes = 3;
 
-	bts_init_cbch_state(&bts->cbch_basic, bts);
-	bts_init_cbch_state(&bts->cbch_extended, bts);
+	memcpy(bts->mr_full.gsm48_ie, &amr_fr_mr_cfg, sizeof(bts->mr_full.gsm48_ie));
+	memcpy(bts->mr_half.gsm48_ie, &amr_hr_mr_cfg, sizeof(bts->mr_half.gsm48_ie));
 
+	/*                   ^ C/I (dB)             |  FR  /  HR  |
+	 *            |      |
+	 *            |      |
+	 *   MODE4    |      |
+	 *          = |  ----+----  THR_MX_Up(3)    | 20.5 / 18.0 |
+	 *          | |      |
+	 *          | =  ----+----  THR_MX_Dn(4)    | 18.5 / 16.0 |
+	 *   MODE3  |        |
+	 *          | =  ----+----  THR_MX_Up(2)    | 14.5 / 14.0 |
+	 *          | |      |
+	 *          = |  ----+----  THR_MX_Dn(3)    | 12.5 / 12.0 |
+	 *   MODE2    |      |
+	 *          = |  ----+----  THR_MX_Up(1)    |  8.5 / 10.0 |
+	 *          | |      |
+	 *          | =  ----+----  THR_MX_Dn(2)    |  6.5 /  8.0 |
+	 *   MODE1  |        |
+	 *          |        |
+	 *          |        |
+	 */
+	static const struct amr_mode amr_fr_ms_bts_mode[] = {
+		{
+			.mode = 0, /* 4.75k */
+			.threshold = 13, /* THR_MX_Dn(2): 6.5 dB */
+			.hysteresis = 4, /* THR_MX_Up(1): 8.5 dB */
+		},
+		{
+			.mode = 2, /* 5.90k */
+			.threshold = 25, /* THR_MX_Dn(3): 12.5 dB */
+			.hysteresis = 4, /* THR_MX_Up(2): 14.5 dB */
+		},
+		{
+			.mode = 5, /* 7.95k */
+			.threshold = 37, /* THR_MX_Dn(4): 18.5 dB */
+			.hysteresis = 4, /* THR_MX_Up(3): 20.5 dB */
+		},
+		{
+			.mode = 7, /* 12.2k */
+			/* this is the last mode, so no threshold */
+		},
+	};
+	static const struct amr_mode amr_hr_ms_bts_mode[] = {
+		{
+			.mode = 0, /* 4.75k */
+			.threshold = 16, /* THR_MX_Dn(2):  8.0 dB */
+			.hysteresis = 4, /* THR_MX_Up(1): 10.0 dB */
+		},
+		{
+			.mode = 2, /* 5.90k */
+			.threshold = 24, /* THR_MX_Dn(3): 12.0 dB */
+			.hysteresis = 4, /* THR_MX_Up(2): 14.0 dB */
+		},
+		{
+			.mode = 3, /* 6.70k */
+			.threshold = 32, /* THR_MX_Dn(4): 16.0 dB */
+			.hysteresis = 4, /* THR_MX_Up(3): 18.0 dB */
+		},
+		{
+			.mode = 5, /* 7.95k */
+			/* this is the last mode, so no threshold */
+		},
+	};
+
+	memcpy(&bts->mr_full.ms_mode[0], &amr_fr_ms_bts_mode[0], sizeof(amr_fr_ms_bts_mode));
+	memcpy(&bts->mr_full.bts_mode[0], &amr_fr_ms_bts_mode[0], sizeof(amr_fr_ms_bts_mode));
+	bts->mr_full.num_modes = ARRAY_SIZE(amr_fr_ms_bts_mode);
+
+	memcpy(&bts->mr_half.ms_mode[0], &amr_hr_ms_bts_mode[0], sizeof(amr_hr_ms_bts_mode));
+	memcpy(&bts->mr_half.bts_mode[0], &amr_hr_ms_bts_mode[0], sizeof(amr_hr_ms_bts_mode));
+	bts->mr_half.num_modes = ARRAY_SIZE(amr_hr_ms_bts_mode);
+
+	bts->use_osmux = OSMUX_USAGE_OFF;
+
+	bts_cbch_init(bts);
+	bts_etws_init(bts);
+
+	bts_setup_ramp_init_bts(bts);
 	acc_mgr_init(&bts->acc_mgr, bts);
 	acc_ramp_init(&bts->acc_ramp, bts);
 
@@ -421,6 +465,79 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 	return bts;
 }
 
+/* Validate BTS configuration (ARFCN settings and physical channel configuration) */
+__attribute__((weak)) int gsm_bts_check_cfg(struct gsm_bts *bts)
+{
+	struct gsm_bts_trx *trx;
+
+	if (!bts->model)
+		return -EFAULT;
+
+	switch (bts->band) {
+	case GSM_BAND_1800:
+		if (bts->c0->arfcn < 512 || bts->c0->arfcn > 885) {
+			LOGP(DNM, LOGL_ERROR, "(bts=%u) GSM1800 channel (%u) must be between 512-885.\n",
+			     bts->nr, bts->c0->arfcn);
+			return -EINVAL;
+		}
+		break;
+	case GSM_BAND_1900:
+		if (bts->c0->arfcn < 512 || bts->c0->arfcn > 810) {
+			LOGP(DNM, LOGL_ERROR, "(bts=%u) GSM1900 channel (%u) must be between 512-810.\n",
+			     bts->nr, bts->c0->arfcn);
+			return -EINVAL;
+		}
+		break;
+	case GSM_BAND_900:
+		if ((bts->c0->arfcn > 124 && bts->c0->arfcn < 955) ||
+		    bts->c0->arfcn > 1023) {
+			LOGP(DNM, LOGL_ERROR, "(bts=%u) GSM900 channel (%u) must be between 0-124, 955-1023.\n",
+			     bts->nr, bts->c0->arfcn);
+			return -EINVAL;
+		}
+		break;
+	case GSM_BAND_850:
+		if (bts->c0->arfcn < 128 || bts->c0->arfcn > 251) {
+			LOGP(DNM, LOGL_ERROR, "(bts=%u) GSM850 channel (%u) must be between 128-251.\n",
+			     bts->nr, bts->c0->arfcn);
+			return -EINVAL;
+		}
+		break;
+	default:
+		LOGP(DNM, LOGL_ERROR, "(bts=%u) Unsupported frequency band.\n", bts->nr);
+		return -EINVAL;
+	}
+
+	if (bts->features_known) {
+		if (!bts_gprs_mode_is_compat(bts, bts->gprs.mode)) {
+			LOGP(DNM, LOGL_ERROR, "(bts=%u) GPRS mode set to '%s', but BTS does not support it\n", bts->nr,
+			     bts_gprs_mode_name(bts->gprs.mode));
+			return -EINVAL;
+		}
+		if (bts->use_osmux == OSMUX_USAGE_ONLY &&
+		    !osmo_bts_has_feature(&bts->features, BTS_FEAT_OSMUX)) {
+			LOGP(DNM, LOGL_ERROR,
+			     "(bts=%u) osmux use set to 'only', but BTS does not support Osmux\n",
+			     bts->nr);
+			return -EINVAL;
+		}
+	}
+
+	/* Verify the physical channel mapping */
+	llist_for_each_entry(trx, &bts->trx_list, list) {
+		if (!trx_has_valid_pchan_config(trx)) {
+			LOGP(DNM, LOGL_ERROR, "TRX %u has invalid timeslot "
+					      "configuration\n", trx->nr);
+			return -EINVAL;
+		}
+	}
+
+	if (!gsm_bts_check_ny1(bts))
+		return -EINVAL;
+
+	return 0;
+}
+
 static char ts2str[255];
 
 char *gsm_bts_name(const struct gsm_bts *bts)
@@ -469,6 +586,7 @@ bool gsm_bts_matches_cell_id(const struct gsm_bts *bts, const struct gsm0808_cel
 	case CELL_IDENT_UTRAN_PLMN_LAC_RNC:
 	case CELL_IDENT_UTRAN_RNC:
 	case CELL_IDENT_UTRAN_LAC_RNC:
+	case CELL_IDENT_SAI:
 		return false;
 	default:
 		OSMO_ASSERT(false);
@@ -506,22 +624,40 @@ void gsm_bts_cell_id_list(struct gsm0808_cell_id_list2 *cell_id_list, const stru
 /* return the gsm_lchan for the CBCH (if it exists at all) */
 struct gsm_lchan *gsm_bts_get_cbch(struct gsm_bts *bts)
 {
-	struct gsm_lchan *lchan = NULL;
 	struct gsm_bts_trx *trx = bts->c0;
 
+	/* According to 3GPP TS 45.002, table 3, CBCH can be allocated
+	 * either on C0/TS0 (CCCH+SDCCH4) or on C0..n/TS0..3 (SDCCH/8). */
 	if (trx->ts[0].pchan_from_config == GSM_PCHAN_CCCH_SDCCH4_CBCH)
-		lchan = &trx->ts[0].lchan[2];
-	else {
-		int i;
-		for (i = 0; i < 8; i++) {
-			if (trx->ts[i].pchan_from_config == GSM_PCHAN_SDCCH8_SACCH8C_CBCH) {
-				lchan = &trx->ts[i].lchan[2];
-				break;
-			}
+		return &trx->ts[0].lchan[2]; /* C0/TS0 */
+
+	llist_for_each_entry(trx, &bts->trx_list, list) { /* C0..n */
+		unsigned int tn;
+		for (tn = 0; tn < 4; tn++) { /* TS0..3 */
+			struct gsm_bts_trx_ts *ts = &trx->ts[tn];
+			if (ts->pchan_from_config == GSM_PCHAN_SDCCH8_SACCH8C_CBCH)
+				return &ts->lchan[2];
 		}
 	}
 
-	return lchan;
+	return NULL;
+}
+
+int gsm_set_bts_model(struct gsm_bts *bts, struct gsm_bts_model *model)
+{
+	bts->model = model;
+
+	/* Copy hardcoded feature list from BTS model, unless the BTS supports
+	 * reporting features at runtime (as of writing nanobts, OsmoBTS). */
+	if (!model || model->features_get_reported) {
+		memset(bts->_features_data, 0, sizeof(bts->_features_data));
+		bts->features_known = false;
+	} else {
+		memcpy(bts->_features_data, bts->model->_features_data, sizeof(bts->_features_data));
+		bts->features_known = true;
+	}
+
+	return 0;
 }
 
 int gsm_set_bts_type(struct gsm_bts *bts, enum gsm_bts_type type)
@@ -536,7 +672,7 @@ int gsm_set_bts_type(struct gsm_bts *bts, enum gsm_bts_type type)
 		return -EINVAL;
 
 	bts->type = type;
-	bts->model = model;
+	gsm_set_bts_model(bts, model);
 
 	if (model->start && !model->started) {
 		int ret = model->start(bts->network);
@@ -589,11 +725,11 @@ int gsm_set_bts_type(struct gsm_bts *bts, enum gsm_bts_type type)
 int bts_gprs_mode_is_compat(struct gsm_bts *bts, enum bts_gprs_mode mode)
 {
 	if (mode != BTS_GPRS_NONE &&
-	    !osmo_bts_has_feature(&bts->model->features, BTS_FEAT_GPRS)) {
+	    !osmo_bts_has_feature(&bts->features, BTS_FEAT_GPRS)) {
 		return 0;
 	}
 	if (mode == BTS_GPRS_EGPRS &&
-	    !osmo_bts_has_feature(&bts->model->features, BTS_FEAT_EGPRS)) {
+	    !osmo_bts_has_feature(&bts->features, BTS_FEAT_EGPRS)) {
 		return 0;
 	}
 
@@ -620,6 +756,61 @@ void bts_store_uptime(struct gsm_bts *bts)
 	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_UPTIME_SECONDS), bts_uptime(bts));
 }
 
+void bts_store_lchan_durations(struct gsm_bts *bts)
+{
+	struct gsm_bts_trx *trx;
+	int i, j;
+	struct timespec now, elapsed;
+	uint64_t elapsed_ms;
+	uint64_t elapsed_tch_ms = 0;
+	uint64_t elapsed_sdcch_ms = 0;
+
+	/* Ignore BTS that are not in operation. */
+	if (!trx_is_usable(bts->c0))
+		return;
+
+	/* Grab storage time to be used for all lchans. */
+	osmo_clock_gettime(CLOCK_MONOTONIC, &now);
+
+	/* Iterate over all lchans. */
+	llist_for_each_entry(trx, &bts->trx_list, list) {
+		for (i = 0; i < ARRAY_SIZE(trx->ts); i++) {
+			struct gsm_bts_trx_ts *ts = &trx->ts[i];
+			for (j = 0; j < ARRAY_SIZE(ts->lchan); j++) {
+				struct gsm_lchan *lchan = &ts->lchan[j];
+
+				/* Ignore lchans whose activation timestamps are not yet set. */
+				if (lchan->active_stored.tv_sec == 0 && lchan->active_stored.tv_nsec == 0)
+					continue;
+
+				/* Calculate elapsed time since last storage. */
+				timespecsub(&now, &lchan->active_stored, &elapsed);
+				elapsed_ms = elapsed.tv_sec * 1000 + elapsed.tv_nsec / 1000000;
+
+				/* Assign elapsed time to appropriate bucket. */
+				switch (lchan->type) {
+				case GSM_LCHAN_TCH_H:
+				case GSM_LCHAN_TCH_F:
+					elapsed_tch_ms += elapsed_ms;
+					break;
+				case GSM_LCHAN_SDCCH:
+					elapsed_sdcch_ms += elapsed_ms;
+					break;
+				default:
+					continue;
+				}
+
+				/* Update storage time. */
+				lchan->active_stored = now;
+			}
+		}
+	}
+
+	/* Export to rate counters. */
+	rate_ctr_add(rate_ctr_group_get_ctr(bts->bts_ctrs, BTS_CTR_CHAN_TCH_ACTIVE_MILLISECONDS_TOTAL), elapsed_tch_ms);
+	rate_ctr_add(rate_ctr_group_get_ctr(bts->bts_ctrs, BTS_CTR_CHAN_SDCCH_ACTIVE_MILLISECONDS_TOTAL), elapsed_sdcch_ms);
+}
+
 unsigned long long bts_uptime(const struct gsm_bts *bts)
 {
 	struct timespec tp;
@@ -677,7 +868,7 @@ void bts_depend_mark(struct gsm_bts *bts, int dep)
 	int idx, bit;
 	depends_calc_index_bit(dep, &idx, &bit);
 
-	bts->depends_on[idx] |= 1 << bit;
+	bts->depends_on[idx] |= 1U << bit;
 }
 
 void bts_depend_clear(struct gsm_bts *bts, int dep)
@@ -685,7 +876,7 @@ void bts_depend_clear(struct gsm_bts *bts, int dep)
 	int idx, bit;
 	depends_calc_index_bit(dep, &idx, &bit);
 
-	bts->depends_on[idx] &= ~(1 << bit);
+	bts->depends_on[idx] &= ~(1U << bit);
 }
 
 int bts_depend_is_depedency(struct gsm_bts *base, struct gsm_bts *other)
@@ -694,7 +885,7 @@ int bts_depend_is_depedency(struct gsm_bts *base, struct gsm_bts *other)
 	depends_calc_index_bit(other->nr, &idx, &bit);
 
 	/* Check if there is a depends bit */
-	return (base->depends_on[idx] & (1 << bit)) > 0;
+	return (base->depends_on[idx] & (1U << bit)) > 0;
 }
 
 static int bts_is_online(struct gsm_bts *bts)
@@ -854,6 +1045,9 @@ void gsm_bts_stats_reset(struct gsm_bts *bts)
 	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_CHAN_SDCCH8_CBCH_TOTAL), 0);
 	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_CHAN_OSMO_DYN_USED), 0);
 	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_CHAN_OSMO_DYN_TOTAL), 0);
+	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_PAGING_T3113), 0);
+	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_PAGING_REQ_QUEUE_LENGTH), paging_pending_requests_nr(bts));
+	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_PAGING_AVAILABLE_SLOTS), bts->paging.available_slots);
 }
 
 const struct rate_ctr_desc bts_ctr_description[] = {
@@ -928,7 +1122,7 @@ const struct rate_ctr_desc bts_ctr_description[] = {
 		  "Received a TEI down on a OML link" },
 	[BTS_CTR_BTS_RSL_FAIL] = \
 		{ "rsl_fail",
-		  "Received a TEI down on a OML link" },
+		  "Received a TEI down on a RSL link" },
 	[BTS_CTR_CODEC_AMR_F] = \
 		{ "codec:amr_f",
 		  "Count the usage of AMR/F codec by channel mode requested" },
@@ -962,6 +1156,9 @@ const struct rate_ctr_desc bts_ctr_description[] = {
 	[BTS_CTR_PAGING_MSC_FLUSH] = \
 		{ "paging:msc_flush",
 		  "Paging flushed due to MSC Reset BSSMAP message" },
+	[BTS_CTR_PAGING_OVERLOAD] = \
+		{ "paging:overload",
+		  "Paging dropped due to BSC Paging queue overload" },
 	[BTS_CTR_CHAN_ACT_TOTAL] = \
 		{ "chan_act:total",
 		  "Total number of Channel Activations" },
@@ -974,6 +1171,18 @@ const struct rate_ctr_desc bts_ctr_description[] = {
 	[BTS_CTR_CHAN_ACT_NACK] = \
 		{ "chan_act:nack",
 		  "Number of Channel Activations that the BTS NACKed" },
+	[BTS_CTR_CHAN_TCH_ACTIVE_MILLISECONDS_TOTAL] = \
+		{ "chan_tch:active_milliseconds:total",
+		  "Cumulative number of milliseconds of TCH channel activity" },
+	[BTS_CTR_CHAN_SDCCH_ACTIVE_MILLISECONDS_TOTAL] = \
+		{ "chan_sdcch:active_milliseconds:total",
+		  "Cumulative number of milliseconds of SDCCH channel activity" },
+	[BTS_CTR_CHAN_TCH_FULLY_ESTABLISHED] = \
+		{ "chan_tch:fully_established",
+		  "Number of TCH channels which have reached the fully established state" },
+	[BTS_CTR_CHAN_SDCCH_FULLY_ESTABLISHED] = \
+		{ "chan_sdcch:fully_established",
+		  "Number of SDCCH channels which have reached the fully established state" },
 	[BTS_CTR_RSL_UNKNOWN] = \
 		{ "rsl:unknown",
 		  "Number of unknown/unsupported RSL messages received from BTS" },
@@ -1490,6 +1699,18 @@ const struct osmo_stat_item_desc bts_stat_desc[] = {
 		{ "num_trx:total",
 		  "Number of configured TRX in this BTS",
 		  "" },
+	[BTS_STAT_PAGING_REQ_QUEUE_LENGTH] = \
+		{ "paging:request_queue_length",
+		  "Paging Request queue length",
+		  "", 60, 0 },
+	[BTS_STAT_PAGING_AVAILABLE_SLOTS] = \
+		{ "paging:available_slots",
+		  "Available paging slots in this BTS",
+		  "", 60, 0 },
+	[BTS_STAT_PAGING_T3113] = \
+		{ "paging:t3113",
+		  "T3113 paging timer",
+		  "s", 60, 0 },
 };
 
 const struct osmo_stat_item_group_desc bts_statg_desc = {
@@ -1499,3 +1720,20 @@ const struct osmo_stat_item_group_desc bts_statg_desc = {
 	.num_items = ARRAY_SIZE(bts_stat_desc),
 	.item_desc = bts_stat_desc,
 };
+
+/* Return 'true' if and only if Ny1 satisfies network requirements */
+bool gsm_bts_check_ny1(const struct gsm_bts *bts)
+{
+	unsigned long T3105, ny1, ny1_recommended;
+	T3105 = osmo_tdef_get(bts->network->T_defs, 3105, OSMO_TDEF_MS, -1);
+	ny1 = osmo_tdef_get(bts->network->T_defs, -3105, OSMO_TDEF_CUSTOM, -1);
+	if (!(T3105 * ny1 > GSM_T3124_MAX + GSM_NY1_REQ_DELTA)) {
+		/* See comment for GSM_NY1_DEFAULT */
+		ny1_recommended = (GSM_T3124_MAX + GSM_NY1_REQ_DELTA)/T3105 + 1;
+		LOGP(DNM, LOGL_ERROR, "Value of Ny1 should be higher. "
+				     "Is: %lu, lowest recommendation: %lu\n",
+		     ny1, ny1_recommended);
+		return false;
+	}
+	return true;
+}
