@@ -24,7 +24,7 @@ int bts_init(void)
 	bts_model_rbs2k_init();
 	bts_model_nanobts_init();
 	bts_model_nokia_site_init();
-	bts_model_sysmobts_init();
+	bts_model_osmobts_init();
 	/* Your new BTS here. */
 	return 0;
 }
