@@ -131,6 +131,7 @@ struct gsm_bts_model bts_model_nanobts = {
 			[NM_ATT_IPACC_REVOC_DATE] =	{ TLV_TYPE_TL16V },
 		},
 	},
+	.features_get_reported = true,
 };
 
 
@@ -167,7 +168,6 @@ static int nm_statechg_event(int evt, struct nm_statechg_signal_data *nsd)
 		break;
 	case NM_OC_CHANNEL:
 		ts = obj;
-		trx = ts->trx;
 		osmo_fsm_inst_dispatch(ts->mo.fi, NM_EV_STATE_CHG_REP, nsd);
 		break;
 	case NM_OC_RADIO_CARRIER:
@@ -184,9 +184,6 @@ static int nm_statechg_event(int evt, struct nm_statechg_signal_data *nsd)
 		break;
 	case NM_OC_GPRS_NSVC:
 		nsvc = obj;
-		/* We skip NSVC1 since we only use NSVC0 */
-		if (nsvc->id == 1)
-			break;
 		osmo_fsm_inst_dispatch(nsvc->mo.fi, NM_EV_STATE_CHG_REP, nsd);
 		break;
 	default:
@@ -412,19 +409,15 @@ static void nm_rx_set_chan_attr_ack(struct msgb *oml_msg)
 	osmo_fsm_inst_dispatch(ts->mo.fi, NM_EV_SET_ATTR_ACK, NULL);
 }
 
-static void nm_rx_ipacc_set_attr_ack(struct msgb *oml_msg)
+static void nm_rx_ipacc_set_attr_ack(struct ipacc_ack_signal_data *sig_data)
 {
-	struct e1inp_sign_link *sign_link = oml_msg->dst;
-	struct gsm_bts *bts = sign_link->trx->bts;
-	struct abis_om_hdr *oh = msgb_l2(oml_msg);
-	uint8_t idstrlen = oh->data[0];
-	struct abis_om_fom_hdr *foh;
+	struct gsm_bts *bts = sig_data->bts;
+	struct abis_om_fom_hdr *foh = sig_data->foh;
 	void *obj;
 	struct gsm_gprs_nse *nse;
 	struct gsm_gprs_cell *cell;
 	struct gsm_gprs_nsvc *nsvc;
 
-	foh = (struct abis_om_fom_hdr *) (oh->data + 1 + idstrlen);
 	obj = gsm_objclass2obj(bts, foh->obj_class, &foh->obj_inst);
 
 	switch (foh->obj_class) {
@@ -446,6 +439,61 @@ static void nm_rx_ipacc_set_attr_ack(struct msgb *oml_msg)
 	}
 }
 
+static void nm_rx_ipacc_rsl_connect_ack(struct ipacc_ack_signal_data *sig_data)
+{
+	struct gsm_bts *bts = sig_data->bts;
+	struct abis_om_fom_hdr *foh = sig_data->foh;
+	struct gsm_bts_trx *trx;
+
+	if (foh->obj_class != NM_OC_BASEB_TRANSC) {
+		LOGPFOH(DNM, LOGL_ERROR, foh, "IPACC RSL Connect ACK received on incorrect object class %d!\n", foh->obj_class);
+		return;
+	}
+
+	trx = gsm_bts_trx_num(bts, foh->obj_inst.trx_nr);
+	osmo_fsm_inst_dispatch(trx->bb_transc.mo.fi, NM_EV_RSL_CONNECT_ACK, NULL);
+}
+
+static void nm_rx_ipacc_ack(struct ipacc_ack_signal_data *sig_data)
+{
+	switch (sig_data->foh->msg_type) {
+	case NM_MT_IPACC_SET_ATTR_ACK:
+		nm_rx_ipacc_set_attr_ack(sig_data);
+		break;
+	case NM_MT_IPACC_RSL_CONNECT_ACK:
+		nm_rx_ipacc_rsl_connect_ack(sig_data);
+		break;
+	default:
+		break;
+	}
+}
+
+static void nm_rx_ipacc_rsl_connect_nack(struct ipacc_ack_signal_data *sig_data)
+{
+	struct gsm_bts *bts = sig_data->bts;
+	struct abis_om_fom_hdr *foh = sig_data->foh;
+	struct gsm_bts_trx *trx;
+
+	if (foh->obj_class != NM_OC_BASEB_TRANSC) {
+		LOGPFOH(DNM, LOGL_ERROR, foh, "IPACC RSL Connect NACK received on incorrect object class %d!\n", foh->obj_class);
+		return;
+	}
+
+	trx = gsm_bts_trx_num(bts, foh->obj_inst.trx_nr);
+	osmo_fsm_inst_dispatch(trx->bb_transc.mo.fi, NM_EV_RSL_CONNECT_NACK, NULL);
+}
+
+static void nm_rx_ipacc_nack(struct ipacc_ack_signal_data *sig_data)
+{
+	switch (sig_data->foh->msg_type) {
+	case NM_MT_IPACC_RSL_CONNECT_ACK:
+		nm_rx_ipacc_rsl_connect_nack(sig_data);
+		break;
+	default:
+		break;
+	}
+}
+
 /* Callback function to be called every time we receive a signal from NM */
 static int bts_ipa_nm_sig_cb(unsigned int subsys, unsigned int signal,
 		     void *handler_data, void *signal_data)
@@ -456,8 +504,7 @@ static int bts_ipa_nm_sig_cb(unsigned int subsys, unsigned int signal,
 	switch (signal) {
 	case S_NM_SW_ACTIV_REP:
 		return sw_activ_rep(signal_data);
-	case S_NM_STATECHG_OPER:
-	case S_NM_STATECHG_ADM:
+	case S_NM_STATECHG:
 		return nm_statechg_event(signal, signal_data);
 	case S_NM_OPSTART_ACK:
 		nm_rx_opstart_ack(signal_data);
@@ -477,8 +524,11 @@ static int bts_ipa_nm_sig_cb(unsigned int subsys, unsigned int signal,
 	case S_NM_SET_CHAN_ATTR_ACK:
 		nm_rx_set_chan_attr_ack(signal_data);
 		return 0;
-	case S_NM_IPACC_SET_ATTR_ACK:
-		nm_rx_ipacc_set_attr_ack(signal_data);
+	case S_NM_IPACC_ACK:
+		nm_rx_ipacc_ack(signal_data);
+		return 0;
+	case S_NM_IPACC_NACK:
+		nm_rx_ipacc_nack(signal_data);
 		return 0;
 	default:
 		break;
@@ -562,9 +612,12 @@ void ipaccess_drop_oml(struct gsm_bts *bts, const char *reason)
 	gsm_bts_stats_reset(bts);
 
 	/* Also drop the associated OSMO link */
+	OSMO_ASSERT(bts->osmo_link);
 	e1inp_sign_link_destroy(bts->osmo_link);
 	bts->osmo_link = NULL;
 
+	bts_setup_ramp_remove(bts);
+
 	/* we have issues reconnecting RSL, drop everything. */
 	llist_for_each_entry(trx, &bts->trx_list, list) {
 		ipaccess_drop_rsl(trx, "OML link drop");
@@ -576,13 +629,14 @@ void ipaccess_drop_oml(struct gsm_bts *bts, const char *reason)
 		}
 	}
 
+	osmo_fsm_inst_dispatch(bts->mo.fi, NM_EV_OML_DOWN, NULL);
+	osmo_fsm_inst_dispatch(bts->gprs.cell.mo.fi, NM_EV_OML_DOWN, NULL);
+
 	osmo_fsm_inst_dispatch(bts->site_mgr->mo.fi, NM_EV_OML_DOWN, NULL);
 	osmo_fsm_inst_dispatch(bts->site_mgr->gprs.nse.mo.fi, NM_EV_OML_DOWN, NULL);
 	for (i = 0; i < ARRAY_SIZE(bts->site_mgr->gprs.nsvc); i++)
 		osmo_fsm_inst_dispatch(bts->site_mgr->gprs.nsvc[i].mo.fi, NM_EV_OML_DOWN, NULL);
 
-	osmo_fsm_inst_dispatch(bts->mo.fi, NM_EV_OML_DOWN, NULL);
-	osmo_fsm_inst_dispatch(bts->gprs.cell.mo.fi, NM_EV_OML_DOWN, NULL);
 	gsm_bts_all_ts_dispatch(bts, TS_EV_OML_DOWN, NULL);
 
 	bts->ip_access.flags = 0;
@@ -696,6 +750,14 @@ ipaccess_sign_link_up(void *unit_data, struct e1inp_line *line,
 	DEBUGP(DLINP, "%s: Identified BTS %u/%u/%u\n", e1inp_signtype_name(type),
 			dev->site_id, dev->bts_id, dev->trx_id);
 
+	/* Check if this BTS has a valid configuration. If not we will drop it
+	 * immediately. */
+	if (gsm_bts_check_cfg(bts) != 0) {
+		LOGP(DLINP, LOGL_NOTICE, "(bts=%u) BTS config invalid, dropping BTS!\n", bts->nr);
+		ipaccess_drop_oml_deferred(bts);
+		return NULL;
+	}
+
 	switch(type) {
 	case E1INP_SIGN_OML:
 		/* remove old OML signal link for this BTS. */
