@@ -204,8 +204,7 @@ static int nm_sig_cb(unsigned int subsys, unsigned int signal,
 		return 0;
 
 	switch (signal) {
-	case S_NM_STATECHG_OPER:
-	case S_NM_STATECHG_ADM:
+	case S_NM_STATECHG:
 		nm_statechg_evt(signal, signal_data);
 		break;
 	default:
@@ -798,6 +797,9 @@ static int make_fu_config(struct gsm_bts_trx *trx, uint8_t id,
 		case GSM_PCHAN_CCCH_SDCCH4:
 			chan_config = 1;
 			break;
+		case GSM_PCHAN_CCCH_SDCCH4_CBCH:
+			chan_config = 3;
+			break;
 		case GSM_PCHAN_TCH_F:
 			chan_config = 6;	/* 9 should work too */
 			break;
@@ -807,6 +809,9 @@ static int make_fu_config(struct gsm_bts_trx *trx, uint8_t id,
 		case GSM_PCHAN_SDCCH8_SACCH8C:
 			chan_config = 4;
 			break;
+		case GSM_PCHAN_SDCCH8_SACCH8C_CBCH:
+			chan_config = 5;
+			break;
 		case GSM_PCHAN_PDCH:
 			chan_config = 11;
 			break;
@@ -1759,14 +1764,6 @@ static struct gsm_network *my_net;
 
 static int bts_model_nokia_site_start(struct gsm_network *net)
 {
-	model_nokia_site.features.data = &model_nokia_site._features_data[0];
-	model_nokia_site.features.data_len =
-	    sizeof(model_nokia_site._features_data);
-
-	osmo_bts_set_feature(&model_nokia_site.features, BTS_FEAT_HOPPING);
-	osmo_bts_set_feature(&model_nokia_site.features, BTS_FEAT_HSCSD);
-        osmo_bts_set_feature(&model_nokia_site.features, BTS_FEAT_MULTI_TSC);
-
 	osmo_signal_register_handler(SS_L_INPUT, inp_sig_cb, NULL);
 	osmo_signal_register_handler(SS_L_GLOBAL, gbl_sig_cb, NULL);
 	osmo_signal_register_handler(SS_NM, nm_sig_cb, NULL);
@@ -1778,5 +1775,12 @@ static int bts_model_nokia_site_start(struct gsm_network *net)
 
 int bts_model_nokia_site_init(void)
 {
+	model_nokia_site.features.data = &model_nokia_site._features_data[0];
+	model_nokia_site.features.data_len = sizeof(model_nokia_site._features_data);
+
+	osmo_bts_set_feature(&model_nokia_site.features, BTS_FEAT_HOPPING);
+	osmo_bts_set_feature(&model_nokia_site.features, BTS_FEAT_HSCSD);
+	osmo_bts_set_feature(&model_nokia_site.features, BTS_FEAT_MULTI_TSC);
+
 	return gsm_bts_model_register(&model_nokia_site);
 }
