@@ -31,6 +31,7 @@
 #include <osmocom/bsc/pcu_if.h>
 #include <osmocom/bsc/debug.h>
 #include <osmocom/bsc/nm_common_fsm.h>
+#include <osmocom/bsc/lchan.h>
 
 static int gsm_bts_trx_talloc_destructor(struct gsm_bts_trx *trx)
 {
@@ -107,14 +108,8 @@ struct gsm_bts_trx *gsm_bts_trx_alloc(struct gsm_bts *bts)
 		ts->hopping.ma.data = ts->hopping.ma_data;
 
 		for (l = 0; l < TS_MAX_LCHAN; l++) {
-			struct gsm_lchan *lchan;
-			lchan = &ts->lchan[l];
-
-			lchan->ts = ts;
-			lchan->nr = l;
-			lchan->type = GSM_LCHAN_NONE;
-
-			lchan_update_name(lchan);
+			struct gsm_lchan *lchan = &ts->lchan[l];
+			lchan_init(lchan, ts, l);
 		}
 	}
 
@@ -331,6 +326,23 @@ bool trx_has_valid_pchan_config(const struct gsm_bts_trx *trx)
 			}
 			break;
 
+		case GSM_PCHAN_PDCH:
+			if (is_ericsson_bts(trx->bts)) {
+				/* NOTE: A static PDCH is usually handled by the BTS/PCU internally, the BSC
+				 * will not actively manage this channel. It will just keep the timeslot
+				 * unused so that it is free for the BTS/PCU to use it as PDCH. Not all BTSs
+				 * work well in this scheme. Ericsson RBS BTSs support dynamic channels natively
+				 * and require a channel activation on RSL level before the PDCH can be used.
+				 * One could work around this by activating the PDCH once on startup and
+				 * leave it on indefinetly but we decided not to do so. Users of Ericsson RBS
+				 * BTSs must configure a dynamic PDCH channel. */
+				LOGP(DNM, LOGL_ERROR, "%s is not allowed, because Ericsson RBS does"
+				     "not support static PDCH (use TCH/F_TCH/H_SDCCH8_PDCH)\n",
+				     gsm_pchan_name(ts->pchan_from_config));
+				result = false;
+			}
+			break;
+
 		default:
 			/* CCCH on TS0 is mandatory for C0 */
 			if (trx->bts->c0 == trx && i == 0) {
@@ -338,6 +350,20 @@ bool trx_has_valid_pchan_config(const struct gsm_bts_trx *trx)
 				result = false;
 			}
 		}
+
+		if (trx->bts->features_known) {
+			const struct bitvec *ft = &trx->bts->features;
+
+			if (ts->hopping.enabled && !osmo_bts_has_feature(ft, BTS_FEAT_HOPPING)) {
+				LOGP(DNM, LOGL_ERROR, "TS%d has freq. hopping enabled, but BTS does not support it\n", i);
+				result = false;
+			}
+
+			if (ts->tsc != -1 && !osmo_bts_has_feature(ft, BTS_FEAT_MULTI_TSC)) {
+				LOGP(DNM, LOGL_ERROR, "TS%d has TSC != BCC, but BTS does not support it\n", i);
+				result = false;
+			}
+		}
 	}
 
 	return result;
@@ -382,7 +408,7 @@ static int rsl_si(struct gsm_bts_trx *trx, enum osmo_sysinfo_type i, int si_len)
 /* set all system information types for a TRX */
 int gsm_bts_trx_set_system_infos(struct gsm_bts_trx *trx)
 {
-	int i, rc;
+	int rc;
 	struct gsm_bts *bts = trx->bts;
 	uint8_t gen_si[_MAX_SYSINFO_TYPE], n_si = 0, n;
 	int si_len[_MAX_SYSINFO_TYPE];
@@ -420,42 +446,52 @@ int gsm_bts_trx_set_system_infos(struct gsm_bts_trx *trx)
 	/* Second, we generate the selected SI via RSL */
 
 	for (n = 0; n < n_si; n++) {
-		i = gen_si[n];
+		const enum osmo_sysinfo_type si_type = gen_si[n];
+
 		/* Only generate SI if this SI is not in "static" (user-defined) mode */
-		if (!(bts->si_mode_static & (1 << i))) {
+		if (!(bts->si_mode_static & (1 << si_type))) {
 			/* Set SI as being valid. gsm_generate_si() might unset
 			 * it, if SI is not required. */
-			bts->si_valid |= (1 << i);
-			rc = gsm_generate_si(bts, i);
+			bts->si_valid |= (1 << si_type);
+			rc = gsm_generate_si(bts, si_type);
 			if (rc < 0)
 				goto err_out;
-			si_len[i] = rc;
+			si_len[si_type] = rc;
 		} else {
-			if (i == SYSINFO_TYPE_5 || i == SYSINFO_TYPE_5bis
-			 || i == SYSINFO_TYPE_5ter)
-				si_len[i] = 18;
-			else if (i == SYSINFO_TYPE_6)
-				si_len[i] = 11;
-			else
-				si_len[i] = 23;
+			switch (si_type) {
+			case SYSINFO_TYPE_5:
+			case SYSINFO_TYPE_5bis:
+			case SYSINFO_TYPE_5ter:
+				si_len[si_type] = 18;
+				break;
+			case SYSINFO_TYPE_6:
+				si_len[si_type] = 11;
+				break;
+			case SYSINFO_TYPE_10:
+				si_len[si_type] = 21;
+				break;
+			default:
+				si_len[si_type] = GSM_MACBLOCK_LEN;
+			}
 		}
 	}
 
 	/* Third, we send the selected SI via RSL */
 
 	for (n = 0; n < n_si; n++) {
-		i = gen_si[n];
+		const enum osmo_sysinfo_type si_type = gen_si[n];
+
 		/* 3GPP TS 08.58 §8.5.1 BCCH INFORMATION. If we don't currently
 		 * have this SI, we send a zero-length RSL BCCH FILLING /
 		 * SACCH FILLING in order to deactivate the SI, in case it
 		 * might have previously been active */
-		if (!GSM_BTS_HAS_SI(bts, i)) {
+		if (!GSM_BTS_HAS_SI(bts, si_type)) {
 			if (bts->si_unused_send_empty)
-				rc = rsl_si(trx, i, 0);
+				rc = rsl_si(trx, si_type, 0);
 			else
 				rc = 0; /* some nanoBTS fw don't like receiving empty unsupported SI */
 		} else
-			rc = rsl_si(trx, i, si_len[i]);
+			rc = rsl_si(trx, si_type, si_len[si_type]);
 		if (rc < 0)
 			return rc;
 	}
@@ -468,6 +504,6 @@ int gsm_bts_trx_set_system_infos(struct gsm_bts_trx *trx)
 err_out:
 	LOGP(DRR, LOGL_ERROR, "Cannot generate SI%s for BTS %u: error <%s>, "
 	     "most likely a problem with neighbor cell list generation\n",
-	     get_value_string(osmo_sitype_strs, i), bts->nr, strerror(-rc));
+	     get_value_string(osmo_sitype_strs, gen_si[n]), bts->nr, strerror(-rc));
 	return rc;
 }
