@@ -39,6 +39,7 @@
 #include <osmocom/bsc/system_information.h>
 #include <osmocom/bsc/debug.h>
 #include <osmocom/bsc/timeslot_fsm.h>
+#include <osmocom/bsc/lchan.h>
 #include <osmocom/bsc/lchan_fsm.h>
 #include <osmocom/bsc/lchan_select.h>
 #include <osmocom/bsc/bts.h>
@@ -316,8 +317,9 @@ DEFUN_USRATTR(cfg_ts_tsc,
 	      "Training Sequence Code of the Timeslot\n" "TSC\n")
 {
 	struct gsm_bts_trx_ts *ts = vty->index;
+	const struct gsm_bts *bts = ts->trx->bts;
 
-	if (!osmo_bts_has_feature(&ts->trx->bts->model->features, BTS_FEAT_MULTI_TSC)) {
+	if (bts->features_known && !osmo_bts_has_feature(&bts->features, BTS_FEAT_MULTI_TSC)) {
 		vty_out(vty, "%% This BTS does not support a TSC != BCC, "
 			"falling back to BCC%s", VTY_NEWLINE);
 		ts->tsc = -1;
@@ -339,13 +341,12 @@ DEFUN_USRATTR(cfg_ts_hopping,
 	      "Disable frequency hopping\n" "Enable frequency hopping\n")
 {
 	struct gsm_bts_trx_ts *ts = vty->index;
+	const struct gsm_bts *bts = ts->trx->bts;
 	int enabled = atoi(argv[0]);
 
-	if (enabled && !osmo_bts_has_feature(&ts->trx->bts->model->features, BTS_FEAT_HOPPING)) {
-		vty_out(vty, "%% BTS model does not seem to support freq. hopping%s", VTY_NEWLINE);
-		/* Allow enabling frequency hopping anyway, because the BTS might not have
-		 * connected yet (thus not sent the feature vector), so we cannot know for
-		 * sure.  Jet print a warning and let it go. */
+	if (enabled && bts->features_known && !osmo_bts_has_feature(&bts->features, BTS_FEAT_HOPPING)) {
+		vty_out(vty, "%% BTS does not support freq. hopping%s", VTY_NEWLINE);
+		return CMD_WARNING;
 	}
 
 	ts->hopping.enabled = enabled;
@@ -401,7 +402,7 @@ DEFUN_USRATTR(cfg_ts_arfcn_add,
 
 	if (bitvec_get_bit_pos(&ts->hopping.arfcns, arfcn) == ONE) {
 		vty_out(vty, "%% ARFCN %" PRIu16 " is already set%s", arfcn, VTY_NEWLINE);
-		return CMD_WARNING;
+		return CMD_SUCCESS;
 	}
 
 	bitvec_set_bit_pos(&ts->hopping.arfcns, arfcn, 1);
@@ -568,7 +569,14 @@ void lchan_dump_full_vty(struct vty *vty, struct gsm_lchan *lchan)
 
 	vty_out(vty, "BTS %u, TRX %u, Timeslot %u, Lchan %u: Type %s%s",
 		lchan->ts->trx->bts->nr, lchan->ts->trx->nr, lchan->ts->nr,
-		lchan->nr, gsm_lchant_name(lchan->type), VTY_NEWLINE);
+		lchan->nr, gsm_chan_t_name(lchan->type), VTY_NEWLINE);
+
+	if (lchan->activate.concluded) {
+		vty_out(vty, "  Active for: %s seconds%s",
+			osmo_int_to_float_str_c(OTC_SELECT, gsm_lchan_active_duration_ms(lchan), 3),
+			VTY_NEWLINE);
+	}
+
 	vty_out_dyn_ts_details(vty, lchan->ts);
 	vty_out(vty, "  Connection: %u, State: %s%s%s%s",
 		lchan->conn ? 1: 0, lchan_state_name(lchan),
@@ -598,25 +606,35 @@ void lchan_dump_full_vty(struct vty *vty, struct gsm_lchan *lchan)
 	if (lchan->conn && lchan->conn->bsub) {
 		vty_out(vty, "  Subscriber:%s", VTY_NEWLINE);
 		bsc_subscr_dump_vty(vty, lchan->conn->bsub);
-	} else
+	} else {
 		vty_out(vty, "  No Subscriber%s", VTY_NEWLINE);
+	}
 	if (is_ipaccess_bts(lchan->ts->trx->bts)) {
 		struct in_addr ia;
 		if (lchan->abis_ip.bound_ip) {
 			ia.s_addr = htonl(lchan->abis_ip.bound_ip);
-			vty_out(vty, "  Bound IP: %s Port %u RTP_TYPE2=%u CONN_ID=%u%s",
+			vty_out(vty, "  Bound IP: %s Port %u CONN_ID=%u",
 				inet_ntoa(ia), lchan->abis_ip.bound_port,
-				lchan->abis_ip.rtp_payload2, lchan->abis_ip.conn_id,
-				VTY_NEWLINE);
+				lchan->abis_ip.conn_id);
+			if (lchan->abis_ip.osmux.use)
+				vty_out(vty, " Osmux_CID=%u%s", lchan->abis_ip.osmux.local_cid, VTY_NEWLINE);
+			else
+				vty_out(vty, " RTP_TYPE2=%u%s", lchan->abis_ip.rtp_payload2, VTY_NEWLINE);
 		}
 		if (lchan->abis_ip.connect_ip) {
 			ia.s_addr = htonl(lchan->abis_ip.connect_ip);
-			vty_out(vty, "  Conn. IP: %s Port %u RTP_TYPE=%u SPEECH_MODE=0x%02x%s",
+			vty_out(vty, "  Conn. IP: %s Port %u SPEECH_MODE=0x%02x",
 				inet_ntoa(ia), lchan->abis_ip.connect_port,
-				lchan->abis_ip.rtp_payload, lchan->abis_ip.speech_mode,
-				VTY_NEWLINE);
+				lchan->abis_ip.speech_mode);
+			if (lchan->abis_ip.osmux.use) {
+				if (lchan->abis_ip.osmux.remote_cid_present)
+					vty_out(vty, " Osmux_CID=%u%s", lchan->abis_ip.osmux.remote_cid, VTY_NEWLINE);
+				else
+					vty_out(vty, " Osmux_CID=?%s", VTY_NEWLINE);
+			} else {
+				vty_out(vty, " RTP_TYPE=%u%s", lchan->abis_ip.rtp_payload, VTY_NEWLINE);
+			}
 		}
-
 	}
 
 	/* we want to report the last measurement report */
@@ -654,7 +672,7 @@ void lchan_dump_short_vty(struct vty *vty, struct gsm_lchan *lchan)
 	}
 
 	vty_out(vty, ", Type %s%s TSC-s%dc%u, State %s - L1 MS Power: %u dBm RXL-FULL-dl: %4d dBm RXL-FULL-ul: %4d dBm%s",
-		gsm_lchant_name(lchan->type),
+		gsm_chan_t_name(lchan->type),
 		lchan->vamos.enabled ? " (VAMOS)" : "",
 		lchan->tsc_set > 0 ? lchan->tsc_set : 1,
 		lchan->tsc,
@@ -731,8 +749,10 @@ void trx_dump_vty(struct vty *vty, struct gsm_bts_trx *trx, bool print_rsl, bool
 	net_dump_nmstate(vty, &trx->mo.nm_state);
 	if (print_rsl)
 		vty_out(vty, "  RSL State: %s%s", trx->rsl_link_primary? "connected" : "disconnected", VTY_NEWLINE);
-	vty_out(vty, "  Baseband Transceiver NM State: ");
-	net_dump_nmstate(vty, &trx->bb_transc.mo.nm_state);
+
+	vty_out(vty, "  %sBaseband Transceiver NM State: ", is_ericsson_bts(trx->bts) ? "[Virtual] " : "");
+		net_dump_nmstate(vty, &trx->bb_transc.mo.nm_state);
+
 	if (is_ipaccess_bts(trx->bts)) {
 		vty_out(vty, "  ip.access stream ID: 0x%02x ", trx->rsl_tei_primary);
 		e1isl_dump_vty_tcp(vty, trx->rsl_link_primary);
@@ -740,6 +760,11 @@ void trx_dump_vty(struct vty *vty, struct gsm_bts_trx *trx, bool print_rsl, bool
 		vty_out(vty, "  E1 Signalling Link:%s", VTY_NEWLINE);
 		e1isl_dump_vty(vty, trx->rsl_link_primary);
 	}
+
+	const struct load_counter *ll = &trx->lchan_load;
+	vty_out(vty, "  Channel load: %u%%%s",
+		ll->total ? ll->used * 100 / ll->total : 0,
+		VTY_NEWLINE);
 }
 
 void config_write_e1_link(struct vty *vty, struct gsm_e1_subslot *e1_link,
