@@ -71,12 +71,14 @@ void bsc_sapi_n_reject(struct gsm_subscriber_connection *conn,
 {
 	int rc;
 	struct msgb *resp;
+	struct gsm_bts *bts;
 
 	if (!msc_connected(conn))
 		return;
 
-	LOGP(DMSC, LOGL_NOTICE, "Tx MSC SAPI N REJECT (dlci=0x%02x, cause='%s')\n",
-	     dlci, gsm0808_cause_name(cause));
+	bts = conn_get_bts(conn);
+	LOG_BTS(bts, DMSC, LOGL_NOTICE, "Tx MSC SAPI N REJECT (dlci=0x%02x, cause='%s')\n",
+		dlci, gsm0808_cause_name(cause));
 	resp = gsm0808_create_sapi_reject_cause(dlci, cause);
 	rate_ctr_inc(rate_ctr_group_get_ctr(conn->sccp.msc->msc_ctrs, MSC_CTR_BSSMAP_TX_DT1_SAPI_N_REJECT));
 	rc = osmo_fsm_inst_dispatch(conn->fi, GSCON_EV_TX_SCCP, resp);
@@ -85,7 +87,7 @@ void bsc_sapi_n_reject(struct gsm_subscriber_connection *conn,
 }
 
 /*! MS->MSC: Tell MSC that ciphering has been enabled. */
-void bsc_cipher_mode_compl(struct gsm_subscriber_connection *conn, struct msgb *msg, uint8_t chosen_encr)
+void bsc_cipher_mode_compl(struct gsm_subscriber_connection *conn, struct msgb *msg, uint8_t chosen_a5_n)
 {
 	int rc;
 	struct msgb *resp;
@@ -94,7 +96,7 @@ void bsc_cipher_mode_compl(struct gsm_subscriber_connection *conn, struct msgb *
 		return;
 
 	LOGP(DMSC, LOGL_DEBUG, "CIPHER MODE COMPLETE from MS, forwarding to MSC\n");
-	resp = gsm0808_create_cipher_complete(msg, chosen_encr);
+	resp = gsm0808_create_cipher_complete(msg, ALG_A5_NR_TO_BSSAP(chosen_a5_n));
 	rate_ctr_inc(rate_ctr_group_get_ctr(conn->sccp.msc->msc_ctrs, MSC_CTR_BSSMAP_TX_DT1_CIPHER_COMPLETE));
 	rc = osmo_fsm_inst_dispatch(conn->fi, GSCON_EV_TX_SCCP, resp);
 	if (rc != 0)
@@ -387,11 +389,10 @@ int bsc_compl_l3(struct gsm_lchan *lchan, struct msgb *msg, uint16_t chosen_chan
 	if (osmo_mobile_identity_decode_from_l3(&mi, msg, false)) {
 		LOG_COMPL_L3(pdisc, mtype, LOGL_ERROR, "Cannot extract Mobile Identity: %s\n",
 			     msgb_hexdump_c(OTC_SELECT, msg));
-		/* Likely this is an invalid Complete Layer 3 message that deserves to be rejected. However, the current
-		 * state of our ttcn3 tests does send invalid Layer 3 Info in some tests and expects osmo-bsc to not
-		 * care about that. So, changing the behavior to rejecting on missing MI causes test failure and, if at
-		 * all, should happen in a separate patch.
-		 * See e.g.  BSC_Tests.TC_chan_rel_rll_rel_ind: "dt := * f_est_dchan('23'O, 23, '00010203040506'O);"
+		/* Likely this is an invalid Complete Layer 3 message that deserves to be rejected. However, the BSC is
+		 * not expected to look at this layer so it's duty of the MSC to reject it.
+		 * Hence, keep on going with the conn without an assigned bsc_subscr, forwarding the L3 to the MSC and
+		 * letting it take decision on the matter.
 		 */
 	} else {
 		bsub = bsc_subscr_find_or_create_by_mi(bsc_gsmnet->bsc_subscribers, &mi, __func__);
@@ -438,7 +439,10 @@ int bsc_compl_l3(struct gsm_lchan *lchan, struct msgb *msg, uint16_t chosen_chan
 	paged_from_msc = NULL;
 	paging_reasons = BSC_PAGING_NONE;
 	if (pdisc == GSM48_PDISC_RR && mtype == GSM48_MT_RR_PAG_RESP) {
-		paging_request_stop(&paged_from_msc, &paging_reasons, bts, conn->bsub);
+		/* It only makes sense to attempt to find a pending paging request if the subscriber from the
+		 * Paging Response can be identified (bsub != NULL). */
+		if (conn->bsub)
+			paging_request_stop(&paged_from_msc, &paging_reasons, bts, conn->bsub);
 		if (!paged_from_msc) {
 			/* This looks like an unsolicited Paging Response. It is required to pick any MSC, because any
 			 * MT-CSFB calls were Paged by the MSC via SGs, and hence are not listed in the BSC. */
@@ -536,15 +540,6 @@ early_exit:
 	return rc;
 }
 
-/* Data Link Connection Identifier (DLCI) is defined in 3GPP TS 48.006, section 9.3.2.
- * .... .SSS - SAPI value used on the radio link;
- * CC.. .... - control channel identification:
- *   00.. .... - indicates that the control channel is not further specified,
- *   10.. .... - represents the FACCH or the SDCCH,
- *   11.. .... - represents the SACCH,
- *   other values are reserved. */
-#define RSL_LINK_ID2DLCI(link_id) \
-	(link_id & 0x40 ? 0xc0 : 0x80) | (link_id & 0x07)
 
 /*! MS->BSC/MSC: Um L3 message. */
 void bsc_dtap(struct gsm_subscriber_connection *conn, uint8_t link_id, struct msgb *msg)
