@@ -46,6 +46,7 @@
 #include <osmocom/bsc/bts.h>
 #include <osmocom/bsc/lcs_loc_req.h>
 #include <osmocom/bsc/bsc_stats.h>
+#include <osmocom/bsc/lchan.h>
 
 #define LOG_FMT_BTS "bts %u lac-ci %u-%u arfcn-bsic %d-%d"
 #define LOG_ARGS_BTS(bts) \
@@ -60,7 +61,7 @@
 		lchan ? lchan->ts->trx->bts->nr : 0, \
 		lchan ? lchan->ts->trx->nr : 0, \
 		lchan ? lchan->ts->nr : 0, \
-		lchan ? gsm_lchant_name(lchan->type) : "?", \
+		lchan ? gsm_chan_t_name(lchan->type) : "?", \
 		lchan ? lchan->nr : 0, \
 		lchan ? gsm48_chan_mode_name(lchan->current_ch_mode_rate.chan_mode) : "?"
 
@@ -138,7 +139,7 @@ const char *handover_status(struct gsm_subscriber_connection *conn)
 				 "("LOG_FMT_FROM_LCHAN") --HO-> ("LOG_FMT_BTS",%s) " LOG_FMT_HO_SCOPE,
 				 LOG_ARGS_FROM_LCHAN(conn->lchan),
 				 LOG_ARGS_BTS(ho->new_bts),
-				 gsm_lchant_name(ho->new_lchan_type),
+				 gsm_chan_t_name(ho->new_lchan_type),
 				 LOG_ARGS_HO_SCOPE(conn));
 		else
 			snprintf(buf, sizeof(buf),
@@ -167,14 +168,14 @@ const char *handover_status(struct gsm_subscriber_connection *conn)
 				 ho->inter_bsc_in.cell_id_serving_name,
 				 ho->inter_bsc_in.cell_id_target_name,
 				 LOG_ARGS_BTS(ho->new_bts),
-				 gsm_lchant_name(ho->new_lchan_type),
+				 gsm_chan_t_name(ho->new_lchan_type),
 				 LOG_ARGS_HO_SCOPE(conn));
 		else
 			snprintf(buf, sizeof(buf),
 				 "(remote:%s) --HO-> (local:%s,%s) " LOG_FMT_HO_SCOPE,
 				 ho->inter_bsc_in.cell_id_serving_name,
 				 ho->inter_bsc_in.cell_id_target_name,
-				 gsm_lchant_name(ho->new_lchan_type),
+				 gsm_chan_t_name(ho->new_lchan_type),
 				 LOG_ARGS_HO_SCOPE(conn));
 	} else
 		snprintf(buf, sizeof(buf), LOG_FMT_HO_SCOPE, LOG_ARGS_HO_SCOPE(conn));
@@ -292,7 +293,7 @@ static void handover_reset(struct gsm_subscriber_connection *conn)
 	};
 }
 
-void handover_fsm_init()
+void handover_fsm_init(void)
 {
 	OSMO_ASSERT(osmo_fsm_register(&ho_fsm) == 0);
 }
@@ -379,7 +380,10 @@ static void handover_start_intra_bsc(struct gsm_subscriber_connection *conn)
 	ho->async = true;
 	gsm_bts_cell_id_list(&ho->target_cell_ids, ho->new_bts);
 
-	ho->new_lchan = lchan_select_by_type(ho->new_bts, ho->new_lchan_type);
+	ho->new_lchan = lchan_select_by_type(ho->new_bts,
+					     ho->new_lchan_type,
+					     SELECT_FOR_HANDOVER,
+					     NULL);
 
 	if (ho->scope & HO_INTRA_CELL) {
 		ho_count(bts, CTR_INTRA_CELL_HO_ATTEMPTED);
@@ -393,7 +397,7 @@ static void handover_start_intra_bsc(struct gsm_subscriber_connection *conn)
 	if (!ho->new_lchan) {
 		ho_fail(HO_RESULT_FAIL_NO_CHANNEL,
 			"No %s lchan available on BTS %u",
-			gsm_lchant_name(ho->new_lchan_type), ho->new_bts->nr);
+			gsm_chan_t_name(ho->new_lchan_type), ho->new_bts->nr);
 		return;
 	}
 	LOG_HO(conn, LOGL_DEBUG, "Selected lchan %s\n", gsm_lchan_name(ho->new_lchan));
@@ -696,7 +700,10 @@ void handover_start_inter_bsc_in(struct gsm_subscriber_connection *conn,
 		       ch_mode_rate.chan_rate == CH_RATE_FULL ?  "full-rate" : "half-rate",
 		       gsm0808_channel_type_name(&req->ct));
 
-		lchan = lchan_select_by_chan_mode(bts, ch_mode_rate.chan_mode, ch_mode_rate.chan_rate);
+		lchan = lchan_select_by_chan_mode(bts,
+						  ch_mode_rate.chan_mode,
+						  ch_mode_rate.chan_rate,
+						  SELECT_FOR_HANDOVER, NULL);
 		if (!lchan) {
 			LOG_HO(conn, LOGL_DEBUG, "BTS %u has no matching free channels\n", bts->nr);
 			continue;
