@@ -41,6 +41,7 @@
 #include <osmocom/bsc/codec_pref.h>
 #include <osmocom/bsc/bts.h>
 #include <osmocom/bsc/bsc_stats.h>
+#include <osmocom/bsc/lchan.h>
 
 static struct osmo_fsm lchan_fsm;
 
@@ -281,15 +282,15 @@ static void lchan_on_fully_established(struct gsm_lchan *lchan)
 }
 
 struct osmo_tdef_state_timeout lchan_fsm_timeouts[32] = {
-	[LCHAN_ST_WAIT_TS_READY]	= { .T=-5 },
-	[LCHAN_ST_WAIT_ACTIV_ACK]	= { .T=-6 },
-	[LCHAN_ST_WAIT_RLL_RTP_ESTABLISH]	= { .T=3101 },
-	[LCHAN_ST_WAIT_RLL_RTP_RELEASED]	= { .T=3109 },
-	[LCHAN_ST_WAIT_BEFORE_RF_RELEASE]	= { .T=3111 },
-	[LCHAN_ST_WAIT_RF_RELEASE_ACK]	= { .T=3111 },
-	[LCHAN_ST_WAIT_AFTER_ERROR]	= { .T=-3111 },
-	[LCHAN_ST_WAIT_RR_CHAN_MODE_MODIFY_ACK]	= { .T=-13 },
-	[LCHAN_ST_WAIT_RSL_CHAN_MODE_MODIFY_ACK]	= { .T=-14 },
+	[LCHAN_ST_WAIT_TS_READY]	= { .T = -5 },
+	[LCHAN_ST_WAIT_ACTIV_ACK]	= { .T = -6 },
+	[LCHAN_ST_WAIT_RLL_RTP_ESTABLISH]	= { .T = 3101 },
+	[LCHAN_ST_WAIT_RLL_RTP_RELEASED]	= { .T = 3109 },
+	[LCHAN_ST_WAIT_BEFORE_RF_RELEASE]	= { .T = 3111 },
+	[LCHAN_ST_WAIT_RF_RELEASE_ACK]	= { .T = 3111 },
+	[LCHAN_ST_WAIT_AFTER_ERROR]	= { .T = -3111 },
+	[LCHAN_ST_WAIT_RR_CHAN_MODE_MODIFY_ACK]	= { .T = -13 },
+	[LCHAN_ST_WAIT_RSL_CHAN_MODE_MODIFY_ACK]	= { .T = -14 },
 };
 
 /* Transition to a state, using the T timer defined in lchan_fsm_timeouts.
@@ -323,7 +324,7 @@ struct osmo_tdef_state_timeout lchan_fsm_timeouts[32] = {
 	} while (0)
 
 /* Which state to transition to when lchan_fail() is called in a given state. */
-uint32_t lchan_fsm_on_error[34] = {
+uint32_t lchan_fsm_on_error[32] = {
 	[LCHAN_ST_UNUSED] 			= LCHAN_ST_UNUSED,
 	[LCHAN_ST_WAIT_TS_READY] 		= LCHAN_ST_UNUSED,
 	[LCHAN_ST_WAIT_ACTIV_ACK] 		= LCHAN_ST_BORKEN,
@@ -462,7 +463,7 @@ void lchan_fsm_update_id(struct gsm_lchan *lchan)
 
 extern void lchan_rtp_fsm_init();
 
-void lchan_fsm_init()
+void lchan_fsm_init(void)
 {
 	OSMO_ASSERT(osmo_fsm_register(&lchan_fsm) == 0);
 	lchan_rtp_fsm_init();
@@ -770,7 +771,7 @@ static void lchan_fsm_wait_ts_ready_onenter(struct osmo_fsm_inst *fi, uint32_t p
 		  lchan->activate.info.requires_voice_stream ?
 			(use_mgwep_ci ? osmo_mgcpc_ep_ci_name(use_mgwep_ci) : "new")
 			: "none",
-		  gsm_lchant_name(lchan->type),
+		  gsm_chan_t_name(lchan->type),
 		  gsm48_chan_mode_name(lchan->activate.ch_mode_rate.chan_mode),
 		  lchan->activate.info.encr.alg_a5_n,
 		  lchan->activate.info.encr.key_len ? osmo_hexdump_nospc(lchan->activate.info.encr.key,
@@ -1309,7 +1310,7 @@ static void lchan_fsm_established(struct osmo_fsm_inst *fi, uint32_t event, void
 			  lchan->modify.info.requires_voice_stream ?
 			  (use_mgwep_ci ? osmo_mgcpc_ep_ci_name(use_mgwep_ci) : "new")
 			  : "none",
-			  gsm_lchant_name(lchan->type),
+			  gsm_chan_t_name(lchan->type),
 			  gsm48_chan_mode_name(lchan->modify.ch_mode_rate.chan_mode),
 			  lchan->modify.tsc_set, lchan->modify.tsc);
 
@@ -1529,16 +1530,6 @@ static void lchan_fsm_borken(struct osmo_fsm_inst *fi, uint32_t event, void *dat
 		lchan->release.rsl_error_cause = RSL_ERR_INTERWORKING;
 		lchan->release.rr_cause = bsc_gsm48_rr_cause_from_rsl_cause(lchan->release.rsl_error_cause);
 		lchan_fsm_state_chg(LCHAN_ST_WAIT_AFTER_ERROR);
-		/* TODO: we used to do this only for sysmobts:
-			int do_free = is_osmobts(ts->trx->bts);
-			LOGP(DRSL, LOGL_NOTICE,
-				"%s CHAN REL ACK for broken channel. %s.\n",
-				gsm_lchan_name(lchan),
-				do_free ? "Releasing it" : "Keeping it broken");
-			if (do_free)
-				do_lchan_free(lchan);
-		 * Clarify the reason. If a BTS sends a RF Chan Rel ACK, we can consider it released,
-		 * independently from the BTS model, right?? */
 		return;
 
 	case LCHAN_EV_RTP_RELEASED:
