@@ -31,6 +31,7 @@
 #include <osmocom/bsc/abis_rsl.h>
 #include <osmocom/bsc/bsc_msc_data.h>
 #include <osmocom/bsc/bts.h>
+#include <osmocom/bsc/lchan.h>
 
 static struct osmo_fsm lchan_rtp_fsm;
 
@@ -43,10 +44,10 @@ struct gsm_lchan *lchan_rtp_fi_lchan(struct osmo_fsm_inst *fi)
 }
 
 struct osmo_tdef_state_timeout lchan_rtp_fsm_timeouts[32] = {
-	[LCHAN_RTP_ST_WAIT_MGW_ENDPOINT_AVAILABLE] = { .T=-9 },
-	[LCHAN_RTP_ST_WAIT_IPACC_CRCX_ACK]	= { .T=-7 },
-	[LCHAN_RTP_ST_WAIT_IPACC_MDCX_ACK]	= { .T=-8 },
-	[LCHAN_RTP_ST_WAIT_MGW_ENDPOINT_CONFIGURED] = { .T=-10 },
+	[LCHAN_RTP_ST_WAIT_MGW_ENDPOINT_AVAILABLE] = { .T = -9 },
+	[LCHAN_RTP_ST_WAIT_IPACC_CRCX_ACK]	= { .T = -7 },
+	[LCHAN_RTP_ST_WAIT_IPACC_MDCX_ACK]	= { .T = -8 },
+	[LCHAN_RTP_ST_WAIT_MGW_ENDPOINT_CONFIGURED] = { .T = -10 },
 };
 
 /* Transition to a state, using the T timer defined in lchan_rtp_fsm_timeouts.
@@ -69,7 +70,7 @@ struct osmo_tdef_state_timeout lchan_rtp_fsm_timeouts[32] = {
 	} while (0)
 
 /* Called from lchan_fsm_init(), does not need to be visible in lchan_rtp_fsm.h */
-void lchan_rtp_fsm_init()
+void lchan_rtp_fsm_init(void)
 {
 	OSMO_ASSERT(osmo_fsm_register(&lchan_rtp_fsm) == 0);
 }
@@ -137,9 +138,10 @@ static void lchan_rtp_fsm_wait_mgw_endpoint_available_onenter(struct osmo_fsm_in
 							      uint32_t prev_state)
 {
 	struct gsm_lchan *lchan = lchan_rtp_fi_lchan(fi);
+	struct gsm_bts *bts = lchan->ts->trx->bts;
 	struct osmo_mgcpc_ep *mgwep;
 	struct osmo_mgcpc_ep_ci *use_mgwep_ci = lchan_use_mgw_endpoint_ci_bts(lchan);
-	struct mgcp_conn_peer crcx_info = {};
+	struct mgcp_conn_peer crcx_info;
 
 	if (!is_ipaccess_bts(lchan->ts->trx->bts)) {
 		LOG_LCHAN_RTP(lchan, LOGL_DEBUG, "Audio link to-BTS via E1, skipping IPACC\n");
@@ -162,14 +164,36 @@ static void lchan_rtp_fsm_wait_mgw_endpoint_available_onenter(struct osmo_fsm_in
 
 	lchan->mgw_endpoint_ci_bts = osmo_mgcpc_ep_ci_add(mgwep, "to-BTS");
 
+	crcx_info = (struct mgcp_conn_peer){
+		.ptime = 20,
+		.x_osmo_osmux_cid = -1, /* -1 is wildcard, .x_osmo_osmux_use set below */
+	};
 	if (lchan->conn) {
 		crcx_info.call_id = lchan->conn->sccp.conn_id;
 		if (lchan->conn->sccp.msc)
 			crcx_info.x_osmo_ign = lchan->conn->sccp.msc->x_osmo_ign;
 	}
-	crcx_info.ptime = 20;
 	mgcp_pick_codec(&crcx_info, lchan, true);
 
+	/* Set up Osmux use in MGW according to configured policy */
+	bool amr_picked = mgcp_codec_is_picked(&crcx_info, CODEC_AMR_8000_1);
+	switch (bts->use_osmux) {
+	case OSMUX_USAGE_OFF:
+		crcx_info.x_osmo_osmux_use = false;
+		break;
+	case OSMUX_USAGE_ON:
+		crcx_info.x_osmo_osmux_use = amr_picked;
+		break;
+	case OSMUX_USAGE_ONLY:
+		if (!amr_picked) {
+			lchan_rtp_fail("Only AMR codec can be used when configured with policy 'osmux only'."
+				       " Check your configuration.");
+			return;
+		}
+		crcx_info.x_osmo_osmux_use = true;
+		break;
+	}
+
 	osmo_mgcpc_ep_ci_request(lchan->mgw_endpoint_ci_bts, MGCP_VERB_CRCX, &crcx_info,
 				fi, LCHAN_RTP_EV_MGW_ENDPOINT_AVAILABLE, LCHAN_RTP_EV_MGW_ENDPOINT_ERROR,
 				0);
@@ -178,11 +202,26 @@ static void lchan_rtp_fsm_wait_mgw_endpoint_available_onenter(struct osmo_fsm_in
 static void lchan_rtp_fsm_wait_mgw_endpoint_available(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
 	struct gsm_lchan *lchan = lchan_rtp_fi_lchan(fi);
+	struct gsm_bts *bts = lchan->ts->trx->bts;
 	switch (event) {
 
 	case LCHAN_RTP_EV_MGW_ENDPOINT_AVAILABLE:
 		LOG_LCHAN_RTP(lchan, LOGL_DEBUG, "MGW endpoint: %s\n",
 			      osmo_mgcpc_ep_ci_name(lchan_use_mgw_endpoint_ci_bts(lchan)));
+		if (osmo_mgcpc_ep_ci_get_crcx_info_to_osmux_cid(lchan->mgw_endpoint_ci_bts,
+								&lchan->abis_ip.osmux.local_cid)) {
+			if (bts->use_osmux == OSMUX_USAGE_OFF) {
+				lchan_rtp_fail("Got Osmux CID from MGW but we didn't ask for it");
+				return;
+			}
+			lchan->abis_ip.osmux.use = true;
+		} else {
+			if (bts->use_osmux == OSMUX_USAGE_ONLY) {
+				lchan_rtp_fail("Got no Osmux CID from MGW but Osmux is mandatory");
+				return;
+			}
+			lchan->abis_ip.osmux.use = false;
+		}
 		lchan_rtp_fsm_state_chg(LCHAN_RTP_ST_WAIT_LCHAN_READY);
 		return;
 
@@ -272,7 +311,7 @@ static void lchan_rtp_fsm_wait_ipacc_crcx_ack_onenter(struct osmo_fsm_inst *fi,
 	if (val < 0) {
 		lchan_rtp_fail("Cannot determine Abis/IP speech mode for tch_mode=%s type=%s",
 			   get_value_string(gsm48_chan_mode_names, lchan->activate.ch_mode_rate.chan_mode),
-			   gsm_lchant_name(lchan->type));
+			   gsm_chan_t_name(lchan->type));
 		return;
 	}
 	lchan->abis_ip.speech_mode = val;
@@ -281,7 +320,7 @@ static void lchan_rtp_fsm_wait_ipacc_crcx_ack_onenter(struct osmo_fsm_inst *fi,
 	if (val < 0) {
 		lchan_rtp_fail("Cannot determine Abis/IP payload type for tch_mode=%s type=%s",
 			   get_value_string(gsm48_chan_mode_names, lchan->activate.ch_mode_rate.chan_mode),
-			   gsm_lchant_name(lchan->type));
+			   gsm_chan_t_name(lchan->type));
 		return;
 	}
 	lchan->abis_ip.rtp_payload = val;
@@ -411,9 +450,16 @@ static void connect_mgw_endpoint_to_lchan(struct osmo_fsm_inst *fi,
 	struct in_addr addr;
 	const char *addr_str;
 
+	if (lchan->abis_ip.osmux.use && !lchan->abis_ip.osmux.remote_cid_present) {
+		lchan_rtp_fail("BTS didn't provide any remote Osmux CID for the call");
+		return;
+	}
+
 	mdcx_info = (struct mgcp_conn_peer){
 		.port = to_lchan->abis_ip.bound_port,
 		.ptime = 20,
+		.x_osmo_osmux_use = lchan->abis_ip.osmux.use,
+		.x_osmo_osmux_cid = lchan->abis_ip.osmux.remote_cid,
 	};
 	mgcp_pick_codec(&mdcx_info, to_lchan, true);
 
@@ -522,6 +568,12 @@ static void lchan_rtp_fsm_ready(struct osmo_fsm_inst *fi, uint32_t event, void *
 		lchan_rtp_fsm_state_chg(LCHAN_RTP_ST_ROLLBACK);
 		return;
 
+	case LCHAN_RTP_EV_READY_TO_SWITCH_RTP:
+		/* Ignore / silence an "event not permitted" error. In case of an inter-BSC incoming handover, there is
+		 * no previous lchan to be switched over, and we are already in this state when the usual handover code
+		 * path emits this event. */
+		return;
+
 	default:
 		OSMO_ASSERT(false);
 	}
@@ -704,6 +756,7 @@ static const struct osmo_fsm_state lchan_rtp_fsm_states[] = {
 			| S(LCHAN_RTP_EV_ESTABLISHED)
 			| S(LCHAN_RTP_EV_RELEASE)
 			| S(LCHAN_RTP_EV_ROLLBACK)
+			| S(LCHAN_RTP_EV_READY_TO_SWITCH_RTP)
 			,
 		.out_state_mask = 0
 			| S(LCHAN_RTP_ST_ESTABLISHED)
@@ -841,7 +894,7 @@ void mgcp_pick_codec(struct mgcp_conn_peer *verb_info, const struct gsm_lchan *l
 	if (codec < 0) {
 		LOG_LCHAN(lchan, LOGL_ERROR,
 			  "Unable to determine MGCP codec type for %s in chan-mode %s\n",
-			  gsm_lchant_name(lchan->type), gsm48_chan_mode_name(lchan->activate.ch_mode_rate.chan_mode));
+			  gsm_chan_t_name(lchan->type), gsm48_chan_mode_name(lchan->activate.ch_mode_rate.chan_mode));
 		verb_info->codecs_len = 0;
 		return;
 	}
@@ -853,8 +906,8 @@ void mgcp_pick_codec(struct mgcp_conn_peer *verb_info, const struct gsm_lchan *l
 	custom_pt = chan_mode_to_mgcp_bss_pt(codec);
 	if (bss_side && custom_pt > 0) {
 		verb_info->ptmap[0].codec = codec;
-	        verb_info->ptmap[0].pt = custom_pt;
-	        verb_info->ptmap_len = 1;
+		verb_info->ptmap[0].pt = custom_pt;
+		verb_info->ptmap_len = 1;
 	}
 
 	/* AMR requires additional parameters to be set up (framing mode) */
@@ -866,7 +919,7 @@ void mgcp_pick_codec(struct mgcp_conn_peer *verb_info, const struct gsm_lchan *l
 	if (bss_side && verb_info->codecs[0] == CODEC_AMR_8000_1) {
 		/* FIXME: At the moment all BTSs we support are using the
 		 * octet-aligned payload format. However, in the future
-		 * we may support BTSs that are using bandwith-efficient
+		 * we may support BTSs that are using bandwidth-efficient
 		 * format. In this case we will have to add functionality
 		 * that distinguishes by the BTS model which mode to use. */
 		verb_info->param.amr_octet_aligned = true;
@@ -875,3 +928,8 @@ void mgcp_pick_codec(struct mgcp_conn_peer *verb_info, const struct gsm_lchan *l
 		verb_info->param.amr_octet_aligned = lchan->conn->sccp.msc->amr_octet_aligned;
 	}
 }
+
+bool mgcp_codec_is_picked(const struct mgcp_conn_peer *verb_info, enum mgcp_codecs codec)
+{
+	return verb_info->codecs[0] == codec;
+}
