@@ -59,14 +59,15 @@ static void st_op_disabled_notinstalled_on_enter(struct osmo_fsm_inst *fi, uint3
 static void st_op_disabled_notinstalled(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
 	struct nm_statechg_signal_data *nsd;
-	struct gsm_nm_state *new_state;
+	const struct gsm_nm_state *new_state;
 
 	switch (event) {
 	case NM_EV_SW_ACT_REP:
+	case NM_EV_SETUP_RAMP_READY:
 		break;
 	case NM_EV_STATE_CHG_REP:
 		nsd = (struct nm_statechg_signal_data *)data;
-		new_state = nsd->new_state;
+		new_state = &nsd->new_state;
 		if (new_state->operational == NM_OPSTATE_ENABLED) {
 			/*should not happen... */
 			nm_bts_fsm_state_chg(fi, NM_BTS_ST_OP_ENABLED);
@@ -88,9 +89,13 @@ static void st_op_disabled_notinstalled(struct osmo_fsm_inst *fi, uint32_t event
 	}
 }
 
-static void configure_loop(struct gsm_bts *bts, struct gsm_nm_state *state, bool allow_opstart) {
+static void configure_loop(struct gsm_bts *bts, const struct gsm_nm_state *state, bool allow_opstart)
+{
 	struct msgb *msgb;
 
+	if (bts_setup_ramp_wait(bts))
+		return;
+
 	/* Request generic BTS-level attributes */
 	if (!bts->mo.get_attr_sent && !bts->mo.get_attr_rep_received) {
 		bts->mo.get_attr_sent = true;
@@ -105,7 +110,7 @@ static void configure_loop(struct gsm_bts *bts, struct gsm_nm_state *state, bool
 	if (bts->mo.get_attr_rep_received &&
 	    !bts->mo.set_attr_sent && !bts->mo.set_attr_ack_received) {
 		bts->mo.set_attr_sent = true;
-		msgb = nanobts_attr_bts_get(bts);
+		msgb = nanobts_gen_set_bts_attr(bts);
 		abis_nm_set_bts_attr(bts, msgb->data, msgb->len);
 		msgb_free(msgb);
 	}
@@ -138,8 +143,11 @@ static void rx_get_attr_rep(struct gsm_bts *bts, bool allow_opstart)
 	bts->mo.get_attr_sent = false;
 
 	/* Announce bts_features are available to related NSVC MOs */
-	nsvc = gsm_bts_sm_nsvc_num(bts->site_mgr, 0); /* we only support NSVC0 so far */
-	osmo_fsm_inst_dispatch(nsvc->mo.fi, NM_EV_FEATURE_NEGOTIATED, NULL);
+	for (int i = 0; i < ARRAY_SIZE(bts->site_mgr->gprs.nsvc); i++) {
+		nsvc = gsm_bts_sm_nsvc_num(bts->site_mgr, i);
+		if (nsvc)
+			osmo_fsm_inst_dispatch(nsvc->mo.fi, NM_EV_FEATURE_NEGOTIATED, NULL);
+	}
 
 	/* Move FSM forward */
 	configure_loop(bts, &bts->mo.nm_state, allow_opstart);
@@ -163,7 +171,7 @@ static void st_op_disabled_dependency(struct osmo_fsm_inst *fi, uint32_t event,
 {
 	struct gsm_bts *bts = (struct gsm_bts *)fi->priv;
 	struct nm_statechg_signal_data *nsd;
-	struct gsm_nm_state *new_state;
+	const struct gsm_nm_state *new_state;
 
 	switch (event) {
 	case NM_EV_GET_ATTR_REP:
@@ -176,7 +184,7 @@ static void st_op_disabled_dependency(struct osmo_fsm_inst *fi, uint32_t event,
 		return;
 	case NM_EV_STATE_CHG_REP:
 		nsd = (struct nm_statechg_signal_data *)data;
-		new_state = nsd->new_state;
+		new_state = &nsd->new_state;
 		if (new_state->operational == NM_OPSTATE_ENABLED) {
 			/* should not happen... */
 			nm_bts_fsm_state_chg(fi, NM_BTS_ST_OP_ENABLED);
@@ -197,6 +205,9 @@ static void st_op_disabled_dependency(struct osmo_fsm_inst *fi, uint32_t event,
 		default:
 			return;
 		}
+	case NM_EV_SETUP_RAMP_READY:
+		configure_loop(bts, &bts->mo.nm_state, false);
+		break;
 	default:
 		OSMO_ASSERT(0);
 	}
@@ -206,8 +217,6 @@ static void st_op_disabled_offline_on_enter(struct osmo_fsm_inst *fi, uint32_t p
 {
 	struct gsm_bts *bts = (struct gsm_bts *)fi->priv;
 
-	/* Warning: In here we may be acessing an state older than new_state
-	   from prev (syncrhonous) FSM state */
 	configure_loop(bts, &bts->mo.nm_state, true);
 }
 
@@ -215,7 +224,7 @@ static void st_op_disabled_offline(struct osmo_fsm_inst *fi, uint32_t event, voi
 {
 	struct gsm_bts *bts = (struct gsm_bts *)fi->priv;
 	struct nm_statechg_signal_data *nsd;
-	struct gsm_nm_state *new_state;
+	const struct gsm_nm_state *new_state;
 
 	switch (event) {
 	case NM_EV_GET_ATTR_REP:
@@ -228,7 +237,7 @@ static void st_op_disabled_offline(struct osmo_fsm_inst *fi, uint32_t event, voi
 		return;
 	case NM_EV_STATE_CHG_REP:
 		nsd = (struct nm_statechg_signal_data *)data;
-		new_state = nsd->new_state;
+		new_state = &nsd->new_state;
 		if (new_state->operational == NM_OPSTATE_ENABLED) {
 			nm_bts_fsm_state_chg(fi, NM_BTS_ST_OP_ENABLED);
 			return;
@@ -257,6 +266,9 @@ static void st_op_disabled_offline(struct osmo_fsm_inst *fi, uint32_t event, voi
 		default:
 			return;
 		}
+	case NM_EV_SETUP_RAMP_READY:
+		configure_loop(bts, &bts->mo.nm_state, true);
+		break;
 	default:
 		OSMO_ASSERT(0);
 	}
@@ -279,12 +291,12 @@ static void st_op_enabled_on_enter(struct osmo_fsm_inst *fi, uint32_t prev_state
 static void st_op_enabled(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
 	struct nm_statechg_signal_data *nsd;
-	struct gsm_nm_state *new_state;
+	const struct gsm_nm_state *new_state;
 
 	switch (event) {
 	case NM_EV_STATE_CHG_REP:
 		nsd = (struct nm_statechg_signal_data *)data;
-		new_state = nsd->new_state;
+		new_state = &nsd->new_state;
 		if (new_state->operational == NM_OPSTATE_ENABLED)
 			return;
 		switch (new_state->availability) { /* operational = DISABLED */
@@ -330,7 +342,8 @@ static struct osmo_fsm_state nm_bts_fsm_states[] = {
 	[NM_BTS_ST_OP_DISABLED_NOTINSTALLED] = {
 		.in_event_mask =
 			X(NM_EV_SW_ACT_REP) |
-			X(NM_EV_STATE_CHG_REP),
+			X(NM_EV_STATE_CHG_REP) |
+			X(NM_EV_SETUP_RAMP_READY),
 		.out_state_mask =
 			X(NM_BTS_ST_OP_DISABLED_DEPENDENCY) |
 			X(NM_BTS_ST_OP_DISABLED_OFFLINE) |
@@ -343,7 +356,8 @@ static struct osmo_fsm_state nm_bts_fsm_states[] = {
 		.in_event_mask =
 			X(NM_EV_STATE_CHG_REP) |
 			X(NM_EV_GET_ATTR_REP) |
-			X(NM_EV_SET_ATTR_ACK),
+			X(NM_EV_SET_ATTR_ACK) |
+			X(NM_EV_SETUP_RAMP_READY),
 		.out_state_mask =
 			X(NM_BTS_ST_OP_DISABLED_NOTINSTALLED) |
 			X(NM_BTS_ST_OP_DISABLED_OFFLINE) |
@@ -356,7 +370,8 @@ static struct osmo_fsm_state nm_bts_fsm_states[] = {
 		.in_event_mask =
 			X(NM_EV_STATE_CHG_REP) |
 			X(NM_EV_GET_ATTR_REP) |
-			X(NM_EV_SET_ATTR_ACK),
+			X(NM_EV_SET_ATTR_ACK) |
+			X(NM_EV_SETUP_RAMP_READY),
 		.out_state_mask =
 			X(NM_BTS_ST_OP_DISABLED_NOTINSTALLED) |
 			X(NM_BTS_ST_OP_DISABLED_DEPENDENCY) |
@@ -366,15 +381,15 @@ static struct osmo_fsm_state nm_bts_fsm_states[] = {
 		.action = st_op_disabled_offline,
 	},
 	[NM_BTS_ST_OP_ENABLED] = {
-	.in_event_mask =
-		X(NM_EV_STATE_CHG_REP),
-	.out_state_mask =
-		X(NM_BTS_ST_OP_DISABLED_NOTINSTALLED) |
-		X(NM_BTS_ST_OP_DISABLED_DEPENDENCY) |
-		X(NM_BTS_ST_OP_DISABLED_OFFLINE),
-	.name = "ENABLED",
-	.onenter = st_op_enabled_on_enter,
-	.action = st_op_enabled,
+		.in_event_mask =
+			X(NM_EV_STATE_CHG_REP),
+		.out_state_mask =
+			X(NM_BTS_ST_OP_DISABLED_NOTINSTALLED) |
+			X(NM_BTS_ST_OP_DISABLED_DEPENDENCY) |
+			X(NM_BTS_ST_OP_DISABLED_OFFLINE),
+		.name = "ENABLED",
+		.onenter = st_op_enabled_on_enter,
+		.action = st_op_enabled,
 	},
 };
 
