@@ -61,6 +61,7 @@ static void st_op_disabled_notinstalled(struct osmo_fsm_inst *fi, uint32_t event
 
 	switch (event) {
 	case NM_EV_SW_ACT_REP:
+	case NM_EV_SETUP_RAMP_READY:
 		break;
 	case NM_EV_STATE_CHG_REP:
 		nsd = (struct nm_statechg_signal_data *)data;
@@ -94,6 +95,9 @@ static void configure_loop(struct gsm_gprs_cell *cell, const struct gsm_nm_state
 	if (bts->gprs.mode == BTS_GPRS_NONE)
 		return;
 
+	if (bts_setup_ramp_wait(bts))
+		return;
+
 	if (!cell->mo.set_attr_sent && !cell->mo.set_attr_ack_received) {
 		cell->mo.set_attr_sent = true;
 		msgb = nanobts_gen_set_cell_attr(bts);
@@ -169,6 +173,9 @@ static void st_op_disabled_dependency(struct osmo_fsm_inst *fi, uint32_t event,
 		default:
 			return;
 		}
+	case NM_EV_SETUP_RAMP_READY:
+		configure_loop(cell, &cell->mo.nm_state, false);
+		break;
 	default:
 		OSMO_ASSERT(0);
 	}
@@ -225,6 +232,9 @@ static void st_op_disabled_offline(struct osmo_fsm_inst *fi, uint32_t event, voi
 		default:
 			return;
 		}
+	case NM_EV_SETUP_RAMP_READY:
+		configure_loop(cell, &cell->mo.nm_state, true);
+		break;
 	default:
 		OSMO_ASSERT(0);
 	}
@@ -303,7 +313,8 @@ static struct osmo_fsm_state nm_gprs_cell_fsm_states[] = {
 	[NM_GPRS_CELL_ST_OP_DISABLED_NOTINSTALLED] = {
 		.in_event_mask =
 			X(NM_EV_SW_ACT_REP) |
-			X(NM_EV_STATE_CHG_REP),
+			X(NM_EV_STATE_CHG_REP) |
+			X(NM_EV_SETUP_RAMP_READY),
 		.out_state_mask =
 			X(NM_GPRS_CELL_ST_OP_DISABLED_DEPENDENCY) |
 			X(NM_GPRS_CELL_ST_OP_DISABLED_OFFLINE) |
@@ -315,7 +326,8 @@ static struct osmo_fsm_state nm_gprs_cell_fsm_states[] = {
 	[NM_GPRS_CELL_ST_OP_DISABLED_DEPENDENCY] = {
 		.in_event_mask =
 			X(NM_EV_STATE_CHG_REP) |
-			X(NM_EV_SET_ATTR_ACK),
+			X(NM_EV_SET_ATTR_ACK) |
+			X(NM_EV_SETUP_RAMP_READY),
 		.out_state_mask =
 			X(NM_GPRS_CELL_ST_OP_DISABLED_NOTINSTALLED) |
 			X(NM_GPRS_CELL_ST_OP_DISABLED_OFFLINE) |
@@ -327,7 +339,8 @@ static struct osmo_fsm_state nm_gprs_cell_fsm_states[] = {
 	[NM_GPRS_CELL_ST_OP_DISABLED_OFFLINE] = {
 		.in_event_mask =
 			X(NM_EV_STATE_CHG_REP) |
-			X(NM_EV_SET_ATTR_ACK),
+			X(NM_EV_SET_ATTR_ACK) |
+			X(NM_EV_SETUP_RAMP_READY),
 		.out_state_mask =
 			X(NM_GPRS_CELL_ST_OP_DISABLED_NOTINSTALLED) |
 			X(NM_GPRS_CELL_ST_OP_DISABLED_DEPENDENCY) |
