@@ -63,8 +63,8 @@ static void st_op_disabled_notinstalled(struct osmo_fsm_inst *fi, uint32_t event
 
 	switch (event) {
 	case NM_EV_FEATURE_NEGOTIATED:
-		break;
 	case NM_EV_SW_ACT_REP:
+	case NM_EV_SETUP_RAMP_READY:
 		break;
 	case NM_EV_STATE_CHG_REP:
 		nsd = (struct nm_statechg_signal_data *)data;
@@ -90,6 +90,17 @@ static void st_op_disabled_notinstalled(struct osmo_fsm_inst *fi, uint32_t event
 	}
 }
 
+static bool has_valid_nsvc(struct gsm_gprs_nsvc *nsvc)
+{
+	switch (nsvc->remote.u.sa.sa_family) {
+	case AF_INET:
+	case AF_INET6:
+		return (nsvc->local_port > 0 && !osmo_sockaddr_is_any(&nsvc->remote));
+	default:
+		return false;
+	}
+}
+
 static void configure_loop(struct gsm_gprs_nsvc *nsvc, const struct gsm_nm_state *state, bool allow_opstart)
 {
 	struct msgb *msgb;
@@ -97,6 +108,9 @@ static void configure_loop(struct gsm_gprs_nsvc *nsvc, const struct gsm_nm_state
 	if (nsvc->bts->gprs.mode == BTS_GPRS_NONE)
 		return;
 
+	if (bts_setup_ramp_wait(nsvc->bts))
+		return;
+
 	/* We need to know BTS features in order to know if we can set IPv6 addresses */
 	if (gsm_bts_features_negotiated(nsvc->bts) && !nsvc->mo.set_attr_sent &&
 	    !nsvc->mo.set_attr_ack_received) {
@@ -107,8 +121,11 @@ static void configure_loop(struct gsm_gprs_nsvc *nsvc, const struct gsm_nm_state
 				 nsvc->bts->nr);
 			return;
 		}
+		if (!has_valid_nsvc(nsvc))
+			return;
+
 		nsvc->mo.set_attr_sent = true;
-		msgb = nanobts_gen_set_nsvc_attr(nsvc->bts);
+		msgb = nanobts_gen_set_nsvc_attr(nsvc);
 		OSMO_ASSERT(msgb);
 		abis_nm_ipaccess_set_attr(nsvc->bts, NM_OC_GPRS_NSVC, nsvc->bts->bts_nr,
 					  nsvc->id, 0xff, msgb->data, msgb->len);
@@ -185,6 +202,9 @@ static void st_op_disabled_dependency(struct osmo_fsm_inst *fi, uint32_t event,
 		default:
 			return;
 		}
+	case NM_EV_SETUP_RAMP_READY:
+		configure_loop(nsvc, &nsvc->mo.nm_state, false);
+		break;
 	default:
 		OSMO_ASSERT(0);
 	}
@@ -243,6 +263,9 @@ static void st_op_disabled_offline(struct osmo_fsm_inst *fi, uint32_t event, voi
 		default:
 			return;
 		}
+	case NM_EV_SETUP_RAMP_READY:
+		configure_loop(nsvc, &nsvc->mo.nm_state, true);
+		break;
 	default:
 		OSMO_ASSERT(0);
 	}
@@ -315,7 +338,8 @@ static struct osmo_fsm_state nm_gprs_nsvc_fsm_states[] = {
 		.in_event_mask =
 			X(NM_EV_SW_ACT_REP) |
 			X(NM_EV_FEATURE_NEGOTIATED) |
-			X(NM_EV_STATE_CHG_REP),
+			X(NM_EV_STATE_CHG_REP) |
+			X(NM_EV_SETUP_RAMP_READY),
 		.out_state_mask =
 			X(NM_GPRS_NSVC_ST_OP_DISABLED_DEPENDENCY) |
 			X(NM_GPRS_NSVC_ST_OP_DISABLED_OFFLINE) |
@@ -328,7 +352,8 @@ static struct osmo_fsm_state nm_gprs_nsvc_fsm_states[] = {
 		.in_event_mask =
 			X(NM_EV_STATE_CHG_REP) |
 			X(NM_EV_FEATURE_NEGOTIATED) |
-			X(NM_EV_SET_ATTR_ACK),
+			X(NM_EV_SET_ATTR_ACK) |
+			X(NM_EV_SETUP_RAMP_READY),
 		.out_state_mask =
 			X(NM_GPRS_NSVC_ST_OP_DISABLED_NOTINSTALLED) |
 			X(NM_GPRS_NSVC_ST_OP_DISABLED_OFFLINE) |
@@ -341,7 +366,8 @@ static struct osmo_fsm_state nm_gprs_nsvc_fsm_states[] = {
 		.in_event_mask =
 			X(NM_EV_STATE_CHG_REP) |
 			X(NM_EV_FEATURE_NEGOTIATED) |
-			X(NM_EV_SET_ATTR_ACK),
+			X(NM_EV_SET_ATTR_ACK) |
+			X(NM_EV_SETUP_RAMP_READY),
 		.out_state_mask =
 			X(NM_GPRS_NSVC_ST_OP_DISABLED_NOTINSTALLED) |
 			X(NM_GPRS_NSVC_ST_OP_DISABLED_DEPENDENCY) |
