@@ -40,6 +40,16 @@
 #define nm_rcarrier_fsm_state_chg(fi, NEXT_STATE) \
 	osmo_fsm_inst_state_chg(fi, NEXT_STATE, 0, 0)
 
+static inline void nm_rcarrier_fsm_becomes_enabled(struct gsm_bts_trx *trx)
+{
+	nm_obj_fsm_becomes_enabled_disabled(trx->bts, trx, NM_OC_RADIO_CARRIER, true);
+}
+
+static inline void nm_rcarrier_fsm_becomes_disabled(struct gsm_bts_trx *trx)
+{
+	nm_obj_fsm_becomes_enabled_disabled(trx->bts, trx, NM_OC_RADIO_CARRIER, false);
+}
+
 //////////////////////////
 // FSM STATE ACTIONS
 //////////////////////////
@@ -57,14 +67,15 @@ static void st_op_disabled_notinstalled_on_enter(struct osmo_fsm_inst *fi, uint3
 static void st_op_disabled_notinstalled(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
 	struct nm_statechg_signal_data *nsd;
-	struct gsm_nm_state *new_state;
+	const struct gsm_nm_state *new_state;
 
 	switch (event) {
 	case NM_EV_SW_ACT_REP:
+	case NM_EV_SETUP_RAMP_READY:
 		break;
 	case NM_EV_STATE_CHG_REP:
 		nsd = (struct nm_statechg_signal_data *)data;
-		new_state = nsd->new_state;
+		new_state = &nsd->new_state;
 		if (new_state->operational == NM_OPSTATE_ENABLED) {
 			/*should not happen... */
 			nm_rcarrier_fsm_state_chg(fi, NM_RCARRIER_ST_OP_ENABLED);
@@ -86,13 +97,16 @@ static void st_op_disabled_notinstalled(struct osmo_fsm_inst *fi, uint32_t event
 	}
 }
 
-static void configure_loop(struct gsm_bts_trx *trx, struct gsm_nm_state *state, bool allow_opstart)
+static void configure_loop(struct gsm_bts_trx *trx, const struct gsm_nm_state *state, bool allow_opstart)
 {
 	struct msgb *msgb;
 
+	if (bts_setup_ramp_wait(trx->bts))
+		return;
+
 	if (!trx->mo.set_attr_sent && !trx->mo.set_attr_ack_received) {
 		trx->mo.set_attr_sent = true;
-		msgb = nanobts_attr_radio_get(trx->bts, trx);
+		msgb = nanobts_gen_set_radio_attr(trx->bts, trx);
 		abis_nm_set_radio_attr(trx, msgb->data, msgb->len);
 		msgb_free(msgb);
 	}
@@ -134,7 +148,7 @@ static void st_op_disabled_dependency(struct osmo_fsm_inst *fi, uint32_t event,
 {
 	struct gsm_bts_trx *trx = (struct gsm_bts_trx *)fi->priv;
 	struct nm_statechg_signal_data *nsd;
-	struct gsm_nm_state *new_state;
+	const struct gsm_nm_state *new_state;
 
 	switch (event) {
 	case NM_EV_SET_ATTR_ACK:
@@ -144,7 +158,7 @@ static void st_op_disabled_dependency(struct osmo_fsm_inst *fi, uint32_t event,
 		return;
 	case NM_EV_STATE_CHG_REP:
 		nsd = (struct nm_statechg_signal_data *)data;
-		new_state = nsd->new_state;
+		new_state = &nsd->new_state;
 		if (new_state->operational == NM_OPSTATE_ENABLED) {
 			/* should not happen... */
 			nm_rcarrier_fsm_state_chg(fi, NM_RCARRIER_ST_OP_ENABLED);
@@ -165,6 +179,9 @@ static void st_op_disabled_dependency(struct osmo_fsm_inst *fi, uint32_t event,
 		default:
 			return;
 		}
+	case NM_EV_SETUP_RAMP_READY:
+		configure_loop(trx, &trx->mo.nm_state, false);
+		break;
 	default:
 		OSMO_ASSERT(0);
 	}
@@ -174,8 +191,6 @@ static void st_op_disabled_offline_on_enter(struct osmo_fsm_inst *fi, uint32_t p
 {
 	struct gsm_bts_trx *trx = (struct gsm_bts_trx *)fi->priv;
 
-	/* Warning: In here we may be acessing an state older than new_state
-	   from prev (syncrhonous) FSM state */
 	configure_loop(trx, &trx->mo.nm_state, true);
 }
 
@@ -183,7 +198,7 @@ static void st_op_disabled_offline(struct osmo_fsm_inst *fi, uint32_t event, voi
 {
 	struct gsm_bts_trx *trx = (struct gsm_bts_trx *)fi->priv;
 	struct nm_statechg_signal_data *nsd;
-	struct gsm_nm_state *new_state;
+	const struct gsm_nm_state *new_state;
 
 	switch (event) {
 	case NM_EV_SET_ATTR_ACK:
@@ -193,7 +208,7 @@ static void st_op_disabled_offline(struct osmo_fsm_inst *fi, uint32_t event, voi
 		return;
 	case NM_EV_STATE_CHG_REP:
 		nsd = (struct nm_statechg_signal_data *)data;
-		new_state = nsd->new_state;
+		new_state = &nsd->new_state;
 		if (new_state->operational == NM_OPSTATE_ENABLED) {
 			nm_rcarrier_fsm_state_chg(fi, NM_RCARRIER_ST_OP_ENABLED);
 			return;
@@ -213,6 +228,9 @@ static void st_op_disabled_offline(struct osmo_fsm_inst *fi, uint32_t event, voi
 		default:
 			return;
 		}
+	case NM_EV_SETUP_RAMP_READY:
+		configure_loop(trx, &trx->mo.nm_state, true);
+		break;
 	default:
 		OSMO_ASSERT(0);
 	}
@@ -228,29 +246,54 @@ static void st_op_enabled_on_enter(struct osmo_fsm_inst *fi, uint32_t prev_state
 	trx->mo.adm_unlock_sent = false;
 	trx->mo.set_attr_ack_received = false;
 	trx->mo.set_attr_sent = false;
+
+	nm_rcarrier_fsm_becomes_enabled(trx);
 }
 
 static void st_op_enabled(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
+	struct gsm_bts_trx *trx = (struct gsm_bts_trx *)fi->priv;
 	struct nm_statechg_signal_data *nsd;
-	struct gsm_nm_state *new_state;
+	const struct gsm_nm_state *new_state;
 
 	switch (event) {
 	case NM_EV_STATE_CHG_REP:
 		nsd = (struct nm_statechg_signal_data *)data;
-		new_state = nsd->new_state;
-		if (new_state->operational == NM_OPSTATE_ENABLED)
+		new_state = &nsd->new_state;
+		/* Op state stays in Enabled, hence either Avail or Admin changed: */
+		if (new_state->operational == NM_OPSTATE_ENABLED) {
+			/* Some sort of availability change we don't care about: */
+			if (nsd->old_state.administrative == new_state->administrative)
+				return;
+			/* HACK: Admin state change without Op state change:
+			 * According to TS 52.021 sec 5.3.1, Locking the NM obj should make
+			 * it go into Disabled Dependency state, but current and older
+			 * versions of osmo-bts (and potentially nanobts?) don't move from
+			 * Operative=Enabled state and only change the Adminsitrative one.
+			 * Let's account for this behavior here: */
+			switch (new_state->administrative) {
+			case NM_STATE_LOCKED:
+				nm_rcarrier_fsm_becomes_disabled(trx);
+				break;
+			case NM_STATE_UNLOCKED:
+				nm_rcarrier_fsm_becomes_enabled(trx);
+				break;
+			}
 			return;
+		}
 		switch (new_state->availability) { /* operational = DISABLED */
 		case NM_AVSTATE_NOT_INSTALLED:
 		case NM_AVSTATE_POWER_OFF:
+			nm_rcarrier_fsm_becomes_disabled(trx);
 			nm_rcarrier_fsm_state_chg(fi, NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED);
 			return;
 		case NM_AVSTATE_DEPENDENCY:
+			nm_rcarrier_fsm_becomes_disabled(trx);
 			nm_rcarrier_fsm_state_chg(fi, NM_RCARRIER_ST_OP_DISABLED_DEPENDENCY);
 			return;
 		case NM_AVSTATE_OFF_LINE:
 		case NM_AVSTATE_OK:
+			nm_rcarrier_fsm_becomes_disabled(trx);
 			nm_rcarrier_fsm_state_chg(fi, NM_RCARRIER_ST_OP_DISABLED_OFFLINE);
 			return;
 		default:
@@ -278,8 +321,11 @@ static void st_op_allstate(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 				      trx->mo.force_rf_lock ? NM_STATE_LOCKED : NM_STATE_UNLOCKED);
 		break;
 	case NM_EV_OML_DOWN:
-		if (fi->state != NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED)
+		if (fi->state != NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED) {
+			if (fi->state == NM_RCARRIER_ST_OP_ENABLED)
+				nm_rcarrier_fsm_becomes_disabled(trx);
 			nm_rcarrier_fsm_state_chg(fi, NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED);
+		}
 		break;
 	default:
 		OSMO_ASSERT(0);
@@ -290,7 +336,8 @@ static struct osmo_fsm_state nm_rcarrier_fsm_states[] = {
 	[NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED] = {
 		.in_event_mask =
 			X(NM_EV_SW_ACT_REP) |
-			X(NM_EV_STATE_CHG_REP),
+			X(NM_EV_STATE_CHG_REP) |
+			X(NM_EV_SETUP_RAMP_READY),
 		.out_state_mask =
 			X(NM_RCARRIER_ST_OP_DISABLED_DEPENDENCY) |
 			X(NM_RCARRIER_ST_OP_DISABLED_OFFLINE) |
@@ -302,7 +349,8 @@ static struct osmo_fsm_state nm_rcarrier_fsm_states[] = {
 	[NM_RCARRIER_ST_OP_DISABLED_DEPENDENCY] = {
 		.in_event_mask =
 			X(NM_EV_STATE_CHG_REP) |
-			X(NM_EV_SET_ATTR_ACK),
+			X(NM_EV_SET_ATTR_ACK) |
+			X(NM_EV_SETUP_RAMP_READY),
 		.out_state_mask =
 			X(NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED) |
 			X(NM_RCARRIER_ST_OP_DISABLED_OFFLINE) |
@@ -314,7 +362,8 @@ static struct osmo_fsm_state nm_rcarrier_fsm_states[] = {
 	[NM_RCARRIER_ST_OP_DISABLED_OFFLINE] = {
 		.in_event_mask =
 			X(NM_EV_STATE_CHG_REP) |
-			X(NM_EV_SET_ATTR_ACK),
+			X(NM_EV_SET_ATTR_ACK) |
+			X(NM_EV_SETUP_RAMP_READY),
 		.out_state_mask =
 			X(NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED) |
 			X(NM_RCARRIER_ST_OP_DISABLED_DEPENDENCY) |
@@ -324,15 +373,15 @@ static struct osmo_fsm_state nm_rcarrier_fsm_states[] = {
 		.action = st_op_disabled_offline,
 	},
 	[NM_RCARRIER_ST_OP_ENABLED] = {
-	.in_event_mask =
-		X(NM_EV_STATE_CHG_REP),
-	.out_state_mask =
-		X(NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED) |
-		X(NM_RCARRIER_ST_OP_DISABLED_DEPENDENCY) |
-		X(NM_RCARRIER_ST_OP_DISABLED_OFFLINE),
-	.name = "ENABLED",
-	.onenter = st_op_enabled_on_enter,
-	.action = st_op_enabled,
+		.in_event_mask =
+			X(NM_EV_STATE_CHG_REP),
+		.out_state_mask =
+			X(NM_RCARRIER_ST_OP_DISABLED_NOTINSTALLED) |
+			X(NM_RCARRIER_ST_OP_DISABLED_DEPENDENCY) |
+			X(NM_RCARRIER_ST_OP_DISABLED_OFFLINE),
+		.name = "ENABLED",
+		.onenter = st_op_enabled_on_enter,
+		.action = st_op_enabled,
 	},
 };
 
