@@ -91,12 +91,12 @@ static const char *config_file = "osmo-bsc.cfg";
 static const char *rf_ctrl = NULL;
 static int daemonize = 0;
 
-static void print_usage()
+static void print_usage(void)
 {
 	printf("Usage: osmo-bsc\n");
 }
 
-static void print_help()
+static void print_help(void)
 {
 	printf("Some useful options:\n");
 	printf("  -h --help 			This text.\n");
@@ -359,24 +359,10 @@ static void bootstrap_rsl(struct gsm_bts_trx *trx)
 		osmo_fsm_inst_dispatch(ts->fi, TS_EV_RSL_READY, NULL);
 	}
 
-	/* Start CBCH transmit timer if CBCH is present */
-	if (trx->nr == 0 && gsm_bts_get_cbch(trx->bts))
-		bts_cbch_timer_schedule(trx->bts);
-
 	/* Drop all expired channel requests in the list */
 	abis_rsl_chan_rqd_queue_flush(trx->bts);
 }
 
-static void all_ts_dispatch_event(struct gsm_bts_trx *trx, uint32_t event)
-{
-	int ts_i;
-	for (ts_i = 0; ts_i < ARRAY_SIZE(trx->ts); ts_i++) {
-		struct gsm_bts_trx_ts *ts = &trx->ts[ts_i];
-		if (ts->fi)
-			osmo_fsm_inst_dispatch(ts->fi, event, 0);
-	}
-}
-
 struct osmo_timer_list update_connection_stats_timer;
 
 /* Periodically call bsc_update_connection_stats() to keep stat items updated.
@@ -389,56 +375,6 @@ static void update_connection_stats_cb(void *data)
 	osmo_timer_schedule(&update_connection_stats_timer, 1, 0);
 }
 
-static int check_bts(struct gsm_bts *bts)
-{
-	struct gsm_bts_trx *trx;
-
-	if (!bts->model)
-		return -EFAULT;
-
-	switch (bts->band) {
-	case GSM_BAND_1800:
-		if (bts->c0->arfcn < 512 || bts->c0->arfcn > 885) {
-			LOGP(DNM, LOGL_ERROR, "(bts=%u) GSM1800 channel (%u) must be between 512-885.\n",
-			     bts->nr, bts->c0->arfcn);
-			return -EINVAL;
-		}
-		break;
-	case GSM_BAND_1900:
-		if (bts->c0->arfcn < 512 || bts->c0->arfcn > 810) {
-			LOGP(DNM, LOGL_ERROR, "(bts=%u) GSM1900 channel (%u) must be between 512-810.\n",
-			     bts->nr, bts->c0->arfcn);
-		}
-		break;
-	case GSM_BAND_900:
-		if ((bts->c0->arfcn > 124 && bts->c0->arfcn < 955) ||
-		    bts->c0->arfcn > 1023)  {
-			LOGP(DNM, LOGL_ERROR, "(bts=%u) GSM900 channel (%u) must be between 0-124, 955-1023.\n",
-			     bts->nr, bts->c0->arfcn);
-		}
-		break;
-	case GSM_BAND_850:
-		if (bts->c0->arfcn < 128 || bts->c0->arfcn > 251) {
-			LOGP(DNM, LOGL_ERROR, "(bts=%u) GSM850 channel (%u) must be between 128-251.\n",
-			     bts->nr, bts->c0->arfcn);
-		}
-		break;
-	default:
-		LOGP(DNM, LOGL_ERROR, "(bts=%u) Unsupported frequency band.\n", bts->nr);
-	}
-
-	/* Verify the physical channel mapping */
-	llist_for_each_entry(trx, &bts->trx_list, list) {
-		if (!trx_has_valid_pchan_config(trx)) {
-			LOGP(DNM, LOGL_ERROR, "TRX %u has invalid timeslot "
-					      "configuration\n", trx->nr);
-			return -EINVAL;
-		}
-	}
-
-	return 0;
-}
-
 static void bootstrap_bts(struct gsm_bts *bts)
 {
 	unsigned int n = 0;
@@ -465,6 +401,8 @@ static void bootstrap_bts(struct gsm_bts *bts)
 		bts->si_common.chan_desc.ccch_conf = (n << 1);
 	}
 
+	bts_setup_ramp_init_bts(bts);
+
 	/* ACC ramping is initialized from vty/config */
 
 	/* Initialize the BTS state */
@@ -492,7 +430,7 @@ static int inp_sig_cb(unsigned int subsys, unsigned int signal,
 	case S_L_INP_TEI_UP:
 		if (isd->link_type == E1INP_SIGN_OML) {
 			/* Check parameters and apply vty config dependent parameters */
-			rc = check_bts(trx->bts);
+			rc = gsm_bts_check_cfg(trx->bts);
 			if (rc < 0) {
 				LOGP(DNM, LOGL_ERROR, "(bts=%u) Error in BTS configuration -- cannot bootstrap BTS\n",
 				     trx->bts->nr);
@@ -501,7 +439,7 @@ static int inp_sig_cb(unsigned int subsys, unsigned int signal,
 			bootstrap_bts(trx->bts);
 		}
 		if (isd->link_type == E1INP_SIGN_RSL) {
-			rc = check_bts(trx->bts);
+			rc = gsm_bts_check_cfg(trx->bts);
 			if (rc < 0) {
 				LOGP(DNM, LOGL_ERROR, "(bts=%u) Error in BTS configuration -- cannot bootstrap RSL\n",
 				     trx->bts->nr);
@@ -515,13 +453,16 @@ static int inp_sig_cb(unsigned int subsys, unsigned int signal,
 
 		if (isd->link_type == E1INP_SIGN_OML) {
 			rate_ctr_inc(rate_ctr_group_get_ctr(trx->bts->bts_ctrs, BTS_CTR_BTS_OML_FAIL));
-			all_ts_dispatch_event(trx, TS_EV_OML_DOWN);
+			/* ip.access BTS models have a single global A-bis/OML link for all
+			 * transceivers, so once it's lost we need to notify them all. */
+			if (is_ipaccess_bts(trx->bts))
+				gsm_bts_all_ts_dispatch(trx->bts, TS_EV_OML_DOWN, NULL);
+			else /* Other BTS models (e.g. Ericsson) have per-TRX OML links */
+				gsm_trx_all_ts_dispatch(trx, TS_EV_OML_DOWN, NULL);
 		} else if (isd->link_type == E1INP_SIGN_RSL) {
 			rate_ctr_inc(rate_ctr_group_get_ctr(trx->bts->bts_ctrs, BTS_CTR_BTS_RSL_FAIL));
 			acc_ramp_abort(&trx->bts->acc_ramp);
-			all_ts_dispatch_event(trx, TS_EV_RSL_DOWN);
-			if (trx->nr == 0)
-				osmo_timer_del(&trx->bts->cbch_timer);
+			gsm_trx_all_ts_dispatch(trx, TS_EV_RSL_DOWN, NULL);
 		}
 
 		gsm_bts_sm_mo_reset(trx->bts->site_mgr);
@@ -547,8 +488,7 @@ static int bsc_network_configure(const char *config_file)
 	}
 
 	/* start telnet after reading config for vty_get_bind_addr() */
-	rc = telnet_init_dynif(tall_bsc_ctx, bsc_gsmnet, vty_get_bind_addr(),
-			       OSMO_VTY_PORT_NITB_BSC);
+	rc = telnet_init_default(tall_bsc_ctx, bsc_gsmnet, OSMO_VTY_PORT_NITB_BSC);
 	if (rc < 0)
 		return rc;
 
@@ -556,7 +496,7 @@ static int bsc_network_configure(const char *config_file)
 	osmo_signal_register_handler(SS_L_INPUT, inp_sig_cb, NULL);
 
 	llist_for_each_entry(bts, &bsc_gsmnet->bts_list, list) {
-		rc = check_bts(bts);
+		rc = gsm_bts_check_cfg(bts);
 		if (rc < 0) {
 			LOGP(DNM, LOGL_FATAL, "(bts=%u) cannot bootstrap BTS, invalid BTS configuration\n", bts->nr);
 			return rc;
@@ -579,6 +519,11 @@ static int bsc_vty_go_parent(struct vty *vty)
 		vty->node = CONFIG_NODE;
 		vty->index = NULL;
 		break;
+	case MGW_NODE:
+		vty->node = GSMNET_NODE;
+		vty->index = bsc_gsmnet;
+		vty->index_sub = NULL;
+		break;
 	case BTS_NODE:
 		vty->node = GSMNET_NODE;
 		{
@@ -675,7 +620,9 @@ static struct vty_app_info vty_info = {
 	.copyright	=
 	"Copyright (C) 2008-2018 Harald Welte, Holger Freyther\r\n"
 	"Contributions by Daniel Willmann, Jan Lübbe, Stefan Schmidt\r\n"
-	"Dieter Spaar, Andreas Eversberg, Sylvain Munaut, Neels Hofmeyr\r\n\r\n"
+	"Dieter Spaar, Andreas Eversberg, Sylvain Munaut, Neels Hofmeyr\r\n"
+	"Copyright (C) 2013-2022 sysmocom - s.f.m.c. GmbH\r\n"
+	"\r\n"
 	"License AGPLv3+: GNU AGPL version 3 or later <http://gnu.org/licenses/agpl-3.0.html>\r\n"
 	"This is free software: you are free to change and redistribute it.\r\n"
 	"There is NO WARRANTY, to the extent permitted by law.\r\n",
@@ -890,7 +837,7 @@ static int bsc_mgw_setup(void)
 	struct mgcp_client *mgcp_client_single;
 	unsigned int pool_members_initalized;
 
-	/* Initalize MGW pool. This initalizes and connects all MGCP clients that are currently configured in
+	/* Initialize MGW pool. This initalizes and connects all MGCP clients that are currently configured in
 	 * the pool. Adding additional MGCP clients to the pool is possible but the user has to configure and
 	 * (re)connect them manually from the VTY. */
 	pool_members_initalized = mgcp_client_pool_connect(bsc_gsmnet->mgw.mgw_pool);
@@ -904,7 +851,7 @@ static int bsc_mgw_setup(void)
 	/* Initialize and connect a single MGCP client. This MGCP client will appear as the one and only pool
 	 * member if there is no MGW pool configured. */
 	LOGP(DNM, LOGL_NOTICE, "No MGW pool configured, using MGW configuration in VTY node 'msc'\n");
-        mgcp_client_single = mgcp_client_init(bsc_gsmnet, bsc_gsmnet->mgw.conf);
+	mgcp_client_single = mgcp_client_init(bsc_gsmnet, bsc_gsmnet->mgw.conf);
 	if (!mgcp_client_single) {
 		LOGP(DNM, LOGL_ERROR, "MGW (single) client initalization failed\n");
 		return -EINVAL;
@@ -941,6 +888,7 @@ int main(int argc, char **argv)
 	rate_ctr_init(tall_bsc_ctx);
 
 	osmo_fsm_set_dealloc_ctx(OTC_SELECT);
+	osmo_fsm_log_timeouts(true);
 
 	/* Allocate global gsm_network struct */
 	rc = bsc_network_alloc();
@@ -983,6 +931,9 @@ int main(int argc, char **argv)
 	assignment_fsm_init();
 	handover_fsm_init();
 	lb_init();
+	acc_ramp_global_init();
+	paging_global_init();
+	smscb_global_init();
 
 	/* Read the config */
 	rc = bsc_network_configure(config_file);
@@ -998,9 +949,7 @@ int main(int argc, char **argv)
 
 	/* start control interface after reading config for
 	 * ctrl_vty_get_bind_addr() */
-	bsc_gsmnet->ctrl = bsc_controlif_setup(bsc_gsmnet,
-					       ctrl_vty_get_bind_addr(),
-					       OSMO_CTRL_PORT_NITB_BSC);
+	bsc_gsmnet->ctrl = bsc_controlif_setup(bsc_gsmnet, OSMO_CTRL_PORT_NITB_BSC);
 	if (!bsc_gsmnet->ctrl) {
 		fprintf(stderr, "Failed to init the control interface. Exiting.\n");
 		exit(1);
@@ -1046,6 +995,9 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if (bsc_mgw_setup() != 0)
+		exit(1);
+
 	llist_for_each_entry(msc, &bsc_gsmnet->mscs, entry) {
 		if (osmo_bsc_msc_init(msc) != 0) {
 			LOGP(DMSC, LOGL_ERROR, "Failed to start up. Exiting.\n");
@@ -1053,9 +1005,6 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (bsc_mgw_setup() != 0)
-		exit(1);
-
 	if (osmo_bsc_sigtran_init(&bsc_gsmnet->mscs) != 0) {
 		LOGP(DNM, LOGL_ERROR, "Failed to initialize sigtran backhaul.\n");
 		exit(1);
@@ -1074,6 +1023,7 @@ int main(int argc, char **argv)
 	osmo_init_ignore_signals();
 
 	update_connection_stats_cb(NULL);
+	chan_counts_sig_init();
 
 	if (daemonize) {
 		rc = osmo_daemonize();
