@@ -33,6 +33,7 @@
 #include <osmocom/core/logging.h>
 #include <osmocom/gsm/l1sap.h>
 #include <osmocom/gsm/gsm0502.h>
+#include <osmocom/bsc/abis_nm.h>
 
 #include <osmocom/bsc/gsm_data.h>
 #include <osmocom/bsc/pcu_if.h>
@@ -45,8 +46,6 @@
 #include <osmocom/bsc/bts_sm.h>
 
 static int pcu_sock_send(struct gsm_bts *bts, struct msgb *msg);
-uint32_t trx_get_hlayer1(struct gsm_bts_trx *trx);
-int pcu_direct = 1;
 
 static const char *sapi_string[] = {
 	[PCU_IF_SAPI_RACH] =	"RACH",
@@ -93,6 +92,20 @@ struct msgb *pcu_msgb_alloc(uint8_t msg_type, uint8_t bts_nr)
 	return msg;
 }
 
+/* Check if the timeslot can be utilized as PDCH now
+ * (PDCH is currently active on BTS) */
+static bool ts_now_usable_as_pdch(const struct gsm_bts_trx_ts *ts)
+{
+	switch (ts->pchan_is) {
+	case GSM_PCHAN_PDCH:
+		/* NOTE: We currently only support Ericsson RBS as a BSC
+		 * co-located BTS. This BTS only supports dynamic channels. */
+		return true;
+	default:
+		return false;
+	}
+}
+
 /* Fill the frequency hopping parameter */
 static void info_ind_fill_fhp(struct gsm_pcu_if_info_trx_ts *ts_info,
 			      const struct gsm_bts_trx_ts *ts)
@@ -105,6 +118,48 @@ static void info_ind_fill_fhp(struct gsm_pcu_if_info_trx_ts *ts_info,
 	ts_info->ma_bit_len = ts->hopping.ma_len * 8 - ts->hopping.ma.cur_bit;
 }
 
+/* Fill the TRX parameter */
+static void info_ind_fill_trx(struct gsm_pcu_if_info_trx *trx_info, const struct gsm_bts_trx *trx)
+{
+	unsigned int tn;
+	const struct gsm_bts_trx_ts *ts;
+
+	trx_info->hlayer1 = 0x2342;
+	trx_info->pdch_mask = 0;
+	trx_info->arfcn = trx->arfcn;
+
+	if (trx->mo.nm_state.operational != NM_OPSTATE_ENABLED ||
+	    trx->mo.nm_state.administrative != NM_STATE_UNLOCKED) {
+		LOG_TRX(trx, DPCU, LOGL_INFO, "unavailable for PCU (op=%s adm=%s)\n",
+			abis_nm_opstate_name(trx->mo.nm_state.operational),
+			abis_nm_admin_name(trx->mo.nm_state.administrative));
+		return;
+	}
+
+	for (tn = 0; tn < ARRAY_SIZE(trx->ts); tn++) {
+		ts = &trx->ts[tn];
+		if (ts->mo.nm_state.operational != NM_OPSTATE_ENABLED)
+			continue;
+		if (!ts_now_usable_as_pdch(ts))
+			continue;
+
+		trx_info->pdch_mask |= (1 << tn);
+		trx_info->ts[tn].tsc =
+				(ts->tsc >= 0) ? ts->tsc : trx->bts->bsic & 7;
+
+		if (ts->hopping.enabled)
+			info_ind_fill_fhp(&trx_info->ts[tn], ts);
+
+		LOG_TRX(trx, DPCU, LOGL_INFO, "PDCH on ts=%u is available (tsc=%u ", ts->nr,
+			trx_info->ts[tn].tsc);
+		if (ts->hopping.enabled)
+			LOGPC(DPCU, LOGL_INFO, "hopping=yes hsn=%u maio=%u ma_bit_len=%u)\n",
+			      ts->hopping.hsn, ts->hopping.maio, trx_info->ts[tn].ma_bit_len);
+		else
+			LOGPC(DPCU, LOGL_INFO, "hopping=no arfcn=%u)\n", trx->arfcn);
+	}
+}
+
 /* Send BTS properties to the PCU */
 static int pcu_tx_info_ind(struct gsm_bts *bts)
 {
@@ -115,16 +170,11 @@ static int pcu_tx_info_ind(struct gsm_bts *bts)
 	struct gsm_bts_sm *bts_sm;
 	struct gsm_gprs_nsvc *nsvc;
 	struct gsm_bts_trx *trx;
-	struct gsm_bts_trx_ts *ts;
-	int i, tn;
-
-	OSMO_ASSERT(bts);
-	OSMO_ASSERT(bts->network);
-	OSMO_ASSERT(bts->site_mgr);
+	int i;
 
 	bts_sm = bts->site_mgr;
 
-	LOGP(DPCU, LOGL_INFO, "Sending info for BTS %d\n",bts->nr);
+	LOGP(DPCU, LOGL_INFO, "Sending info for BTS %d\n", bts->nr);
 
 	rlcc = &bts->gprs.cell.rlc_cfg;
 
@@ -136,9 +186,7 @@ static int pcu_tx_info_ind(struct gsm_bts *bts)
 	info_ind = &pcu_prim->u.info_ind;
 	info_ind->version = PCU_IF_VERSION;
 	info_ind->flags |= PCU_IF_FLAG_ACTIVE;
-
-	if (pcu_direct)
-		info_ind->flags |= PCU_IF_FLAG_SYSMO;
+	info_ind->flags |= PCU_IF_FLAG_SYSMO;
 
 	/* RAI */
 	info_ind->mcc = bts->network->plmn.mcc;
@@ -153,6 +201,7 @@ static int pcu_tx_info_ind(struct gsm_bts *bts)
 	memcpy(info_ind->cell_timer, bts->gprs.cell.timer, 11);
 
 	/* cell attributes */
+	info_ind->bsic = bts->bsic;
 	info_ind->cell_id = bts->cell_identity;
 	info_ind->repeat_time = rlcc->paging.repeat_time;
 	info_ind->repeat_count = rlcc->paging.repeat_count;
@@ -211,14 +260,14 @@ static int pcu_tx_info_ind(struct gsm_bts *bts)
 		case AF_INET:
 			info_ind->address_type[i] = PCU_IF_ADDR_TYPE_IPV4;
 			info_ind->remote_ip[i].v4 = nsvc->remote.u.sin.sin_addr;
-			info_ind->remote_port[i] = nsvc->remote.u.sin.sin_port;
+			info_ind->remote_port[i] = ntohs(nsvc->remote.u.sin.sin_port);
 			break;
 		case AF_INET6:
 			info_ind->address_type[i] = PCU_IF_ADDR_TYPE_IPV6;
 			memcpy(&info_ind->remote_ip[i].v6,
 			       &nsvc->remote.u.sin6.sin6_addr,
 			       sizeof(struct in6_addr));
-			info_ind->remote_port[i] = nsvc->remote.u.sin6.sin6_port;
+			info_ind->remote_port[i] = ntohs(nsvc->remote.u.sin6.sin6_port);
 			break;
 		default:
 			info_ind->address_type[i] = PCU_IF_ADDR_TYPE_UNSPEC;
@@ -230,30 +279,13 @@ static int pcu_tx_info_ind(struct gsm_bts *bts)
 		trx = gsm_bts_trx_num(bts, i);
 		if (!trx)
 			continue;
-		info_ind->trx[i].hlayer1 = 0x2342;
-		info_ind->trx[i].pdch_mask = 0;
-		info_ind->trx[i].arfcn = trx->arfcn;
-		for (tn = 0; tn < ARRAY_SIZE(trx->ts); tn++) {
-			ts = &trx->ts[tn];
-			if (ts->mo.nm_state.operational != NM_OPSTATE_ENABLED ||
-			    ts->pchan_is != GSM_PCHAN_PDCH)
-				continue;
-
-			info_ind->trx[i].pdch_mask |= (1 << tn);
-			info_ind->trx[i].ts[tn].tsc =
-					(ts->tsc >= 0) ? ts->tsc : bts->bsic & 7;
-
-			if (ts->hopping.enabled)
-				info_ind_fill_fhp(&info_ind->trx[i].ts[tn], ts);
-
-			LOGP(DPCU, LOGL_INFO, "trx=%d ts=%d: PDCH is available "
-			     "(tsc=%u ", trx->nr, ts->nr, info_ind->trx[i].ts[tn].tsc);
-			if (ts->hopping.enabled)
-				LOGPC(DPCU, LOGL_INFO, "hopping=yes hsn=%u maio=%u ma_bit_len=%u)\n",
-				      ts->hopping.hsn, ts->hopping.maio, trx->ts[tn].hopping.ma.data_len - trx->ts[tn].hopping.ma.cur_bit);
-			else
-				LOGPC(DPCU, LOGL_INFO, "hopping=no arfcn=%u)\n", trx->arfcn);
+		if (trx->nr >= ARRAY_SIZE(info_ind->trx)) {
+			LOG_TRX(trx, DPCU, LOGL_NOTICE, "PCU interface (version %u) "
+				"cannot handle more than %zu transceivers => skipped\n",
+				PCU_IF_VERSION, ARRAY_SIZE(info_ind->trx));
+			break;
 		}
+		info_ind_fill_trx(&info_ind->trx[trx->nr], trx);
 	}
 
 	return pcu_sock_send(bts, msg);
@@ -415,8 +447,8 @@ static int pcu_rx_data_req(struct gsm_bts *bts, uint8_t msg_type,
 		/* DT = direct tlli. A tlli is prefixed */
 
 		if (data_req->len < 5) {
-			LOGP(DPCU, LOGL_ERROR, "Received PCU data request with "
-					"invalid/small length %d\n", data_req->len);
+			LOGP(DPCU, LOGL_ERROR, "Received PCU data request with invalid/small length %d\n",
+			     data_req->len);
 			break;
 		}
 		memcpy(&tlli, data_req->data, 4);
@@ -448,8 +480,17 @@ static int pcu_rx_data_req(struct gsm_bts *bts, uint8_t msg_type,
 	return rc;
 }
 
+#define CHECK_IF_MSG_SIZE(prim_len, prim_msg) \
+	do { \
+		size_t _len = PCUIF_HDR_SIZE + sizeof(prim_msg); \
+		if (prim_len < _len) { \
+			LOGP(DPCU, LOGL_ERROR, "Received %zu bytes on PCU Socket, but primitive %s " \
+			     "size is %zu, discarding\n", prim_len, #prim_msg, _len); \
+			return -EINVAL; \
+		} \
+	} while (0)
 static int pcu_rx(struct gsm_network *net, uint8_t msg_type,
-	struct gsm_pcu_if *pcu_prim)
+	struct gsm_pcu_if *pcu_prim, size_t prim_len)
 {
 	int rc = 0;
 	struct gsm_bts *bts;
@@ -460,6 +501,7 @@ static int pcu_rx(struct gsm_network *net, uint8_t msg_type,
 	switch (msg_type) {
 	case PCU_IF_MSG_DATA_REQ:
 	case PCU_IF_MSG_PAG_REQ:
+		CHECK_IF_MSG_SIZE(prim_len, pcu_prim->u.data_req);
 		rc = pcu_rx_data_req(bts, msg_type, &pcu_prim->u.data_req);
 		break;
 	default:
@@ -556,7 +598,7 @@ static int pcu_sock_read(struct osmo_fd *bfd)
 	struct msgb *msg;
 	int rc;
 
-	msg = msgb_alloc(sizeof(*pcu_prim), "pcu_sock_rx");
+	msg = msgb_alloc(sizeof(*pcu_prim) + 1000, "pcu_sock_rx");
 	if (!msg)
 		return -ENOMEM;
 
@@ -567,12 +609,21 @@ static int pcu_sock_read(struct osmo_fd *bfd)
 		goto close;
 
 	if (rc < 0) {
-		if (errno == EAGAIN)
+		if (errno == EAGAIN) {
+			msgb_free(msg);
 			return 0;
+		}
 		goto close;
 	}
 
-	rc = pcu_rx(state->net, pcu_prim->msg_type, pcu_prim);
+	if (rc < PCUIF_HDR_SIZE) {
+		LOGP(DPCU, LOGL_ERROR, "Received %d bytes on PCU Socket, but primitive hdr size "
+		     "is %zu, discarding\n", rc, PCUIF_HDR_SIZE);
+		msgb_free(msg);
+		return 0;
+	}
+
+	rc = pcu_rx(state->net, pcu_prim->msg_type, pcu_prim, rc);
 
 	/* as we always synchronously process the message in pcu_rx() and
 	 * its callbacks, we can free the message here. */
