@@ -52,7 +52,7 @@ struct gsm0808_cell_id_list2;
  * array. DCS1800 and PCS1900 can not be used at the same time so conserve
  * memory and do the below.
  */
-static int band_compatible(const struct gsm_bts *bts, int arfcn)
+int band_compatible(const struct gsm_bts *bts, int arfcn)
 {
 	enum gsm_band band;
 
@@ -504,20 +504,29 @@ static int bitvec2freq_list(uint8_t *chan_list, const struct bitvec *bv,
 	bool pgsm = false;
 	memset(chan_list, 0, 16);
 
-	if (bts->band == GSM_BAND_900
-	 && bts->c0->arfcn >= 1 && bts->c0->arfcn <= 124)
+	/* According to 3GPP TS 44.018, section 10.5.2.1b.2, only ARFCN values
+	 * in range 1..124 can be encoded using the 'bit map 0' format. */
+	if (bts->band == GSM_BAND_900)
 		pgsm = true;
+	/* Check presence of E-GSM ARFCN 0 */
+	if (pgsm && bitvec_get_bit_pos(bv, 0) == ONE)
+		pgsm = false;
+	/* Check presence of E-GSM ARFCNs 975..1023 */
+	for (i = 975; pgsm && i <= 1023; i++) {
+		if (bitvec_get_bit_pos(bv, i) == ONE)
+			pgsm = false;
+	}
+
 	/* P-GSM-only handsets only support 'bit map 0 format' */
 	if (!bis && !ter && pgsm) {
 		chan_list[0] = 0;
 
-		for (i = 0; i < bv->data_len*8; i++) {
-			if (i >= 1 && i <= 124
-			 && bitvec_get_bit_pos(bv, i)) {
-				rc = freq_list_bm0_set_arfcn(chan_list, i);
-				if (rc < 0)
-					return rc;
-			}
+		for (i = 1; i <= 124; i++) {
+			if (!bitvec_get_bit_pos(bv, i))
+				continue;
+			rc = freq_list_bm0_set_arfcn(chan_list, i);
+			if (rc < 0)
+				return rc;
 		}
 		return 0;
 	}
@@ -553,7 +562,7 @@ static int bitvec2freq_list(uint8_t *chan_list, const struct bitvec *bv,
 			max = i;
 	}
 
-	if (max == -1) {
+	if (arfcns == 0) {
 		/* Empty set, use 'bit map 0 format' */
 		chan_list[0] = 0;
 		return 0;
@@ -709,7 +718,7 @@ static int list_arfcn(uint8_t *chan_list, uint8_t mask, char *text)
 	struct gsm_sysinfo_freq freq[1024];
 
 	memset(freq, 0, sizeof(freq));
-	gsm48_decode_freq_list(freq, chan_list, 16, 0xce, 1);
+	gsm48_decode_freq_list(freq, chan_list, 16, mask, 1);
 	for (i = 0; i < 1024; i++) {
 		if (freq[i].mask) {
 			if (!n)
@@ -954,6 +963,10 @@ static int generate_si3(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 	si3->header.skip_indicator = 0;
 	si3->header.system_information = GSM48_MT_RR_SYSINFO_3;
 
+	/* The value in bts->si_common.chan_desc may get out of sync with the actual value
+	 * in net->T_defs (e.g. after changing it via the VTY), so we need to sync it here. */
+	bts->si_common.chan_desc.t3212 = osmo_tdef_get(bts->network->T_defs, 3212, OSMO_TDEF_CUSTOM, 0);
+
 	si3->cell_identity = htons(bts->cell_identity);
 	gsm48_generate_lai2(&si3->lai, bts_lai(bts));
 	si3->control_channel_desc = bts->si_common.chan_desc;
@@ -1057,14 +1070,9 @@ static int generate_si5(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 	memset(output, GSM_MACBLOCK_PADDING, GSM_MACBLOCK_LEN);
 
 	/* ip.access nanoBTS needs l2_plen!! */
-	switch (bts->type) {
-	case GSM_BTS_TYPE_NANOBTS:
-	case GSM_BTS_TYPE_OSMOBTS:
+	if (is_ipaccess_bts(bts)) {
 		*output++ = GSM48_LEN2PLEN(l2_plen);
 		l2_plen++;
-		break;
-	default:
-		break;
 	}
 
 	si5 = (struct gsm48_system_information_type_5 *) output;
@@ -1093,14 +1101,9 @@ static int generate_si5bis(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 	memset(output, GSM_MACBLOCK_PADDING, GSM_MACBLOCK_LEN);
 
 	/* ip.access nanoBTS needs l2_plen!! */
-	switch (bts->type) {
-	case GSM_BTS_TYPE_NANOBTS:
-	case GSM_BTS_TYPE_OSMOBTS:
+	if (is_ipaccess_bts(bts)) {
 		*output++ = GSM48_LEN2PLEN(l2_plen);
 		l2_plen++;
-		break;
-	default:
-		break;
 	}
 
 	si5b = (struct gsm48_system_information_type_5bis *) output;
@@ -1137,14 +1140,9 @@ static int generate_si5ter(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 	memset(output, GSM_MACBLOCK_PADDING, GSM_MACBLOCK_LEN);
 
 	/* ip.access nanoBTS needs l2_plen!! */
-	switch (bts->type) {
-	case GSM_BTS_TYPE_NANOBTS:
-	case GSM_BTS_TYPE_OSMOBTS:
+	if (is_ipaccess_bts(bts)) {
 		*output++ = GSM48_LEN2PLEN(l2_plen);
 		l2_plen++;
-		break;
-	default:
-		break;
 	}
 
 	si5t = (struct gsm48_system_information_type_5ter *) output;
@@ -1177,14 +1175,9 @@ static int generate_si6(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 	memset(&si6_ro_info, 0, sizeof(si6_ro_info));
 
 	/* ip.access nanoBTS needs l2_plen!! */
-	switch (bts->type) {
-	case GSM_BTS_TYPE_NANOBTS:
-	case GSM_BTS_TYPE_OSMOBTS:
+	if (is_ipaccess_bts(bts)) {
 		*output++ = GSM48_LEN2PLEN(l2_plen);
 		l2_plen++;
-		break;
-	default:
-		break;
 	}
 
 	si6 = (struct gsm48_system_information_type_6 *) output;
@@ -1201,50 +1194,17 @@ static int generate_si6(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 	gsm48_set_dtx(&si6->cell_options, bts->dtxu, bts->dtxu, false);
 
 	/* SI6 Rest Octets: 10.5.2.35a: PCH / NCH info, VBS/VGCS options */
-	si6_ro_info.band_indicator_1900 = is_dcs_net(bts);
+	si6_ro_info.band_indicator_1900 = !is_dcs_net(bts);
 	rc = osmo_gsm48_rest_octets_si6_encode(si6->rest_octets, &si6_ro_info);
 
 	return l2_plen + rc;
 }
 
-static struct osmo_gsm48_si13_info si13_default = {
-	.cell_opts = {
-		.nmo 		= GPRS_NMO_II,
-		.t3168		= 2000,
-		.t3192		= 1500,
-		.drx_timer_max	= 3,
-		.bs_cv_max	= 15,
-		.ctrl_ack_type_use_block = true,
-		.ext_info_present = 0,
-		.ext_info = {
-			.egprs_supported = 0,		/* overridden in gsm_generate_si() */
-			.use_egprs_p_ch_req = 0,	/* overridden in generate_si13() */
-			.bep_period = 5,
-			.pfc_supported = 0,
-			.dtm_supported = 0,
-			.bss_paging_coordination = 0,
-			.ccn_active = false,
-		},
-	},
-	.pwr_ctrl_pars = {
-		.alpha		= 0,	/* a = 0.0 */
-		.t_avg_w	= 16,
-		.t_avg_t	= 16,
-		.pc_meas_chan	= 0, 	/* downling measured on CCCH */
-		.n_avg_i	= 8,
-	},
-	.bcch_change_mark	= 1,
-	.si_change_field	= 0,
-	.rac		= 0,	/* needs to be patched */
-	.spgc_ccch_sup 	= 0,
-	.net_ctrl_ord	= 0,
-	.prio_acc_thr	= 6,
-};
-
 static int generate_si13(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 {
 	struct gsm48_system_information_type_13 *si13 =
 		(struct gsm48_system_information_type_13 *) GSM_BTS_SI(bts, t);
+	struct osmo_gsm48_si13_info si13_info;
 	int ret;
 
 	memset(si13, GSM_MACBLOCK_PADDING, GSM_MACBLOCK_LEN);
@@ -1253,33 +1213,66 @@ static int generate_si13(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 	si13->header.skip_indicator = 0;
 	si13->header.system_information = GSM48_MT_RR_SYSINFO_13;
 
-	si13_default.rac = bts->gprs.rac;
-	si13_default.net_ctrl_ord = bts->gprs.net_ctrl_ord;
+	si13_info = (struct osmo_gsm48_si13_info){
+		.cell_opts = {
+			.nmo		= GPRS_NMO_II,
+			.t3168		= 2000,
+			.t3192		= 1500,
+			.drx_timer_max	= 3,
+			.bs_cv_max	= 15,
+			.ctrl_ack_type_use_block = bts->gprs.ctrl_ack_type_use_block,
+			.ext_info_present = true,
+			.ext_info = {
+				.egprs_supported = 0,		/* overridden below */
+				.use_egprs_p_ch_req = 0,	/* overridden below */
+				.bep_period = 5,
+				.pfc_supported = 0,
+				.dtm_supported = 0,
+				.bss_paging_coordination = 0,	/* overridden below */
+				.ccn_active = false,		/* overridden below */
+			},
+		},
+		.pwr_ctrl_pars = {
+			.alpha		= bts->gprs.pwr_ctrl.alpha,	/* a = 0.0 */
+			.t_avg_w	= 16,
+			.t_avg_t	= 16,
+			.pc_meas_chan	= 0,	/* downling measured on CCCH */
+			.n_avg_i	= 8,
+		},
+		.bcch_change_mark	= bts->bcch_change_mark, /* Information about the other SIs */
+		.si_change_field	= 0,
+		.rac		= bts->gprs.rac,
+		.spgc_ccch_sup	= 0,
+		.net_ctrl_ord	= bts->gprs.net_ctrl_ord,
+		.prio_acc_thr	= 6,
+	};
 
-	si13_default.cell_opts.ctrl_ack_type_use_block =
-		bts->gprs.ctrl_ack_type_use_block;
-
-	/* Information about the other SIs */
-	si13_default.bcch_change_mark = bts->bcch_change_mark;
-
-	/* Whether EGPRS capable MSs shall use EGPRS PACKET CHANNEL REQUEST */
-	if (bts->gprs.egprs_pkt_chan_request)
-		si13_default.cell_opts.ext_info.use_egprs_p_ch_req = 1;
-	else
-		si13_default.cell_opts.ext_info.use_egprs_p_ch_req = 0;
+	switch (bts->gprs.mode) {
+	case BTS_GPRS_EGPRS:
+		si13_info.cell_opts.ext_info.egprs_supported = 1;
+		/* Whether EGPRS capable MSs shall use EGPRS PACKET CHANNEL REQUEST */
+		if (bts->gprs.egprs_pkt_chan_request)
+			si13_info.cell_opts.ext_info.use_egprs_p_ch_req = 1;
+		else
+			si13_info.cell_opts.ext_info.use_egprs_p_ch_req = 0;
+		break;
+	case BTS_GPRS_GPRS:
+	case BTS_GPRS_NONE:
+		si13_info.cell_opts.ext_info.egprs_supported = 0;
+		si13_info.cell_opts.ext_info.use_egprs_p_ch_req = 0;
+		break;
+	}
 
 	if (osmo_bts_has_feature(&bts->features, BTS_FEAT_PAGING_COORDINATION))
-		si13_default.cell_opts.ext_info.bss_paging_coordination = 1;
+		si13_info.cell_opts.ext_info.bss_paging_coordination = 1;
 	else
-		si13_default.cell_opts.ext_info.bss_paging_coordination = 0;
+		si13_info.cell_opts.ext_info.bss_paging_coordination = 0;
 
-	si13_default.cell_opts.ext_info.ccn_active = bts->gprs.ccn.forced_vty ?
-						     bts->gprs.ccn.active :
-						     osmo_bts_has_feature(&bts->model->features,
-									  BTS_FEAT_CCN);
-	si13_default.pwr_ctrl_pars.alpha = bts->gprs.pwr_ctrl.alpha;
+	si13_info.cell_opts.ext_info.ccn_active = bts->gprs.ccn.forced_vty ?
+						  bts->gprs.ccn.active :
+						  osmo_bts_has_feature(&bts->features, BTS_FEAT_CCN);
 
-	ret = osmo_gsm48_rest_octets_si13_encode(si13->rest_octets, &si13_default);
+	ret = osmo_gsm48_rest_octets_si13_encode(si13->rest_octets, &si13_info);
 	if (ret < 0)
 		return ret;
 
@@ -1313,10 +1306,7 @@ int gsm_generate_si(struct gsm_bts *bts, enum osmo_sysinfo_type si_type)
 
 	switch (bts->gprs.mode) {
 	case BTS_GPRS_EGPRS:
-		si13_default.cell_opts.ext_info.egprs_supported = 1;
-		/* fallthrough */
 	case BTS_GPRS_GPRS:
-		si13_default.cell_opts.ext_info_present = 1;
 		si_info.gprs_ind.present = 1;
 		break;
 	case BTS_GPRS_NONE:
