@@ -20,6 +20,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <inttypes.h>
 
 #include <assert.h>
 
@@ -52,6 +53,7 @@
 #include <osmocom/bsc/bts.h>
 #include <osmocom/bsc/paging.h>
 #include <osmocom/bsc/vty.h>
+#include <osmocom/bsc/lchan.h>
 #include <osmocom/mgcp_client/mgcp_client_pool.h>
 
 #include "../../bscconfig.h"
@@ -250,13 +252,13 @@ static struct gsm_bts *_create_bts(int num_trx, const char * const *ts_args, int
 
 	fprintf(stderr, "- Creating BTS %d, %d TRX\n", bsc_gsmnet->num_bts, num_trx);
 
-	bts = bsc_bts_alloc_register(bsc_gsmnet, GSM_BTS_TYPE_UNKNOWN, 0x3f);
+	bts = bsc_bts_alloc_register(bsc_gsmnet, GSM_BTS_TYPE_UNKNOWN, HARDCODED_BSIC);
 	if (!bts) {
 		fprintf(stderr, "No resource for bts1\n");
 		return NULL;
 	}
 
-	bts->location_area_code = 23;
+	bts->location_area_code = 0x0017;
 	bts->cell_identity = ci++;
 	bts->c0->arfcn = arfcn++;
 
@@ -479,6 +481,8 @@ struct gsm_lchan *lchan_act(struct gsm_lchan *lchan, int full_rate, const char *
 		.len = 5,
 	};
 
+	chan_counts_ts_update(lchan->ts);
+
 	return lchan;
 }
 
@@ -486,7 +490,8 @@ struct gsm_lchan *create_lchan(struct gsm_bts *bts, int full_rate, const char *c
 {
 	struct gsm_lchan *lchan;
 
-	lchan = lchan_select_by_type(bts, (full_rate) ? GSM_LCHAN_TCH_F : GSM_LCHAN_TCH_H);
+	lchan = lchan_select_by_type(bts, (full_rate) ? GSM_LCHAN_TCH_F : GSM_LCHAN_TCH_H,
+				     SELECT_FOR_HANDOVER, NULL);
 	if (!lchan) {
 		fprintf(stderr, "No resource for lchan\n");
 		exit(EXIT_FAILURE);
@@ -522,6 +527,7 @@ static void ts_clear(struct gsm_bts_trx_ts *ts)
 			continue;
 		lchan_clear(lchan);
 	}
+	chan_counts_ts_update(ts);
 }
 
 bool _set_ts_use(struct gsm_bts *bts, struct gsm_bts_trx *trx, const char * const *ts_use)
@@ -1428,6 +1434,59 @@ DEFUN(codec_h, codec_h_cmd,
 	return CMD_SUCCESS;
 }
 
+DEFUN(set_arfcn, set_arfcn_cmd,
+	"set-arfcn trx <0-255> <0-255> <0-1023>",
+	"Set the ARFCN for a BTS' TRX\n"
+	"Indicate a BTS and TRX\n" "BTS nr\n" "TRX nr\n"
+	"Absolute Radio Frequency Channel Number\n")
+{
+	enum gsm_band unused;
+	struct gsm_bts *bts = bts_by_num_str(argv[0]);
+	struct gsm_bts_trx *trx = trx_by_num_str(bts, argv[1]);
+	int arfcn = atoi(argv[2]);
+	VTY_ECHO();
+
+	if (gsm_arfcn2band_rc(arfcn, &unused) < 0) {
+		vty_out(vty, "%% Invalid arfcn %" PRIu16 " detected%s", arfcn, VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	trx->arfcn = arfcn;
+
+	if (generate_cell_chan_alloc(trx->bts) != 0) {
+		vty_out(vty, "%% Failed to re-generate Cell Allocation%s", VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	for (int i = 0; i < bsc_gsmnet->num_bts; i++) {
+		if (gsm_generate_si(gsm_bts_num(bsc_gsmnet, i), SYSINFO_TYPE_2) <= 0)
+			fprintf(stderr, "Error generating SI2\n");
+	}
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(set_band, set_band_cmd,
+	"set-band bts <0-255> BAND",
+	"Set the frequency band for a BTS\n"
+	"Indicate a BTS\n" "BTS nr\n"
+	"Frequency band\n")
+{
+	struct gsm_bts *bts = bts_by_num_str(argv[0]);
+	int band = gsm_band_parse(argv[1]);
+	VTY_ECHO();
+
+	if (band < 0) {
+		vty_out(vty, "%% BAND %d is not a valid GSM band%s",
+			band, VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	bts->band = band;
+
+	return CMD_SUCCESS;
+}
+
 DEFUN(set_ts_use, set_ts_use_cmd,
 	"set-ts-use trx <0-255> <0-255> states" TS_USE TS_USE TS_USE TS_USE TS_USE TS_USE TS_USE TS_USE,
 	"Put timeslots of a BTS' TRX into a specific state\n"
@@ -1485,6 +1544,8 @@ static void ho_test_vty_init()
 	install_element(CONFIG_NODE, &expect_ts_use_cmd);
 	install_element(CONFIG_NODE, &codec_f_cmd);
 	install_element(CONFIG_NODE, &codec_h_cmd);
+	install_element(CONFIG_NODE, &set_arfcn_cmd);
+	install_element(CONFIG_NODE, &set_band_cmd);
 	install_element(CONFIG_NODE, &set_ts_use_cmd);
 	install_element(CONFIG_NODE, &wait_cmd);
 }
@@ -1685,7 +1746,7 @@ void trau_send_frame() {}
 /* Stub */
 int osmo_bsc_sigtran_open_conn(struct gsm_subscriber_connection *conn, struct msgb *msg) { return 0; }
 void bsc_sapi_n_reject(struct gsm_subscriber_connection *conn, uint8_t dlci, enum gsm0808_cause cause) {}
-void bsc_cipher_mode_compl(struct gsm_subscriber_connection *conn, struct msgb *msg, uint8_t chosen_encr) {}
+void bsc_cipher_mode_compl(struct gsm_subscriber_connection *conn, struct msgb *msg, uint8_t chosen_a5_n) {}
 int bsc_compl_l3(struct gsm_lchan *lchan, struct msgb *msg, uint16_t chosen_channel)
 { return 0; }
 void bsc_dtap(struct gsm_subscriber_connection *conn, uint8_t link_id, struct msgb *msg) {}
@@ -1710,3 +1771,7 @@ const struct mgcp_conn_peer *osmo_mgcpc_ep_ci_get_rtp_info(const struct osmo_mgc
 	};
 	return &ret;
 }
+struct mgcp_client *osmo_mgcpc_ep_client(const struct osmo_mgcpc_ep *ep)
+{
+	return NULL;
+}
